@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -316,8 +316,8 @@ int main(int argc, const char **argv)
 
     // Initialize simulator frames
     sim.set_fMo(wMo);                   // Position of the object in the world reference frame
-    sim.setCameraPositionRelObj(cMo);   // initial position of the camera
-    sim.setDesiredCameraPosition(cdMo); // desired position of the camera
+    sim.setCameraPositionRelObj(cMo);   // Initial position of the object in the camera frame
+    sim.setDesiredCameraPosition(cdMo); // Desired position of the object in the camera frame
 
     // Set the External camera position
     vpHomogeneousMatrix camMf(vpHomogeneousMatrix(0.0, 0, 3.5, vpMath::rad(0), vpMath::rad(30), 0));
@@ -445,17 +445,21 @@ int main(int argc, const char **argv)
     task.print();
     task.kill();
 
-    return 0;
-  } catch (vpException &e) {
+    return EXIT_SUCCESS;
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e << std::endl;
-    return 1;
+    return EXIT_FAILURE;
   }
 }
 #else
 int main()
 {
-  vpERROR_TRACE("You do not have X11, OpenCV, GDI, D3D9 or GTK display "
-                "functionalities...");
+  std::cout << "You do not have X11, or GDI (Graphical Device Interface), or GTK functionalities to display images..." << std::endl;
+  std::cout << "Tip if you are on a unix-like system:" << std::endl;
+  std::cout << "- Install X11, configure again ViSP using cmake and build again this example" << std::endl;
+  std::cout << "Tip if you are on a windows-like system:" << std::endl;
+  std::cout << "- Install GDI, configure again ViSP using cmake and build again this example" << std::endl;
+  return EXIT_SUCCESS;
 }
 
 #endif
