@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -255,7 +255,7 @@ static void *mainLoop(void *_simu)
   task.setServo(vpServo::EYEINHAND_L_cVe_eJe);
   task.setInteractionMatrixType(vpServo::CURRENT);
 
-  // Set the position of the camera in the end-effector frame
+  // Set the position of the end-effector frame in the camera frame as identity
   vpHomogeneousMatrix cMe;
   vpVelocityTwistMatrix cVe(cMe);
   task.set_cVe(cVe);
@@ -392,13 +392,19 @@ int main(int argc, const char **argv)
 
       simu.mainLoop();
     }
-    return 0;
-  } catch (vpException &e) {
+    return EXIT_SUCCESS;
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e << std::endl;
-    return 1;
+    return EXIT_FAILURE;
   }
 }
 
 #else
-int main() { vpTRACE("You should install Coin3D and SoQT or SoWin or SoXt"); }
+int main()
+{
+  std::cout << "You do not have Coin3D and SoQT or SoWin or SoXt functionalities enabled..." << std::endl;
+  std::cout << "Tip:" << std::endl;
+  std::cout << "- Install Coin3D and SoQT or SoWin or SoXt, configure ViSP again using cmake and build again this example" << std::endl;
+  return EXIT_SUCCESS;
+}
 #endif
