@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,6 +45,7 @@
 
 */
 
+#include <iostream>
 #include <visp3/core/vpConfig.h>
 #include <visp3/core/vpDebug.h>
 
@@ -426,12 +427,18 @@ int main(int argc, const char **argv)
         std::cout << "Bye" << std::endl;
       }
     }
-    return 0;
-  } catch (vpException &e) {
+    return EXIT_SUCCESS;
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e << std::endl;
-    return 1;
+    return EXIT_FAILURE;
   }
 }
 #else
-int main() { vpERROR_TRACE("Direct 3D library is not available..."); }
+int main()
+{
+  std::cout << "You do not have Direct 3D functionalities to display images..." << std::endl;
+  std::cout << "Tip if you are on a windows system:" << std::endl;
+  std::cout << "- Install Direct 3D, configure again ViSP using cmake and build again this example" << std::endl;
+  return EXIT_SUCCESS;
+}
 #endif
