@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -221,13 +221,13 @@ int main(int argc, const char **argv)
 
     if (inliers == inliers_ground_truth) {
       std::cout << "Ransac estimation succeed" << std::endl;
-      return 0;
+      return EXIT_SUCCESS;
     } else {
       std::cout << "Ransac estimation fails" << std::endl;
-      return 1;
+      return EXIT_FAILURE;
     }
-  } catch (vpException &e) {
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e << std::endl;
-    return 1;
+    return EXIT_FAILURE;
   }
 }
