@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -84,7 +84,7 @@
 static void *mainLoop(void *_simu)
 {
   // pointer copy of the vpSimulator instance
-  vpSimulator *simu = static_cast<vpSimulator *> _simu;
+  vpSimulator *simu = static_cast<vpSimulator *> (_simu);
 
   // Simulation initialization
   simu->initMainApplication();
@@ -178,7 +178,7 @@ static void *mainLoop(void *_simu)
   task.setServo(vpServo::EYEINHAND_L_cVe_eJe);
   task.setInteractionMatrixType(vpServo::DESIRED);
 
-  // Set the position of the camera in the end-effector frame
+  // Set the position of the end-effector frame in the camera frame as identity
   vpHomogeneousMatrix cMe;
   vpVelocityTwistMatrix cVe(cMe);
   task.set_cVe(cVe);
@@ -279,13 +279,21 @@ int main()
     simu.initApplication(&mainLoop);
     // Run the simulator
     simu.mainLoop();
-    return 0;
-  } catch (vpException &e) {
+    return EXIT_SUCCESS;
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e << std::endl;
-    return 1;
+    return EXIT_FAILURE;
   }
 }
 
 #else
-int main() { vpTRACE("You should install Coin3D and SoQT or SoWin or SoXt"); }
+int main()
+{
+  std::cout << "You do not have X11, GTK, or OpenCV, or GDI (Graphical Device Interface) functionalities to display images..." << std::endl;
+  std::cout << "Tip if you are on a unix-like system:" << std::endl;
+  std::cout << "- Install X11, configure again ViSP using cmake and build again this example" << std::endl;
+  std::cout << "Tip if you are on a windows-like system:" << std::endl;
+  std::cout << "- Install GDI, configure again ViSP using cmake and build again this example" << std::endl;
+  return EXIT_SUCCESS;
+}
 #endif
