@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -145,19 +145,19 @@ OPTIONS:                                               Default\n\
 */
 bool getOptions(int argc, const char **argv, std::string &ipath, std::string &ppath, bool &click_allowed)
 {
-  const char *optarg;
+  const char *optarg_;
   int c;
-  while ((c = vpParseArgv::parse(argc, argv, GETOPTARGS, &optarg)) > 1) {
+  while ((c = vpParseArgv::parse(argc, argv, GETOPTARGS, &optarg_)) > 1) {
 
     switch (c) {
     case 'c':
       click_allowed = false;
       break;
     case 'i':
-      ipath = optarg;
+      ipath = optarg_;
       break;
     case 'p':
-      ppath = optarg;
+      ppath = optarg_;
       break;
     case 'h':
       usage(argv[0], NULL, ipath, ppath);
@@ -165,7 +165,7 @@ bool getOptions(int argc, const char **argv, std::string &ipath, std::string &pp
       break;
 
     default:
-      usage(argv[0], optarg, ipath, ppath);
+      usage(argv[0], optarg_, ipath, ppath);
       return false;
       break;
     }
@@ -175,7 +175,7 @@ bool getOptions(int argc, const char **argv, std::string &ipath, std::string &pp
     // standalone param or error
     usage(argv[0], NULL, ipath, ppath);
     std::cerr << "ERROR: " << std::endl;
-    std::cerr << "  Bad argument " << optarg << std::endl << std::endl;
+    std::cerr << "  Bad argument " << optarg_ << std::endl << std::endl;
     return false;
   }
 
@@ -188,9 +188,9 @@ class vpAROgreExample : public vpAROgre
 {
 public:
   // The constructor doesn't change here
-  vpAROgreExample(const vpCameraParameters &mcam = vpCameraParameters(), unsigned int width = 640,
+  vpAROgreExample(const vpCameraParameters &cam = vpCameraParameters(), unsigned int width = 640,
                   unsigned int height = 480, const char *resourcePath = NULL)
-    : vpAROgre(mcam, width, height)
+    : vpAROgre(cam, width, height)
   {
     // Direction vectors
     if (resourcePath)
@@ -440,7 +440,7 @@ void computeInitialPose(vpCameraParameters *mcam, vpImage<unsigned char> &I, vpP
         vpDisplay::flush(I);
       }
     }
-  } catch (vpException &e) {
+  } catch (const vpException &e) {
     vpERROR_TRACE("Error while tracking dots");
     vpCTRACE << e;
     return;
@@ -699,23 +699,32 @@ int main(int argc, const char **argv)
     // Close the grabber
     grabber.close();
 
-    return 0;
-  } catch (vpException &e) {
+    return EXIT_SUCCESS;
+  } catch (const vpException &e) {
     std::cout << "Catch a ViSP exception: " << e << std::endl;
-    return 1;
+    return EXIT_FAILURE;
   } catch (Ogre::Exception &e) {
     std::cout << "Catch an Ogre exception: " << e.getDescription() << std::endl;
-    return 1;
+    return EXIT_FAILURE;
   } catch (...) {
     std::cout << "Catch an exception " << std::endl;
-    return 1;
+    return EXIT_FAILURE;
   }
 }
 #else // VISP_HAVE_OGRE && VISP_HAVE_DISPLAY
 int main()
 {
-  std::cout << "You should install Ogre3D or a display (GTK or OpenCV...) to "
-               "run this example..."
-            << std::endl;
+#if (!(defined(VISP_HAVE_X11) || defined(VISP_HAVE_GTK) || defined(VISP_HAVE_GDI)))
+  std::cout << "You do not have X11, or GTK, or GDI (Graphical Device Interface) functionalities to display images..." << std::endl;
+  std::cout << "Tip if you are on a unix-like system:" << std::endl;
+  std::cout << "- Install X11, configure again ViSP using cmake and build again this example" << std::endl;
+  std::cout << "Tip if you are on a windows-like system:" << std::endl;
+  std::cout << "- Install GDI, configure again ViSP using cmake and build again this example" << std::endl;
+#else
+  std::cout << "You do not have Ogre functionalities" << std::endl;
+  std::cout << "Tip:" << std::endl;
+  std::cout << "- Install Ogre3D, configure again ViSP using cmake and build again this example" << std::endl;
+#endif
+  return EXIT_SUCCESS;
 }
 #endif
