@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -117,7 +117,7 @@ OPTIONS:                                               Default\n\
 
 /*!
 
-Set the program options.
+  Set the program options.
 
   \param argc : Command line number of parameters.
   \param argv : Array of command line parameters.
@@ -171,7 +171,7 @@ int main(int argc, const char **argv)
 
     // Read the command line options
     if (getOptions(argc, argv, opt_click_allowed, opt_display) == false) {
-      exit(-1);
+      return(EXIT_FAILURE);
     }
 
     vpImage<unsigned char> I(512, 512, 0);
@@ -188,26 +188,19 @@ int main(int argc, const char **argv)
 #endif
 
     if (opt_display) {
-      try {
-        // Display size is automatically defined by the image (I) size
-        display.init(I, 100, 100, "Camera view...");
-        // Display the image
-        // The image class has a member that specify a pointer toward
-        // the display that has been initialized in the display declaration
-        // therefore is is no longuer necessary to make a reference to the
-        // display variable.
-        vpDisplay::display(I);
-        vpDisplay::flush(I);
-      } catch (...) {
-        vpERROR_TRACE("Error while displaying the image");
-        exit(-1);
-      }
+      // Display size is automatically defined by the image (I) size
+      display.init(I, 100, 100, "Camera view...");
+      // Display the image
+      // The image class has a member that specify a pointer toward
+      // the display that has been initialized in the display declaration
+      // therefore is is no longuer necessary to make a reference to the
+      // display variable.
+      vpDisplay::display(I);
+      vpDisplay::flush(I);
     }
 
-    double px, py;
-    px = py = 600;
-    double u0, v0;
-    u0 = v0 = 256;
+    double px = 600, py = 600;
+    double u0 = I.getWidth()/2., v0 = I.getHeight() / 2.;
 
     vpCameraParameters cam(px, py, u0, v0);
 
@@ -258,7 +251,7 @@ int main(int argc, const char **argv)
 
       // get the robot position
       robot.getPosition(wMc);
-      // Compute the position of the camera wrt the object frame
+      // Compute the position of the object frame in the camera frame
       cMo = wMc.inverse() * wMo;
 
       // new circle position
@@ -291,13 +284,20 @@ int main(int argc, const char **argv)
       std::cout << "Click in the camera view window to end..." << std::endl;
       vpDisplay::getClick(I);
     }
-    return 0;
-  } catch (vpException &e) {
+    return EXIT_SUCCESS;
+  } catch (const vpException &e) {
     std::cout << "Catch a ViSP exception: " << e << std::endl;
-    return 1;
+    return EXIT_FAILURE;
   }
 }
 #else
-int main() { vpERROR_TRACE("You do not have X11, GTK, GDI or OpenCV display functionalities..."); }
-
+int main()
+{
+  std::cout << "You do not have X11, or GTK, or GDI (Graphical Device Interface) functionalities to display images..." << std::endl;
+  std::cout << "Tip if you are on a unix-like system:" << std::endl;
+  std::cout << "- Install X11, configure again ViSP using cmake and build again this example" << std::endl;
+  std::cout << "Tip if you are on a windows-like system:" << std::endl;
+  std::cout << "- Install GDI, configure again ViSP using cmake and build again this example" << std::endl;
+  return EXIT_SUCCESS;
+}
 #endif
