@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -152,7 +152,7 @@ int main(int argc, const char **argv)
     // In simulation, we have to define the scene frane Ro and the
     // camera frame Rc.
     // The camera location is given by an homogenous matrix cMo that
-    // describes the position of the camera in the scene frame.
+    // describes the position of the scene or object frame in the camera frame.
 
     vpServo task;
 
@@ -297,7 +297,7 @@ int main(int argc, const char **argv)
 
       // get the robot position
       robot.getPosition(wMc);
-      // Compute the position of the camera wrt the object frame
+      // Compute the position of the object frame in the camera frame
       cMo = wMc.inverse() * wMo;
 
       // update the feature
@@ -322,9 +322,9 @@ int main(int argc, const char **argv)
     task.kill();
     // Final camera location
     std::cout << "Final camera location: \n" << cMo << std::endl;
-    return 0;
-  } catch (vpException &e) {
+    return EXIT_SUCCESS;
+  } catch (const vpException &e) {
     std::cout << "Catch a ViSP exception: " << e << std::endl;
-    return 1;
+    return EXIT_SUCCESS;
   }
 }
