@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -189,7 +189,7 @@ int main()
     task.setServo(vpServo::EYEINHAND_L_cVe_eJe);
     task.setInteractionMatrixType(vpServo::DESIRED, vpServo::PSEUDO_INVERSE);
 
-    vpTRACE("Set the position of the camera in the end-effector frame ");
+    vpTRACE("Set the position of the end-effector frame in the camera frame");
     vpHomogeneousMatrix cMe;
     //  robot.get_cMe(cMe) ;
 
@@ -295,17 +295,17 @@ int main()
     vpTRACE("Display task information ");
     task.print();
     task.kill();
-    return 0;
-  } catch (vpException &e) {
+    return EXIT_SUCCESS;
+  } catch (const vpException &e) {
     std::cout << "Catch a ViSP exception: " << e << std::endl;
-    return 1;
+    return EXIT_FAILURE;
   }
 }
 
 #else
 int main()
 {
-  vpERROR_TRACE("You do not have an afma4 robot or a firewire framegrabber "
-                "connected to your computer...");
+  std::cout << "You do not have an afma4 robot connected to your computer..." << std::endl;
+  return EXIT_SUCCESS;
 }
 #endif
