@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@
 
 #include <iostream>
 
+#include <visp3/robot/vpRobotPioneer.h> // Include first to avoid build issues with Status, None, isfinite
 #include <visp3/core/vpConfig.h>
 #include <visp3/core/vpDisplay.h>
 #include <visp3/core/vpImage.h>
@@ -46,7 +47,6 @@
 #include <visp3/gui/vpDisplayGDI.h>
 #include <visp3/gui/vpDisplayX.h>
 #include <visp3/io/vpImageIo.h>
-#include <visp3/robot/vpRobotPioneer.h> // Include before vpDisplayX to avoid build issues
 
 #ifndef VISP_HAVE_PIONEER
 int main()
@@ -54,7 +54,7 @@ int main()
   std::cout << "\nThis example requires Aria 3rd party library. You should "
                "install it.\n"
             << std::endl;
-  return 0;
+  return EXIT_SUCCESS;
 }
 
 #else
@@ -343,10 +343,10 @@ int main(int argc, char **argv)
 
     // exit
     ArLog::log(ArLog::Normal, "simpleMotionCommands: Exiting.");
-    return 0;
-  } catch (vpException &e) {
+    return EXIT_SUCCESS;
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e << std::endl;
-    return 1;
+    return EXIT_FAILURE;
   }
 }
 
