@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -290,7 +290,7 @@ int main(int argc, const char **argv)
     else
       ipath = vpIoTools::createFilePath(env_ipath, "mbt/cube/image%04d.pgm");
 
-#if defined(VISP_HAVE_XML2) && USE_XML
+#if defined(VISP_HAVE_PUGIXML) && USE_XML
     std::string configFile;
     if (!opt_configFile.empty())
       configFile = opt_configFile;
@@ -386,7 +386,7 @@ int main(int argc, const char **argv)
 #endif
 
     if (opt_display) {
-#if (defined VISP_HAVE_DISPLAY)
+#if defined(VISP_HAVE_DISPLAY)
       display1.setDownScalingFactor(vpDisplay::SCALE_AUTO);
       display2.setDownScalingFactor(vpDisplay::SCALE_AUTO);
       display3.setDownScalingFactor(vpDisplay::SCALE_AUTO);
@@ -404,12 +404,13 @@ int main(int argc, const char **argv)
       vpDisplay::flush(I3);
     }
 
+    // Object pointer to check that inheritance is ok
     vpMbTracker *tracker = new vpMbGenericTracker(3, trackerType);
     std::map<std::string, vpHomogeneousMatrix> mapOfCameraPoses;
     std::map<std::string, vpCameraParameters> mapOfCameraParams;
 
 // Initialise the tracker: camera parameters, moving edge and KLT settings
-#if defined(VISP_HAVE_XML2) && USE_XML
+#if defined(VISP_HAVE_PUGIXML) && USE_XML
     // From the xml file
     std::map<std::string, std::string> mapOfConfigFiles;
     mapOfConfigFiles["Camera1"] = configFile;
@@ -574,17 +575,15 @@ int main(int argc, const char **argv)
         }
 
         tracker->resetTracker();
-#if defined(VISP_HAVE_XML2) && USE_XML
+#if defined(VISP_HAVE_PUGIXML) && USE_XML
         dynamic_cast<vpMbGenericTracker *>(tracker)->loadConfigFile(mapOfConfigFiles);
 #else
         // By setting the parameters:
-        vpCameraParameters cam;
         cam.initPersProjWithoutDistortion(547, 542, 338, 234);
         mapOfCameraParams["Camera1"] = cam;
         mapOfCameraParams["Camera2"] = cam;
         mapOfCameraParams["Camera3"] = cam;
 
-        vpMe me;
         me.setMaskSize(5);
         me.setMaskNumber(180);
         me.setRange(7);
@@ -592,13 +591,12 @@ int main(int argc, const char **argv)
         me.setMu1(0.5);
         me.setMu2(0.5);
         me.setSampleStep(4);
-        std::map<std::string, vpMe> mapOfMe;
+
         mapOfMe["Camera1"] = me;
         mapOfMe["Camera2"] = me;
         mapOfMe["Camera3"] = me;
 
 #if defined(VISP_HAVE_MODULE_KLT) && (defined(VISP_HAVE_OPENCV) && (VISP_HAVE_OPENCV_VERSION >= 0x020100))
-        vpKltOpencv klt;
         klt.setMaxFeatures(10000);
         klt.setWindowSize(5);
         klt.setQuality(0.01);
@@ -606,7 +604,7 @@ int main(int argc, const char **argv)
         klt.setHarrisFreeParameter(0.01);
         klt.setBlockSize(3);
         klt.setPyramidLevels(3);
-        std::map<std::string, vpKltOpencv> mapOfKlt;
+
         mapOfKlt["Camera1"] = klt;
         mapOfKlt["Camera2"] = klt;
         mapOfKlt["Camera3"] = klt;
@@ -624,7 +622,6 @@ int main(int argc, const char **argv)
         tracker->setNearClippingDistance(0.01);
         tracker->setFarClippingDistance(0.90);
 
-        std::map<std::string, unsigned int> mapOfClippingFlags;
         dynamic_cast<vpMbGenericTracker *>(tracker)->getClipping(mapOfClippingFlags);
         for (std::map<std::string, unsigned int>::iterator it = mapOfClippingFlags.begin();
              it != mapOfClippingFlags.end(); ++it) {
@@ -681,7 +678,7 @@ int main(int argc, const char **argv)
         }
       }
 
-      if (opt_click_allowed) {
+      if (opt_click_allowed && opt_display) {
         vpDisplay::displayText(I1, 10, 10, "Click to quit", vpColor::red);
         vpMouseButton::vpMouseButtonType button;
         if (vpDisplay::getClick(I1, button, click)) {
@@ -708,9 +705,11 @@ int main(int argc, const char **argv)
         std::cout << "Projection error: " << tracker->getProjectionError() << std::endl << std::endl;
       }
 
-      vpDisplay::flush(I1);
-      vpDisplay::flush(I2);
-      vpDisplay::flush(I3);
+      if (opt_display) {
+        vpDisplay::flush(I1);
+        vpDisplay::flush(I2);
+        vpDisplay::flush(I3);
+      }
     }
 
     std::cout << "Reached last frame: " << reader.getFrameIndex() << std::endl;
@@ -727,13 +726,7 @@ int main(int argc, const char **argv)
     delete tracker;
     tracker = NULL;
 
-#if defined(VISP_HAVE_XML2) && USE_XML
-    // Cleanup memory allocated by xml library used to parse the xml config
-    // file in vpMbGenericTracker::loadConfigFile()
-    vpXmlParser::cleanup();
-#endif
-
-#if defined(VISP_HAVE_COIN3D) && (COIN_MAJOR_VERSION == 2 || COIN_MAJOR_VERSION == 3)
+#if defined(VISP_HAVE_COIN3D) && (COIN_MAJOR_VERSION >= 2)
     // Cleanup memory allocated by Coin library used to load a vrml model in
     // vpMbGenericTracker::loadModel() We clean only if Coin was used.
     if (!cao3DModel)
@@ -741,7 +734,7 @@ int main(int argc, const char **argv)
 #endif
 
     return EXIT_SUCCESS;
-  } catch (vpException &e) {
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e << std::endl;
     return EXIT_FAILURE;
   }
@@ -754,7 +747,7 @@ int main()
   std::cout << "visp_mbt, visp_gui modules and OpenCV are required to run "
                "this example."
             << std::endl;
-  return 0;
+  return EXIT_SUCCESS;
 }
 
 #endif
