@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -229,8 +229,7 @@ Create a display for each different cameras.
     sim.setDesiredViewColor(vpColor::red);
     sim.setCameraColor(vpColor::green);
     /*
-    Set the current and the desired position of the camera relative to the
-    object.
+    Set the position of the object frame in the current and desired camera frame
   */
     sim.setCameraPositionRelObj(cMo);
     sim.setDesiredCameraPosition(cdMo);
@@ -257,8 +256,7 @@ Create a display for each different cameras.
       // Get the view of the main external camera
       sim.getExternalImage(Iext1);
       // Get the view of an external camera that you can positionned thanks
-      // to a vpHomogeneousMatrix which describes the position of the camera
-      // relative to the world reference frame.
+      // to a vpHomogeneousMatrix which describes the position of the world reference frame in the camera frame
       sim.getExternalImage(Iext2, camoMw);
       // Display the views.
 
@@ -299,8 +297,7 @@ Create a display for each different cameras.
       // Get the view of the main external camera
       sim.getExternalImage(Iext1);
       // Get the view of an external camera that you can positionned thanks
-      // to a vpHomogeneousMatrix which describes the position of the camera
-      // relative to the world reference frame.
+      // to a vpHomogeneousMatrix which describes the position of the world reference frame in the camera frame
       sim.getExternalImage(Iext2, camoMw);
       // Display the views.
 
@@ -343,17 +340,21 @@ Create a display for each different cameras.
                  "refers to the html documentation to access the list of all "
                  "functions"
               << std::endl;
-    return 0;
-  } catch (vpException &e) {
+    return EXIT_SUCCESS;
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e << std::endl;
-    return 1;
+    return EXIT_SUCCESS;
   }
 }
 #else
 int main()
 {
-  vpERROR_TRACE("You do not have X11, OpenCV, GDI, D3D9 or GTK display "
-                "functionalities...");
+  std::cout << "You do not have X11, or GDI (Graphical Device Interface), or GTK functionalities to display images..." << std::endl;
+  std::cout << "Tip if you are on a unix-like system:" << std::endl;
+  std::cout << "- Install X11, configure again ViSP using cmake and build again this example" << std::endl;
+  std::cout << "Tip if you are on a windows-like system:" << std::endl;
+  std::cout << "- Install GDI, configure again ViSP using cmake and build again this example" << std::endl;
+  return EXIT_SUCCESS;
 }
 
 #endif
