@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -222,7 +222,7 @@ SoSeparator *createFrame(float longueurFleche = LONGUEUR_FLECHE, float proportio
   return frame;
 }
 
-SoSeparator *createCameraObject(const float zoomFactor = 1.0)
+SoSeparator *createCameraObject(float zoomFactor = 1.0)
 {
   vpDEBUG_TRACE(15, "# Entree.");
 
@@ -407,7 +407,7 @@ void vpSimulator::initSceneGraph()
 
   \param zoom: zoom factor of the objects. By default, 1.
 */
-void vpSimulator::setZoomFactor(const float zoom)
+void vpSimulator::setZoomFactor(float zoom)
 {
   zoomFactor = zoom;
   static bool firstTime = true;
@@ -440,14 +440,14 @@ void vpSimulator::setZoomFactor(const float zoom)
   \param zoomFactor : the new zoom use to specify the apparent size of the
   object \param index : the index of the Scale object to modify (see comments)
 */
-void vpSimulator::changeZoomFactor(const float zoomFactor, const int index)
+void vpSimulator::changeZoomFactor(float zoomFactor, int index)
 {
   SoScale *taille = (SoScale *)this->scene->getChild(index);
   taille->scaleFactor.setValue(zoomFactor, zoomFactor, zoomFactor);
   //  this->setZoomFactor(zoomFactor);
 }
 
-void vpSimulator::initInternalViewer(const unsigned int width, const unsigned int height)
+void vpSimulator::initInternalViewer(unsigned int width, unsigned int height)
 {
   internal_width = width;
   internal_height = height;
@@ -481,7 +481,7 @@ void vpSimulator::initInternalViewer(const unsigned int width, const unsigned in
   bufferView = new unsigned char[3 * width * height];
 }
 
-void vpSimulator::initExternalViewer(const unsigned int width, const unsigned int height)
+void vpSimulator::initExternalViewer(unsigned int width, unsigned int height)
 {
 
   external_width = width;
