@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -98,7 +98,7 @@ public:
                   vpColVector &weights);
 
   vpRobust &operator=(const vpRobust &other);
-#ifdef VISP_HAVE_CPP11_COMPATIBILITY
+#if (VISP_CXX_STANDARD >= VISP_CXX_STANDARD_11)
   vpRobust &operator=(const vpRobust &&other);
 #endif
 
@@ -106,13 +106,13 @@ public:
   void resize(unsigned int n_data);
 
   //! Set iteration
-  void setIteration(const unsigned int iter) { it = iter; }
+  void setIteration(unsigned int iter) { it = iter; }
 
   /*!
     Set maximal noise threshold.
     \param noise_threshold : Maximal noise threshold.
   */
-  inline void setThreshold(const double noise_threshold) { NoiseThreshold = noise_threshold; }
+  inline void setThreshold(double noise_threshold) { NoiseThreshold = noise_threshold; }
 
   //! Simult Mestimator
   vpColVector simultMEstimator(vpColVector &residues);
