@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -97,8 +97,7 @@ vpCameraParameters::vpCameraParameters(const vpCameraParameters &c)
   \param cam_u0,cam_v0 : principal points
 
  */
-vpCameraParameters::vpCameraParameters(const double cam_px, const double cam_py, const double cam_u0,
-                                       const double cam_v0)
+vpCameraParameters::vpCameraParameters(double cam_px, double cam_py, double cam_u0, double cam_v0)
   : px(DEFAULT_PX_PARAMETER), py(DEFAULT_PY_PARAMETER), u0(DEFAULT_U0_PARAMETER), v0(DEFAULT_V0_PARAMETER),
     kud(DEFAULT_KUD_PARAMETER), kdu(DEFAULT_KDU_PARAMETER), width(0), height(0), isFov(false), m_hFovAngle(0),
     m_vFovAngle(0), fovNormals(), inv_px(1. / DEFAULT_PX_PARAMETER), inv_py(1. / DEFAULT_PY_PARAMETER),
@@ -116,8 +115,8 @@ vpCameraParameters::vpCameraParameters(const double cam_px, const double cam_py,
   \param cam_kdu : distorted to undistorted radial distortion
 
  */
-vpCameraParameters::vpCameraParameters(const double cam_px, const double cam_py, const double cam_u0,
-                                       const double cam_v0, const double cam_kud, const double cam_kdu)
+vpCameraParameters::vpCameraParameters(double cam_px, double cam_py, double cam_u0,
+                                       double cam_v0, double cam_kud, double cam_kdu)
   : px(DEFAULT_PX_PARAMETER), py(DEFAULT_PY_PARAMETER), u0(DEFAULT_U0_PARAMETER), v0(DEFAULT_V0_PARAMETER),
     kud(DEFAULT_KUD_PARAMETER), kdu(DEFAULT_KDU_PARAMETER), width(0), height(0), isFov(false), m_hFovAngle(0),
     m_vFovAngle(0), fovNormals(), inv_px(1. / DEFAULT_PX_PARAMETER), inv_py(1. / DEFAULT_PY_PARAMETER),
@@ -180,8 +179,8 @@ Field of view (horizontal: 56.145 and vertical: 43.6028 degrees)
    \endcode
 
  */
-void vpCameraParameters::initPersProjWithoutDistortion(const double cam_px, const double cam_py, const double cam_u0,
-                                                       const double cam_v0)
+void vpCameraParameters::initPersProjWithoutDistortion(double cam_px, double cam_py, double cam_u0,
+                                                       double cam_v0)
 {
   this->projModel = vpCameraParameters::perspectiveProjWithoutDistortion;
 
@@ -246,8 +245,8 @@ Camera parameters for perspective projection with distortion:
 Field of view (horizontal: 56.14497387 and vertical: 43.60281897 degrees)
 \endcode
 */
-void vpCameraParameters::initPersProjWithDistortion(const double cam_px, const double cam_py, const double cam_u0,
-                                                    const double cam_v0, const double cam_kud, const double cam_kdu)
+void vpCameraParameters::initPersProjWithDistortion(double cam_px, double cam_py, double cam_u0,
+                                                    double cam_v0, double cam_kud, double cam_kdu)
 {
   this->projModel = vpCameraParameters::perspectiveProjWithDistortion;
 
@@ -381,6 +380,49 @@ vpCameraParameters &vpCameraParameters::operator=(const vpCameraParameters &cam)
   return *this;
 }
 
+/*!
+  True if the two objects are absolutely identical.
+ */
+bool vpCameraParameters::operator==(const vpCameraParameters &c) const {
+  if (projModel != c.projModel)
+    return false;
+
+  if (!vpMath::equal(px, c.px, std::numeric_limits<double>::epsilon()) ||
+      !vpMath::equal(py, c.py, std::numeric_limits<double>::epsilon()) ||
+      !vpMath::equal(u0, c.u0, std::numeric_limits<double>::epsilon()) ||
+      !vpMath::equal(v0, c.v0, std::numeric_limits<double>::epsilon()) ||
+      !vpMath::equal(kud, c.kud, std::numeric_limits<double>::epsilon()) ||
+      !vpMath::equal(kdu, c.kdu, std::numeric_limits<double>::epsilon()) ||
+      !vpMath::equal(inv_px, c.inv_px, std::numeric_limits<double>::epsilon()) ||
+      !vpMath::equal(inv_py, c.inv_py, std::numeric_limits<double>::epsilon()))
+    return false;
+
+  if (isFov != c.isFov ||
+      !vpMath::equal(m_hFovAngle, c.m_hFovAngle, std::numeric_limits<double>::epsilon()) ||
+      !vpMath::equal(m_vFovAngle, c.m_vFovAngle, std::numeric_limits<double>::epsilon()) ||
+      width != c.width || height != c.height)
+    return false;
+
+  if (fovNormals.size() != c.fovNormals.size())
+    return false;
+
+  std::vector<vpColVector>::const_iterator it1 = fovNormals.begin();
+  std::vector<vpColVector>::const_iterator it2 = c.fovNormals.begin();
+  for (; it1 != fovNormals.end() && it2 != c.fovNormals.end(); ++it1, ++it2) {
+    if (*it1 != *it2)
+      return false;
+  }
+
+  return true;
+}
+
+/*!
+  False if the two objects are absolutely identical.
+ */
+bool vpCameraParameters::operator!=(const vpCameraParameters &c) const {
+  return !(*this == c);
+}
+
 /*!
   Compute angles and normals of the FOV.
 
