@@ -1,3 +1,40 @@
+/****************************************************************************
+ *
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
+ *
+ * This software is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * See the file LICENSE.txt at the root directory of this source
+ * distribution for additional information about the GNU GPL.
+ *
+ * For using ViSP with software that can not be combined with the GNU
+ * GPL, please contact Inria about acquiring a ViSP Professional
+ * Edition License.
+ *
+ * See http://visp.inria.fr for more information.
+ *
+ * This software was developed at:
+ * Inria Rennes - Bretagne Atlantique
+ * Campus Universitaire de Beaulieu
+ * 35042 Rennes Cedex
+ * France
+ *
+ * If you have questions regarding the use of this file, please contact
+ * Inria at visp@inria.fr
+ *
+ * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
+ * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ *
+ * Description:
+ * Display implementation.
+ *
+ * Authors:
+ * Fabien Spindler
+ *
+ *****************************************************************************/
 #include <visp3/core/vpMeterPixelConversion.h>
 #include <visp3/core/vpPoint.h>
 
@@ -321,11 +358,19 @@ void vp_display_display_point(const vpImage<Type> &I, int i, int j, const vpColo
 
 template <class Type>
 void vp_display_display_polygon(const vpImage<Type> &I, const std::vector<vpImagePoint> &vip, const vpColor &color,
-                                unsigned int thickness)
+                                unsigned int thickness, bool closed=true)
 {
   if (I.display != NULL) {
-    for (unsigned int i = 0; i < vip.size(); i++)
-      (I.display)->displayLine(vip[i], vip[(i + 1) % vip.size()], color, thickness);
+    if (closed) {
+      for (unsigned int i = 0; i < vip.size(); i++) {
+        (I.display)->displayLine(vip[i], vip[(i + 1) % vip.size()], color, thickness);
+      }
+    }
+    else {
+      for (unsigned int i = 1; i < vip.size(); i++) {
+        (I.display)->displayLine(vip[i-1], vip[i], color, thickness);
+      }
+    }
   }
 }
 
