@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@
 
 #include <visp3/core/vpImageConvert.h>
 #include <visp3/core/vpImageFilter.h>
+#include <visp3/core/vpRGBa.h>
 #if defined(VISP_HAVE_OPENCV) && (VISP_HAVE_OPENCV_VERSION >= 0x020408)
 #include <opencv2/imgproc/imgproc.hpp>
 #elif defined(VISP_HAVE_OPENCV) && (VISP_HAVE_OPENCV_VERSION >= 0x020101)
@@ -73,7 +74,7 @@
   \f]
   Only pixels in the input image fully covered by the kernel are considered.
 */
-void vpImageFilter::filter(const vpImage<unsigned char> &I, vpImage<double> &If, const vpMatrix &M, const bool convolve)
+void vpImageFilter::filter(const vpImage<unsigned char> &I, vpImage<double> &If, const vpMatrix &M, bool convolve)
 {
   unsigned int size_y = M.getRows(), size_x = M.getCols();
   unsigned int half_size_y = size_y / 2, half_size_x = size_x / 2;
@@ -124,7 +125,7 @@ void vpImageFilter::filter(const vpImage<unsigned char> &I, vpImage<double> &If,
   \param convolve : If true, perform a convolution otherwise a correlation.
 */
 void vpImageFilter::filter(const vpImage<double> &I, vpImage<double> &Iu, vpImage<double> &Iv, const vpMatrix &M,
-                           const bool convolve)
+                           bool convolve)
 {
   unsigned int size = M.getRows();
   unsigned int half_size = size / 2;
@@ -295,8 +296,8 @@ int main()
   \param apertureSobel : Size of the mask for the Sobel operator (odd number).
 */
 void vpImageFilter::canny(const vpImage<unsigned char> &Isrc, vpImage<unsigned char> &Ires,
-                          const unsigned int gaussianFilterSize, const double thresholdCanny,
-                          const unsigned int apertureSobel)
+                          unsigned int gaussianFilterSize, double thresholdCanny,
+                          unsigned int apertureSobel)
 {
 #if (VISP_HAVE_OPENCV_VERSION < 0x020408)
   IplImage *img_ipl = NULL;
@@ -361,6 +362,28 @@ void vpImageFilter::filterX(const vpImage<unsigned char> &I, vpImage<double> &dI
     }
   }
 }
+void vpImageFilter::filterX(const vpImage<vpRGBa> &I, vpImage<vpRGBa> &dIx, const double *filter,
+                            unsigned int size)
+{
+  dIx.resize(I.getHeight(), I.getWidth());
+  for (unsigned int i = 0; i < I.getHeight(); i++) {
+    for (unsigned int j = 0; j < (size - 1) / 2; j++) {
+      dIx[i][j].R = static_cast<unsigned char>(vpImageFilter::filterXLeftBorderR(I, i, j, filter, size));
+      dIx[i][j].G = static_cast<unsigned char>(vpImageFilter::filterXLeftBorderG(I, i, j, filter, size));
+      dIx[i][j].B = static_cast<unsigned char>(vpImageFilter::filterXLeftBorderB(I, i, j, filter, size));
+    }
+    for (unsigned int j = (size - 1) / 2; j < I.getWidth() - (size - 1) / 2; j++) {
+      dIx[i][j].R = static_cast<unsigned char>(vpImageFilter::filterXR(I, i, j, filter, size));
+      dIx[i][j].G = static_cast<unsigned char>(vpImageFilter::filterXG(I, i, j, filter, size));
+      dIx[i][j].B = static_cast<unsigned char>(vpImageFilter::filterXB(I, i, j, filter, size));
+    }
+    for (unsigned int j = I.getWidth() - (size - 1) / 2; j < I.getWidth(); j++) {
+      dIx[i][j].R = static_cast<unsigned char>(vpImageFilter::filterXRightBorderR(I, i, j, filter, size));
+      dIx[i][j].G = static_cast<unsigned char>(vpImageFilter::filterXRightBorderG(I, i, j, filter, size));
+      dIx[i][j].B = static_cast<unsigned char>(vpImageFilter::filterXRightBorderB(I, i, j, filter, size));
+    }
+  }
+}
 void vpImageFilter::filterX(const vpImage<double> &I, vpImage<double> &dIx, const double *filter, unsigned int size)
 {
   dIx.resize(I.getHeight(), I.getWidth());
@@ -398,6 +421,32 @@ void vpImageFilter::filterY(const vpImage<unsigned char> &I, vpImage<double> &dI
     }
   }
 }
+void vpImageFilter::filterY(const vpImage<vpRGBa> &I, vpImage<vpRGBa> &dIy, const double *filter,
+                            unsigned int size)
+{
+  dIy.resize(I.getHeight(), I.getWidth());
+  for (unsigned int i = 0; i < (size - 1) / 2; i++) {
+    for (unsigned int j = 0; j < I.getWidth(); j++) {
+      dIy[i][j].R = static_cast<unsigned char>(vpImageFilter::filterYTopBorderR(I, i, j, filter, size));
+      dIy[i][j].G = static_cast<unsigned char>(vpImageFilter::filterYTopBorderG(I, i, j, filter, size));
+      dIy[i][j].B = static_cast<unsigned char>(vpImageFilter::filterYTopBorderB(I, i, j, filter, size));
+    }
+  }
+  for (unsigned int i = (size - 1) / 2; i < I.getHeight() - (size - 1) / 2; i++) {
+    for (unsigned int j = 0; j < I.getWidth(); j++) {
+      dIy[i][j].R = static_cast<unsigned char>(vpImageFilter::filterYR(I, i, j, filter, size));
+      dIy[i][j].G = static_cast<unsigned char>(vpImageFilter::filterYG(I, i, j, filter, size));
+      dIy[i][j].B = static_cast<unsigned char>(vpImageFilter::filterYB(I, i, j, filter, size));
+    }
+  }
+  for (unsigned int i = I.getHeight() - (size - 1) / 2; i < I.getHeight(); i++) {
+    for (unsigned int j = 0; j < I.getWidth(); j++) {
+      dIy[i][j].R = static_cast<unsigned char>(vpImageFilter::filterYBottomBorderR(I, i, j, filter, size));
+      dIy[i][j].G = static_cast<unsigned char>(vpImageFilter::filterYBottomBorderG(I, i, j, filter, size));
+      dIy[i][j].B = static_cast<unsigned char>(vpImageFilter::filterYBottomBorderB(I, i, j, filter, size));
+    }
+  }
+}
 void vpImageFilter::filterY(const vpImage<double> &I, vpImage<double> &dIy, const double *filter, unsigned int size)
 {
   dIy.resize(I.getHeight(), I.getWidth());
@@ -424,10 +473,11 @@ void vpImageFilter::filterY(const vpImage<double> &I, vpImage<double> &dIy, cons
   \param GI : Filtered image.
   \param size : Filter size. This value should be odd.
   \param sigma : Gaussian standard deviation. If it is equal to zero or
-  negative, it is computed from filter size as sigma = (size-1)/6. \param
-  normalize : Flag indicating whether to normalize the filter coefficients or
+  negative, it is computed from filter size as sigma = (size-1)/6.
+  \param normalize : Flag indicating whether to normalize the filter coefficients or
   not.
 
+  \sa getGaussianKernel() to know which kernel is used.
  */
 void vpImageFilter::gaussianBlur(const vpImage<unsigned char> &I, vpImage<double> &GI, unsigned int size, double sigma,
                                  bool normalize)
@@ -441,16 +491,39 @@ void vpImageFilter::gaussianBlur(const vpImage<unsigned char> &I, vpImage<double
   delete[] fg;
 }
 
+/*!
+  Apply a Gaussian blur to RGB color image.
+  \param I : Input image.
+  \param GI : Filtered image.
+  \param size : Filter size. This value should be odd.
+  \param sigma : Gaussian standard deviation. If it is equal to zero or
+  negative, it is computed from filter size as sigma = (size-1)/6.
+  \param normalize : Flag indicating whether to normalize the filter coefficients or not.
+
+  \sa getGaussianKernel() to know which kernel is used.
+ */
+void vpImageFilter::gaussianBlur(const vpImage<vpRGBa> &I, vpImage<vpRGBa> &GI, unsigned int size, double sigma,
+                                 bool normalize)
+{
+  double *fg = new double[(size + 1) / 2];
+  vpImageFilter::getGaussianKernel(fg, size, sigma, normalize);
+  vpImage<vpRGBa> GIx;
+  vpImageFilter::filterX(I, GIx, fg, size);
+  vpImageFilter::filterY(GIx, GI, fg, size);
+  GIx.destroy();
+  delete[] fg;
+}
+
 /*!
   Apply a Gaussian blur to a double image.
   \param I : Input double image.
   \param GI : Filtered image.
   \param size : Filter size. This value should be odd.
   \param sigma : Gaussian standard deviation. If it is equal to zero or
-  negative, it is computed from filter size as sigma = (size-1)/6. \param
-  normalize : Flag indicating whether to normalize the filter coefficients or
-  not.
+  negative, it is computed from filter size as sigma = (size-1)/6.
+  \param normalize : Flag indicating whether to normalize the filter coefficients or not.
 
+  \sa getGaussianKernel() to know which kernel is used.
  */
 void vpImageFilter::gaussianBlur(const vpImage<double> &I, vpImage<double> &GI, unsigned int size, double sigma,
                                  bool normalize)
@@ -465,15 +538,19 @@ void vpImageFilter::gaussianBlur(const vpImage<double> &I, vpImage<double> &GI,
 }
 
 /*!
-  Return the coefficients of a Gaussian filter.
+  Return the coefficients \f$G_i\f$ of a Gaussian filter.
 
-  \param filter : Pointer to the filter kernel that should refer to a
+  \param[out] filter : Pointer to the half size filter kernel that should refer to a
   (size+1)/2 array. The first value refers to the central coefficient, the
   next one to the right coefficients. Left coefficients could be deduced by
-  symmetry. \param size : Filter size. This value should be odd. \param sigma
-  : Gaussian standard deviation. If it is equal to zero or negative, it is
-  computed from filter size as sigma = (size-1)/6. \param normalize : Flag
-  indicating whether to normalize the filter coefficients or not.
+  symmetry.
+  \param[in] size : Filter size. This value should be odd and positive.
+  \param[in] sigma : Gaussian standard deviation \f$ \sigma \f$. If it is equal to zero or negative, it is
+  computed from filter size as sigma = (size-1)/6.
+  \param[in] normalize : Flag indicating whether to normalize the filter coefficients or not. In that case \f$\Sigma G_i = 1 \f$.
+
+  The function computes the \e (size+1)/2 values of the Gaussian filter cooefficients \f$ G_i \f$ as:
+  \f[ G_i = \frac{1}{\sigma  \sqrt{2 \pi}} \exp{(-i^2 / (2. * \sigma^2))}\f]
 */
 void vpImageFilter::getGaussianKernel(double *filter, unsigned int size, double sigma, bool normalize)
 {
@@ -509,10 +586,11 @@ void vpImageFilter::getGaussianKernel(double *filter, unsigned int size, double
   \param filter : Pointer to the filter kernel that should refer to a
   (size+1)/2 array. The first value refers to the central coefficient, the
   next one to the right coefficients. Left coefficients could be deduced by
-  symmetry. \param size : Filter size. This value should be odd. \param sigma
-  : Gaussian standard deviation. If it is equal to zero or negative, it is
-  computed from filter size as sigma = (size-1)/6. \param normalize : Flag
-  indicating whether to normalize the filter coefficients or not.
+  symmetry.
+  \param size : Filter size. This value should be odd.
+  \param sigma : Gaussian standard deviation. If it is equal to zero or negative, it is
+  computed from filter size as sigma = (size-1)/6.
+  \param normalize : Flag indicating whether to normalize the filter coefficients or not.
 */
 void vpImageFilter::getGaussianDerivativeKernel(double *filter, unsigned int size, double sigma, bool normalize)
 {
@@ -664,11 +742,10 @@ void vpImageFilter::getGradY(const vpImage<double> &I, vpImage<double> &dIy, con
    Compute the gradient along X after applying a gaussian filter along Y.
    \param I : Input image
    \param dIx : Gradient along X.
-   \param gaussianKernel : Gaussian kernel which values should be computed
-   using vpImageFilter::getGaussianKernel(). \param gaussianDerivativeKernel :
-   Gaussian derivative kernel which values should be computed using
-   vpImageFilter::getGaussianDerivativeKernel(). \param size : Size of the
-   Gaussian and Gaussian derivative kernels.
+   \param gaussianKernel : Gaussian kernel which values should be computed using vpImageFilter::getGaussianKernel().
+   \param gaussianDerivativeKernel : Gaussian derivative kernel which values should be computed using
+   vpImageFilter::getGaussianDerivativeKernel().
+   \param size : Size of the Gaussian and Gaussian derivative kernels.
  */
 void vpImageFilter::getGradXGauss2D(const vpImage<unsigned char> &I, vpImage<double> &dIx, const double *gaussianKernel,
                                     const double *gaussianDerivativeKernel, unsigned int size)
@@ -682,11 +759,10 @@ void vpImageFilter::getGradXGauss2D(const vpImage<unsigned char> &I, vpImage<dou
    Compute the gradient along Y after applying a gaussian filter along X.
    \param I : Input image
    \param dIy : Gradient along Y.
-   \param gaussianKernel : Gaussian kernel which values should be computed
-   using vpImageFilter::getGaussianKernel(). \param gaussianDerivativeKernel :
-   Gaussian derivative kernel which values should be computed using
-   vpImageFilter::getGaussianDerivativeKernel(). \param size : Size of the
-   Gaussian and Gaussian derivative kernels.
+   \param gaussianKernel : Gaussian kernel which values should be computed  using vpImageFilter::getGaussianKernel().
+   \param gaussianDerivativeKernel : Gaussian derivative kernel which values should be computed using
+   vpImageFilter::getGaussianDerivativeKernel().
+   \param size : Size of the Gaussian and Gaussian derivative kernels.
  */
 void vpImageFilter::getGradYGauss2D(const vpImage<unsigned char> &I, vpImage<double> &dIy, const double *gaussianKernel,
                                     const double *gaussianDerivativeKernel, unsigned int size)
@@ -782,3 +858,83 @@ void vpImageFilter::getGaussYPyramidal(const vpImage<unsigned char> &I, vpImage<
   }
 #endif
 }
+
+/*!
+  Get Sobel kernel for X-direction.
+
+  \param filter : Pointer to a double array already allocated.
+  \param size : Kernel size computed as: kernel_size = size*2 + 1 (max size is 20).
+  \return Scaling factor.
+ */
+double vpImageFilter::getSobelKernelX(double *filter, unsigned int size)
+{
+  if (size == 0)
+    throw vpException(vpException::dimensionError, "Cannot get Sobel kernel of size 0!");
+  if (size > 20)
+    throw vpException(vpException::dimensionError, "Cannot get Sobel kernel of size > 20!");
+
+  vpMatrix SobelY(size*2+1, size*2+1);
+  double norm = getSobelKernelY(SobelY.data, size);
+  memcpy(filter, SobelY.t().data, SobelY.getRows()*SobelY.getCols()*sizeof(double));
+  return norm;
+}
+
+/*!
+  Get Sobel kernel for Y-direction.
+
+  \param filter : Pointer to a double array already allocated.
+  \param size : Kernel size computed as: kernel_size = size*2 + 1 (max size is 20).
+  \return Scaling factor.
+ */
+double vpImageFilter::getSobelKernelY(double *filter, unsigned int size)
+{
+  //Sobel kernel pre-computed for the usual size
+  static const double SobelY3x3[9] = {-1.0, -2.0, -1.0,
+                                      0.0, 0.0, 0.0,
+                                      1.0, 2.0, 1.0};
+  static const double SobelY5x5[25] = {-1.0, -4.0, -6.0, -4.0, -1.0,
+                                       -2.0, -8.0, -12.0, -8.0, -2.0,
+                                       0.0, 0.0, 0.0, 0.0, 0.0,
+                                       2.0, 8.0, 12.0, 8.0, 2.0,
+                                       1.0, 4.0, 6.0, 4.0, 1.0};
+  static const double SobelY7x7[49] = {-1, -6, -15, -20, -15, -6, -1,
+                                       -4, -24, -60, -80, -60, -24, -4,
+                                       -5, -30, -75, -100, -75, -30, -5,
+                                        0, 0, 0, 0, 0, 0, 0,
+                                        5, 30, 75, 100, 75, 30, 5,
+                                        4, 24, 60, 80, 60, 24, 4,
+                                        1, 6, 15, 20, 15, 6, 1};
+  static const vpMatrix smoothingKernel(3,3);
+  smoothingKernel[0][0] = 1.0;  smoothingKernel[0][1] = 2.0;  smoothingKernel[0][2] = 1.0;
+  smoothingKernel[1][0] = 2.0;  smoothingKernel[1][1] = 4.0;  smoothingKernel[1][2] = 2.0;
+  smoothingKernel[2][0] = 1.0;  smoothingKernel[2][1] = 2.0;  smoothingKernel[2][2] = 1.0;
+
+  if (size == 0)
+    throw vpException(vpException::dimensionError, "Cannot get Sobel kernel of size 0!");
+  if (size > 20)
+    throw vpException(vpException::dimensionError, "Cannot get Sobel kernel of size > 20!");
+
+  const unsigned int kernel_size = size*2+1;
+  if (kernel_size == 3) {
+    memcpy(filter, SobelY3x3, kernel_size*kernel_size*sizeof(double));
+    return 1/8.0;
+  }
+  if (kernel_size == 5) {
+    memcpy(filter, SobelY5x5, kernel_size*kernel_size*sizeof(double));
+    return 1/16.0;
+  }
+  if (kernel_size == 7) {
+    memcpy(filter, SobelY7x7, kernel_size*kernel_size*sizeof(double));
+    return 1/16.0;
+  }
+
+  vpMatrix sobelY(7,7);
+  memcpy(sobelY.data, SobelY7x7, sobelY.getRows()*sobelY.getCols()*sizeof(double));
+  for (unsigned int i = 4; i <= size; i++) {
+    sobelY = vpMatrix::conv2(sobelY, smoothingKernel, "full");
+  }
+
+  memcpy(filter, sobelY.data, sobelY.getRows()*sobelY.getCols()*sizeof(double));
+
+  return 1/16.0;
+}
