@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -323,7 +323,10 @@ vpColVector::vpColVector(const std::vector<float> &v) : vpArray2D<double>((unsig
     (*this)[i] = (double)(v[i]);
 }
 
-#ifdef VISP_HAVE_CPP11_COMPATIBILITY
+#if (VISP_CXX_STANDARD >= VISP_CXX_STANDARD_11)
+/*!
+  Move constructor that take rvalue.
+ */
 vpColVector::vpColVector(vpColVector &&v) : vpArray2D<double>()
 {
   rowNum = v.rowNum;
@@ -626,6 +629,56 @@ vpColVector &vpColVector::operator<<(double *x)
   return *this;
 }
 
+/*!
+  This operator could be used to set column vector elements:
+  \code
+#include <visp3/code/vpColVector.h
+
+int main()
+{
+  vpColVector v;
+  v << -1, -2.1, -3;
+  std::cout << "v:" << v << std::endl;
+}
+  \endcode
+  It produces the following printings:
+  \code
+v: -1  -2.1  -3
+  \endcode
+  \sa operator,()
+*/
+vpColVector& vpColVector::operator<<(double val)
+{
+  resize(1, false);
+  data[0] = val;
+  return *this;
+}
+
+/*!
+  This operator could be used to set column vector elements:
+  \code
+#include <visp3/code/vpColVector.h
+
+int main()
+{
+  vpColVector v;
+  v << -1, -2.1, -3;
+  std::cout << "v:" << v << std::endl;
+}
+  \endcode
+  It produces the following printings:
+  \code
+v: -1  -2.1  -3
+  \endcode
+  \sa operator<<()
+*/
+vpColVector& vpColVector::operator,(double val)
+{
+  resize(rowNum + 1, false);
+  data[rowNum - 1] = val;
+  return *this;
+}
+
 //! Set each element of the column vector to x.
 vpColVector &vpColVector::operator=(double x)
 {
@@ -636,7 +689,23 @@ vpColVector &vpColVector::operator=(double x)
   return *this;
 }
 
-#ifdef VISP_HAVE_CPP11_COMPATIBILITY
+/*!
+ * Converts the vpColVector to a std::vector.
+ * \return The corresponding std::vector<double>.
+ */
+std::vector<double> vpColVector::toStdVector()
+{
+  std::vector<double> v(this->size());
+
+  for (unsigned int i = 0; i < this->size(); i++)
+    v[i] = data[i];
+  return v;
+}
+
+#if (VISP_CXX_STANDARD >= VISP_CXX_STANDARD_11)
+/*!
+  Overloaded move assignment operator taking rvalue.
+ */
 vpColVector &vpColVector::operator=(vpColVector &&other)
 {
   if (this != &other) {
@@ -658,8 +727,55 @@ vpColVector &vpColVector::operator=(vpColVector &&other)
 
   return *this;
 }
+
+/*!
+  Set vector elements and size from a list of values.
+  \param list : List of double. Vector size matches the number of elements.
+  \return The modified vector.
+  \code
+#include <visp3/core/vpColVector.h>
+
+int main()
+{
+  vpColVector c;
+  c = { 0, -1, -2 };
+  std::cout << "c:\n" << c << std::endl;
+}
+  \endcode
+  It produces the following printings:
+  \code
+c:
+0
+-1
+-2
+  \endcode
+  \sa operator<<()
+ */
+vpColVector& vpColVector::operator=(const std::initializer_list<double> &list)
+{
+  resize(static_cast<unsigned int>(list.size()), false);
+  std::copy(list.begin(), list.end(), data);
+  return *this;
+}
 #endif
 
+bool vpColVector::operator==(const vpColVector &v) const {
+  if (rowNum != v.rowNum ||
+      colNum != v.colNum /* should not happen */)
+    return false;
+
+  for (unsigned int i = 0; i < rowNum; i++) {
+    if (!vpMath::equal(data[i], v.data[i], std::numeric_limits<double>::epsilon()))
+      return false;
+  }
+
+  return true;
+}
+
+bool vpColVector::operator!=(const vpColVector &v) const {
+  return !(*this == v);
+}
+
 /*!
   Transpose the column vector. The resulting vector becomes a row vector.
 */
@@ -761,7 +877,31 @@ vpColVector &vpColVector::normalize()
 }
 
 /*!
-   Return a column vector with elements of \e v that are reverse sorted.
+   Return a column vector with elements of \e v that are reverse sorted with
+   values going from greatest to lowest.
+
+   Example:
+   \code
+#include <visp3/core/vpColVector.h>
+
+int main()
+{
+  vpColVector v(10);
+  v[0] = 5; v[1] = 7; v[2] = 4; v[3] = 2; v[4] = 8;
+  v[5] = 6; v[6] = 1; v[7] = 9; v[8] = 0; v[9] = 3;
+
+  std::cout << "v: " << v.t() << std::endl;
+
+  vpColVector s = vpColVector::invSort(v);
+  std::cout << "s: " << s.t() << std::endl;
+}
+   \endcode
+   Output:
+   \code
+v: 5  7  4  2  8  6  1  9  0  3
+s: 9  8  7  6  5  4  3  2  1  0
+   \endcode
+
    \sa sort()
  */
 vpColVector vpColVector::invSort(const vpColVector &v)
@@ -790,7 +930,30 @@ vpColVector vpColVector::invSort(const vpColVector &v)
 }
 
 /*!
-   Return a column vector with elements of \e v that are sorted.
+   Return a column vector with elements of \e v that are sorted with values
+   going from lowest to geatest.
+
+   Example:
+   \code
+#include <visp3/core/vpColVector.h>
+
+int main()
+{
+  vpColVector v(10);
+  v[0] = 5; v[1] = 7; v[2] = 4; v[3] = 2; v[4] = 8;
+  v[5] = 6; v[6] = 1; v[7] = 9; v[8] = 0; v[9] = 3;
+
+  std::cout << "v: " << v.t() << std::endl;
+
+  vpColVector s = vpColVector::sort(v);
+  std::cout << "s: " << s.t() << std::endl;
+}
+   \endcode
+   Output:
+   \code
+v: 5  7  4  2  8  6  1  9  0  3
+s: 0  1  2  3  4  5  6  7  8  9
+   \endcode
    \sa invSort()
  */
 vpColVector vpColVector::sort(const vpColVector &v)
@@ -834,7 +997,7 @@ vpColVector vpColVector::sort(const vpColVector &v)
   \sa stack(const vpColVector &, const vpColVector &, vpColVector &)
 
 */
-void vpColVector::stack(const double &d)
+void vpColVector::stack(double d)
 {
   this->resize(rowNum + 1, false);
   (*this)[rowNum - 1] = d;
@@ -971,7 +1134,7 @@ double vpColVector::median(const vpColVector &v)
 /*!
   Compute the standard deviation value of all the elements of the vector.
 */
-double vpColVector::stdev(const vpColVector &v, const bool useBesselCorrection)
+double vpColVector::stdev(const vpColVector &v, bool useBesselCorrection)
 {
   if (v.data == NULL || v.size() == 0) {
     throw(vpException(vpException::dimensionError, "Cannot compute column vector stdev: vector empty"));
@@ -1088,7 +1251,7 @@ vpColVector vpColVector::crossProd(const vpColVector &a, const vpColVector &b)
 
   \sa reshape(vpMatrix &, const unsigned int &, const unsigned int &)
 */
-vpMatrix vpColVector::reshape(const unsigned int &nrows, const unsigned int &ncols)
+vpMatrix vpColVector::reshape(unsigned int nrows, unsigned int ncols)
 {
   vpMatrix M(nrows, ncols);
   reshape(M, nrows, ncols);
@@ -1106,7 +1269,7 @@ have not the same size.
 
   The following example shows how to use this method.
   \code
-#include <visp/vpColVector.h>
+#include <visp3/core/vpColVector.h>
 
 int main()
 {
@@ -1398,20 +1561,31 @@ double vpColVector::sumSquare() const
 }
 
 /*!
-  Compute and return the Euclidean norm \f$ ||x|| = \sqrt{ \sum{v_{i}^2}} \f$.
+  \deprecated This function is deprecated. You should rather use frobeniusNorm().
+
+  Compute and return the Euclidean norm also called Fronebius norm \f$ ||v|| = \sqrt{ \sum{v_{i}^2}} \f$.
 
   \return The Euclidean norm if the vector is initialized, 0 otherwise.
 
+  \sa frobeniusNorm(), infinityNorm()
+
 */
 double vpColVector::euclideanNorm() const
 {
-  // Use directly sumSquare() function
-  double norm = sumSquare();
+  return frobeniusNorm();
+}
 
-  // Old code used
-  //  for (unsigned int i=0;i<dsize;i++) {
-  //    double x = *(data +i); norm += x*x;
-  //  }
+/*!
+  Compute and return the Fronebius norm \f$ ||v|| = \sqrt{ \sum{v_{i}^2}} \f$.
+
+  \return The Fronebius norm if the vector is initialized, 0 otherwise.
+
+  \sa infinityNorm()
+
+*/
+double vpColVector::frobeniusNorm() const
+{
+  double norm = sumSquare();
 
   return sqrt(norm);
 }
@@ -1451,14 +1625,14 @@ vpColVector vpColVector::hadamard(const vpColVector &v) const
 
 /*!
 
-  Compute and return the infinity norm \f$ {||x||}_{\infty} =
-  max\left({\mid x_{i} \mid}\right) \f$ with \f$i \in
-  \{0, ..., m-1\}\f$ where \e m is the vector size and \f$x_i\f$ an element of
+  Compute and return the infinity norm \f$ {||v||}_{\infty} =
+  max\left({\mid v_{i} \mid}\right) \f$ with \f$i \in
+  \{0, ..., m-1\}\f$ where \e m is the vector size and \f$v_i\f$ an element of
   the vector.
 
   \return The infinity norm if the matrix is initialized, 0 otherwise.
 
-  \sa euclideanNorm()
+  \sa frobeniusNorm()
 */
 double vpColVector::infinityNorm() const
 {
@@ -1657,7 +1831,7 @@ std::ostream &vpColVector::matlabPrint(std::ostream &os) const
   \param c : Not used.
 
  */
-void vpColVector::insert(const vpColVector &v, const unsigned int r, const unsigned int c)
+void vpColVector::insert(const vpColVector &v, unsigned int r, unsigned int c)
 {
   (void)c;
   insert(r, v);
