@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,14 +54,18 @@
 #endif
 
 #ifdef VISP_HAVE_LAPACK
-#ifdef VISP_HAVE_LAPACK_BUILT_IN
+#  ifdef VISP_HAVE_MKL
+#include <mkl.h>
+typedef MKL_INT integer;
+#  else
+#    ifdef VISP_HAVE_LAPACK_BUILT_IN
 typedef long int integer;
-#else
+#    else
 typedef int integer;
-#endif
-
+#    endif
 extern "C" void dpotrf_(char *uplo, integer *n, double *a, integer *lda, integer *info);
 extern "C" int dpotri_(char *uplo, integer *n, double *a, integer *lda, integer *info);
+#  endif
 #endif
 
 /*!
@@ -104,7 +108,7 @@ int main()
 
 vpMatrix vpMatrix::inverseByCholesky() const
 {
-#ifdef VISP_HAVE_LAPACK
+#if defined(VISP_HAVE_LAPACK)
   return inverseByCholeskyLapack();
 #elif (VISP_HAVE_OPENCV_VERSION >= 0x020101)
   return inverseByCholeskyOpenCV();
@@ -115,9 +119,7 @@ vpMatrix vpMatrix::inverseByCholesky() const
 #endif
 }
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
-
-#ifdef VISP_HAVE_LAPACK
+#if defined(VISP_HAVE_LAPACK)
 /*!
   Compute the inverse of a n-by-n matrix using the Cholesky decomposition with
 Lapack 3rd party. The matrix must be real symmetric positive defined.
@@ -239,5 +241,3 @@ vpMatrix vpMatrix::inverseByCholeskyOpenCV() const
   return A;
 }
 #endif
-
-#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS
