@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -124,6 +124,73 @@ vpRowVector &vpRowVector::operator=(double x)
   return *this;
 }
 
+#if (VISP_CXX_STANDARD >= VISP_CXX_STANDARD_11)
+vpRowVector &vpRowVector::operator=(vpRowVector &&other)
+{
+  if (this != &other) {
+    free(data);
+    free(rowPtrs);
+
+    rowNum = other.rowNum;
+    colNum = other.colNum;
+    rowPtrs = other.rowPtrs;
+    dsize = other.dsize;
+    data = other.data;
+
+    other.rowNum = 0;
+    other.colNum = 0;
+    other.rowPtrs = NULL;
+    other.dsize = 0;
+    other.data = NULL;
+  }
+
+  return *this;
+}
+
+
+/*!
+  Set vector elements from a list of double.
+  \code
+#include <visp3/core/vpRowVector.cpp>
+
+int main()
+{
+  vpRowVector r;
+  r = {1, 2, 3};
+  std::cout << "r: " << r << std::endl;
+}
+  \endcode
+  It produces the following printings:
+  \code
+r: 1  2  3
+  \endcode
+  \sa operator<<()
+*/
+vpRowVector &vpRowVector::operator=(const std::initializer_list<double> &list)
+{
+  resize(1, static_cast<unsigned int>(list.size()), false);
+  std::copy(list.begin(), list.end(), data);
+  return *this;
+}
+#endif
+
+bool vpRowVector::operator==(const vpRowVector &v) const {
+  if (colNum != v.colNum ||
+      rowNum != v.rowNum /* should not happen */)
+    return false;
+
+  for (unsigned int i = 0; i < colNum; i++) {
+    if (!vpMath::equal(data[i], v.data[i], std::numeric_limits<double>::epsilon()))
+      return false;
+  }
+
+  return true;
+}
+
+bool vpRowVector::operator!=(const vpRowVector &v) const {
+  return !(*this == v);
+}
+
 /*!
 
   Multiply a row vector by a column vector.
@@ -420,6 +487,20 @@ vpRowVector &vpRowVector::operator<<(const vpRowVector &v)
   return *this;
 }
 
+vpRowVector& vpRowVector::operator<<(double val)
+{
+  resize(1, false);
+  data[0] = val;
+  return *this;
+}
+
+vpRowVector& vpRowVector::operator,(double val)
+{
+  resize(colNum + 1, false);
+  data[colNum - 1] = val;
+  return *this;
+}
+
 /*!
   Transpose the row vector. The resulting vector becomes a column vector.
 */
@@ -502,6 +583,23 @@ vpRowVector::vpRowVector(const vpRowVector &v, unsigned int c, unsigned int ncol
   init(v, c, ncols);
 }
 
+#if (VISP_CXX_STANDARD >= VISP_CXX_STANDARD_11)
+vpRowVector::vpRowVector(vpRowVector &&v) : vpArray2D<double>()
+{
+  rowNum = v.rowNum;
+  colNum = v.colNum;
+  rowPtrs = v.rowPtrs;
+  dsize = v.dsize;
+  data = v.data;
+
+  v.rowNum = 0;
+  v.colNum = 0;
+  v.rowPtrs = NULL;
+  v.dsize = 0;
+  v.data = NULL;
+}
+#endif
+
 /*!
   Normalise the vector given as input parameter and return the normalized
   vector:
@@ -548,7 +646,7 @@ vpRowVector &vpRowVector::normalize()
 
   \sa reshape(vpMatrix &, const unsigned int &, const unsigned int &)
 */
-vpMatrix vpRowVector::reshape(const unsigned int &nrows, const unsigned int &ncols)
+vpMatrix vpRowVector::reshape(unsigned int nrows, unsigned int ncols)
 {
   vpMatrix M(nrows, ncols);
   reshape(M, nrows, ncols);
@@ -566,7 +664,7 @@ not the same size.
 
   The following example shows how to use this method.
   \code
-#include <visp/vpRowVector.h>
+#include <visp3/core/vpRowVector.h>
 
 int main()
 {
@@ -622,7 +720,7 @@ void vpRowVector::reshape(vpMatrix &M, const unsigned int &nrows, const unsigned
 
   The following example shows how to use this function:
   \code
-#include <visp/vpRowVector.h>
+#include <visp3/core/vpRowVector.h>
 
 int main()
 {
@@ -657,6 +755,19 @@ void vpRowVector::insert(unsigned int i, const vpRowVector &v)
     (*this)[i + j] = v[j];
 }
 
+/*!
+ * Converts the vpRowVector to a std::vector.
+ * \return The corresponding std::vector<double>.
+ */
+std::vector<double> vpRowVector::toStdVector()
+{
+  std::vector<double> v(this->size());
+
+  for (unsigned int i = 0; i < this->size(); i++)
+    v[i] = data[i];
+  return v;
+}
+
 /*!
   Stack row vector with a new element at the end of the vector.
 
@@ -673,7 +784,7 @@ void vpRowVector::insert(unsigned int i, const vpRowVector &v)
   \sa stack(const vpRowVector &, const vpRowVector &, vpRowVector &)
 
 */
-void vpRowVector::stack(const double &d)
+void vpRowVector::stack(double d)
 {
   this->resize(colNum + 1, false);
   (*this)[colNum - 1] = d;
@@ -811,7 +922,7 @@ double vpRowVector::median(const vpRowVector &v)
 /*!
   Compute the standard deviation value of all the elements of the vector.
 */
-double vpRowVector::stdev(const vpRowVector &v, const bool useBesselCorrection)
+double vpRowVector::stdev(const vpRowVector &v, bool useBesselCorrection)
 {
   if (v.data == NULL || v.size() == 0) {
     throw(vpException(vpException::dimensionError, "Cannot compute mean value of an empty row vector"));
@@ -978,17 +1089,27 @@ double vpRowVector::sumSquare() const
 }
 
 /*!
-  Compute and return the Euclidean norm \f$ ||x|| = \sqrt{ \sum{v_{i}^2}} \f$.
+  \deprecated This function is deprecated. You should rather use frobeniusNorm().
+
+  Compute and return the Euclidean norm also called Fronebius norm \f$ ||v|| = \sqrt{ \sum{v_{i}^2}} \f$.
 
   \return The Euclidean norm if the vector is initialized, 0 otherwise.
+
+  \sa frobeniusNorm()
 */
 double vpRowVector::euclideanNorm() const
 {
-  double norm = 0.0;
-  for (unsigned int i = 0; i < dsize; i++) {
-    double x = *(data + i);
-    norm += x * x;
-  }
+  return frobeniusNorm();
+}
+
+/*!
+  Compute and return the Fronebius norm \f$ ||v|| = \sqrt{ \sum{v_{i}^2}} \f$.
+
+  \return The Fronebius norm if the vector is initialized, 0 otherwise.
+*/
+double vpRowVector::frobeniusNorm() const
+{
+  double norm = sumSquare();
 
   return sqrt(norm);
 }
@@ -998,8 +1119,8 @@ double vpRowVector::euclideanNorm() const
 
   \param v : Input row vector used for initialization.
   \param c : column index in \e v that corresponds to the first element of the
-row vector to contruct. \param ncols : Number of columns of the constructed
-row vector.
+  row vector to contruct. \param ncols : Number of columns of the constructed
+  row vector.
 
   The sub-vector starting from v[c] element and ending on v[c+ncols-1] element
   is used to initialize the contructed row vector.
