@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -81,7 +81,7 @@ const double vpMath::ang_min_mc = 2.5e-4;
    \param value : Double number to check.
    \return Return true if value is not a number.
  */
-bool vpMath::isNaN(const double value)
+bool vpMath::isNaN(double value)
 {
 #if defined(VISP_HAVE_FUNC_ISNAN)
   return isnan(value);
@@ -107,7 +107,7 @@ bool vpMath::isNaN(const double value)
    negative infinity). \param value : Double number to check. \return Return
    true if value is infinity.
  */
-bool vpMath::isInf(const double value)
+bool vpMath::isInf(double value)
 {
 #if defined(VISP_HAVE_FUNC_ISINF)
   return isinf(value);
@@ -249,7 +249,7 @@ double vpMath::getMedian(const std::vector<double> &v)
 
   \return The standard deviation value.
 */
-double vpMath::getStdev(const std::vector<double> &v, const bool useBesselCorrection)
+double vpMath::getStdev(const std::vector<double> &v, bool useBesselCorrection)
 {
   if (v.empty()) {
     throw vpException(vpException::notInitialized, "Empty vector !");
@@ -258,7 +258,12 @@ double vpMath::getStdev(const std::vector<double> &v, const bool useBesselCorrec
   double mean = getMean(v);
 
   std::vector<double> diff(v.size());
+#if VISP_CXX_STANDARD > VISP_CXX_STANDARD_98
+  std::transform(v.begin(), v.end(), diff.begin(), std::bind(std::minus<double>(), std::placeholders::_1, mean));
+#else
   std::transform(v.begin(), v.end(), diff.begin(), std::bind2nd(std::minus<double>(), mean));
+#endif
+
   double sq_sum = std::inner_product(diff.begin(), diff.end(), diff.begin(), 0.0);
   double divisor = (double)v.size();
   if (useBesselCorrection && v.size() > 1) {
@@ -278,4 +283,4 @@ double vpMath::getStdev(const std::vector<double> &v, const bool useBesselCorrec
 
   \return The modified modulo of a mod n.
 */
-int vpMath::modulo(const int a, const int n) { return ((a % n) + n) % n; }
+int vpMath::modulo(int a, int n) { return ((a % n) + n) % n; }
