@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -198,8 +198,7 @@ vpHomogeneousMatrix::vpHomogeneousMatrix(const std::vector<double> &v) : vpArray
   t_y, t_z)^T\f$ and a \f$\theta {\bf u}=(\theta u_x, \theta u_y, \theta
   u_z)^T\f$ rotation vector.
  */
-vpHomogeneousMatrix::vpHomogeneousMatrix(const double tx, const double ty, const double tz, const double tux,
-                                         const double tuy, const double tuz)
+vpHomogeneousMatrix::vpHomogeneousMatrix(double tx, double ty, double tz, double tux, double tuy, double tuz)
   : vpArray2D<double>(4, 4)
 {
   buildFrom(tx, ty, tz, tux, tuy, tuz);
@@ -253,8 +252,7 @@ void vpHomogeneousMatrix::buildFrom(const vpTranslationVector &t, const vpQuater
   t_z)^T\f$ and a \f$\theta {\bf u}=(\theta u_x, \theta u_y, \theta u_z)^T\f$
   rotation vector.
  */
-void vpHomogeneousMatrix::buildFrom(const double tx, const double ty, const double tz, const double tux,
-                                    const double tuy, const double tuz)
+void vpHomogeneousMatrix::buildFrom(double tx, double ty, double tz, double tux, double tuy, double tuz)
 {
   vpRotationMatrix R(tux, tuy, tuz);
   vpTranslationVector t(tx, ty, tz);
@@ -748,7 +746,7 @@ void vpHomogeneousMatrix::load(std::ifstream &f)
       }
     }
   } else {
-    throw(vpException(vpException::ioError, "Cannot laad homogeneous matrix: ifstream not open"));
+    throw(vpException(vpException::ioError, "Cannot load homogeneous matrix: ifstream not open"));
   }
 }
 
@@ -840,7 +838,7 @@ Last column:
 0
   \endcode
  */
-vpColVector vpHomogeneousMatrix::getCol(const unsigned int j) const
+vpColVector vpHomogeneousMatrix::getCol(unsigned int j) const
 {
   if (j >= getCols())
     throw(vpException(vpException::dimensionError, "Unable to extract a column vector from the homogeneous matrix"));
@@ -851,6 +849,50 @@ vpColVector vpHomogeneousMatrix::getCol(const unsigned int j) const
   return c;
 }
 
+/*!
+  Compute the Euclidean mean of the homogeneous matrices.
+  The Euclidean mean of the rotation matrices is computed following Moakher's method (SIAM 2002).
+
+  \param[in] vec_M : Set of homogeneous matrices.
+  \return The Euclidian mean of the homogeneous matrices.
+
+  \sa vpTranslationVector::mean(), vpRotationMatrix::mean()
+ */
+vpHomogeneousMatrix vpHomogeneousMatrix::mean(const std::vector<vpHomogeneousMatrix> &vec_M)
+{
+  vpMatrix meanR(3, 3);
+  vpColVector meanT(3);
+  vpRotationMatrix R;
+  for (size_t i = 0; i < vec_M.size(); i++) {
+    R = vec_M[i].getRotationMatrix();
+    meanR += (vpMatrix) R;
+    meanT += (vpColVector) vec_M[i].getTranslationVector();
+  }
+  meanR /= static_cast<double>(vec_M.size());
+  meanT /= static_cast<double>(vec_M.size());
+
+  // Euclidean mean of the rotation matrix following Moakher's method (SIAM 2002)
+  vpMatrix M, U, V;
+  vpColVector sv;
+  meanR.pseudoInverse(M, sv, 1e-6, U, V);
+  double det = sv[0]*sv[1]*sv[2];
+  if (det > 0) {
+    meanR = U * V.t();
+  }
+  else {
+    vpMatrix D(3,3);
+    D = 0.0;
+    D[0][0] = D[1][1] = 1.0; D[2][2] = -1;
+    meanR = U * D * V.t();
+  }
+
+  R = meanR;
+
+  vpTranslationVector t(meanT);
+  vpHomogeneousMatrix mean(t, R);
+  return mean;
+}
+
 #if defined(VISP_BUILD_DEPRECATED_FUNCTIONS)
 
 /*!
