@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -80,8 +80,7 @@ vpPoseVector::vpPoseVector() : vpArray2D<double>(6, 1) {}
   u_z]^\top\f$ respectively around the x, y and z axis (in radians).
 
 */
-vpPoseVector::vpPoseVector(const double tx, const double ty, const double tz, const double tux, const double tuy,
-                           const double tuz)
+vpPoseVector::vpPoseVector(double tx, double ty, double tz, double tux, double tuy, double tuz)
   : vpArray2D<double>(6, 1)
 {
   (*this)[0] = tx;
@@ -152,8 +151,7 @@ vpPoseVector::vpPoseVector(const vpHomogeneousMatrix &M) : vpArray2D<double>(6,
   u_z]^\top\f$ respectively around the x, y and z axis (in radians).
 
 */
-void vpPoseVector::set(const double tx, const double ty, const double tz, const double tux, const double tuy,
-                       const double tuz)
+void vpPoseVector::set(double tx, double ty, double tz, double tux, double tuy, double tuz)
 {
   (*this)[0] = tx;
   (*this)[1] = ty;
@@ -180,8 +178,7 @@ void vpPoseVector::set(const double tx, const double ty, const double tz, const
 
   \sa set()
 */
-vpPoseVector vpPoseVector::buildFrom(const double tx, const double ty, const double tz, const double tux,
-                                     const double tuy, const double tuz)
+vpPoseVector vpPoseVector::buildFrom(double tx, double ty, double tz, double tux, double tuy, double tuz)
 {
   (*this)[0] = tx;
   (*this)[1] = ty;
