@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,7 +54,7 @@
   in meters.
 
 */
-vpTranslationVector::vpTranslationVector(const double tx, const double ty, const double tz) : vpArray2D<double>(3, 1)
+vpTranslationVector::vpTranslationVector(double tx, double ty, double tz) : vpArray2D<double>(3, 1), m_index(0)
 {
   (*this)[0] = tx;
   (*this)[1] = ty;
@@ -68,7 +68,7 @@ vpTranslationVector::vpTranslationVector(const double tx, const double ty, const
   \param M : Homogeneous matrix where translations are in meters.
 
 */
-vpTranslationVector::vpTranslationVector(const vpHomogeneousMatrix &M) : vpArray2D<double>(3, 1) { M.extract(*this); }
+vpTranslationVector::vpTranslationVector(const vpHomogeneousMatrix &M) : vpArray2D<double>(3, 1), m_index(0) { M.extract(*this); }
 
 /*!
   Construct a translation vector \f$ \bf t \f$ from the translation contained
@@ -77,7 +77,7 @@ vpTranslationVector::vpTranslationVector(const vpHomogeneousMatrix &M) : vpArray
   \param p : Pose vector where translations are in meters.
 
 */
-vpTranslationVector::vpTranslationVector(const vpPoseVector &p) : vpArray2D<double>(3, 1)
+vpTranslationVector::vpTranslationVector(const vpPoseVector &p) : vpArray2D<double>(3, 1), m_index(0)
 {
   (*this)[0] = p[0];
   (*this)[1] = p[1];
@@ -94,7 +94,7 @@ vpTranslationVector::vpTranslationVector(const vpPoseVector &p) : vpArray2D<doub
   vpTranslationVector t2(t1);    // t2 is now a copy of t1
   \endcode
 */
-vpTranslationVector::vpTranslationVector(const vpTranslationVector &tv) : vpArray2D<double>(tv) {}
+vpTranslationVector::vpTranslationVector(const vpTranslationVector &tv) : vpArray2D<double>(tv), m_index(0) {}
 
 /*!
   Construct a translation vector \f$ \bf t \f$ from a 3-dimension column
@@ -110,7 +110,7 @@ vpTranslationVector::vpTranslationVector(const vpTranslationVector &tv) : vpArra
   \endcode
 
 */
-vpTranslationVector::vpTranslationVector(const vpColVector &v) : vpArray2D<double>(v)
+vpTranslationVector::vpTranslationVector(const vpColVector &v) : vpArray2D<double>(v), m_index(0)
 {
   if (v.size() != 3) {
     throw(vpException(vpException::dimensionError,
@@ -184,7 +184,7 @@ vpTranslationVector vpTranslationVector::buildFrom(const vpColVector &v)
   \return The build translation vector.
   \sa set()
 */
-vpTranslationVector vpTranslationVector::buildFrom(const double tx, const double ty, const double tz)
+vpTranslationVector vpTranslationVector::buildFrom(double tx, double ty, double tz)
 {
   set(tx, ty, tz);
   return *this;
@@ -196,7 +196,7 @@ vpTranslationVector vpTranslationVector::buildFrom(const double tx, const double
   \param tx,ty,tz : Translation respectively along x, y and z axis in meter.
 
 */
-void vpTranslationVector::set(const double tx, const double ty, const double tz)
+void vpTranslationVector::set(double tx, double ty, double tz)
 {
   (*this)[0] = tx;
   (*this)[1] = ty;
@@ -333,7 +333,7 @@ vpTranslationVector vpTranslationVector::operator-() const // negate
   // t2 is now equal to : [3, 6, 9]
   \endcode
 */
-vpTranslationVector vpTranslationVector::operator*(const double x) const
+vpTranslationVector vpTranslationVector::operator*(double x) const
 {
   vpTranslationVector tv;
   for (unsigned int i = 0; i < dsize; i++) {
@@ -408,7 +408,7 @@ vpTranslationVector &vpTranslationVector::operator/=(double x)
   // t2 is now equal to : [4, 2, 1]
   \endcode
 */
-vpTranslationVector vpTranslationVector::operator/(const double x) const
+vpTranslationVector vpTranslationVector::operator/(double x) const
 {
   vpTranslationVector tv;
   for (unsigned int i = 0; i < dsize; i++) {
@@ -505,6 +505,97 @@ vpTranslationVector &vpTranslationVector::operator=(double x)
   return *this;
 }
 
+#if (VISP_CXX_STANDARD >= VISP_CXX_STANDARD_11)
+/*!
+  Set vector from a list of 3 double values in meters.
+  \code
+#include <visp3/core/vpTranslationVector.cpp>
+
+int main()
+{
+  vpTranslationVector t = {0, 0.1, 0.5};
+  std::cout << "t: " << t.t() << std::endl;
+}
+  \endcode
+  It produces the following printings:
+  \code
+t: 0  0.1  0.5
+  \endcode
+  \sa operator<<()
+*/
+vpTranslationVector &vpTranslationVector::operator=(const std::initializer_list<double> &list)
+{
+  if (list.size() > size()) {
+    throw(vpException(vpException::dimensionError, "Cannot set translation vector out of bounds. It has only %d values while you try to initialize with %d values", size(), list.size()));
+  }
+  std::copy(list.begin(), list.end(), data);
+  return *this;
+}
+#endif
+
+/*!
+  Set vector first element value.
+  \param val : Value of the vector first element [meter].
+  \return An updated vector.
+
+  The following example shows how to initialize a translation vector from a list of 3 values [meter].
+  \code
+#include <visp3/core/vpTranslationVector.h>
+
+int main()
+{
+  vpTranslationVector t;
+  t << 0, 0.1, 0.5;
+  std::cout << "t: " << t.t() << std::endl;
+}
+  \endcode
+  It produces the following printings:
+  \code
+t: 0  0.1  0.5
+  \endcode
+
+  \sa operator,()
+ */
+vpTranslationVector& vpTranslationVector::operator<<(double val)
+{
+  m_index = 0;
+  data[m_index] = val;
+  return *this;
+}
+
+/*!
+  Set vector second and third element values.
+  \param val : Value of the vector element [meter].
+  \return An updated vector.
+
+  The following example shows how to initialize a translations vector from a list of 3 values [meter].
+  \code
+#include <visp3/core/vpTranslationVector.h>
+
+int main()
+{
+  vpTranslationVector t;
+  t << 0, 0.1, 0.5;
+  std::cout << "t: " << t.t() << std::endl;
+}
+  \endcode
+  It produces the following printings:
+  \code
+t: 0  0.1  0.5
+  \endcode
+
+  \sa operator<<()
+ */
+vpTranslationVector& vpTranslationVector::operator,(double val)
+{
+  m_index ++;
+  if (m_index >= size()) {
+    throw(vpException(vpException::dimensionError, "Cannot set translation vector out of bounds. It has only %d values while you try to initialize with %d values", size(), m_index+1));
+  }
+  data[m_index] = val;
+  return *this;
+}
+
 /*!
   Compute the skew symmetric matrix \f$M\f$ of translation vector \e tv.
 
@@ -612,19 +703,28 @@ vpRowVector vpTranslationVector::t() const
 }
 
 /*!
-  Compute and return the Euclidean norm of the translation vector
-  \f$ ||x|| = \sqrt{ \sum{t_{i}^2}} \f$.
+  \deprecated This function is deprecated. You should rather use frobeniusNorm().
+
+  Compute and return the Euclidean norm also called Fronebius nom of the translation vector
+  \f$ ||t|| = \sqrt{ \sum{t_{i}^2}} \f$.
 
   \return The Euclidean norm if the vector is initialized, 0 otherwise.
 
+  \sa frobeniusNorm()
 */
 double vpTranslationVector::euclideanNorm() const
 {
-  double norm = 0.0;
-  for (unsigned int i = 0; i < dsize; i++) {
-    double x = *(data + i);
-    norm += x * x;
-  }
+  return frobeniusNorm();
+}
+
+/*!
+  Compute and return the Fronebius norm \f$ ||t|| = \sqrt{ \sum{t_{i}^2}} \f$.
+
+  \return The Fronebius norm if the vector is initialized, 0 otherwise.
+*/
+double vpTranslationVector::frobeniusNorm() const
+{
+  double norm = sumSquare();
 
   return sqrt(norm);
 }
@@ -646,3 +746,43 @@ double vpTranslationVector::sumSquare() const
 
   return sum_square;
 }
+
+/*!
+  Compute the Euclidean mean of the translation vector extracted from a vector of homogeneous matrices.
+
+  \param[in] vec_M : Set of homogeneous matrices.
+  \return The Euclidian mean of the translation vectors.
+
+  \sa vpRotationMatrix::mean()
+ */
+vpTranslationVector vpTranslationVector::mean(const std::vector<vpHomogeneousMatrix> &vec_M)
+{
+  vpColVector meanT(3);
+  for (size_t i = 0; i < vec_M.size(); i++) {
+    meanT += (vpColVector) vec_M[i].getTranslationVector();
+  }
+  meanT /= static_cast<double>(vec_M.size());
+
+  vpTranslationVector t(meanT);
+  return t;
+}
+
+/*!
+  Compute the Euclidean mean of a vector of translation vector.
+
+  \param[in] vec_t : Set of translation vectors.
+  \return The Euclidian mean of the translation vectors.
+
+  \sa vpRotationMatrix::mean()
+ */
+vpTranslationVector vpTranslationVector::mean(const std::vector<vpTranslationVector> &vec_t)
+{
+  vpColVector meanT(3);
+  for (size_t i = 0; i < vec_t.size(); i++) {
+    meanT += (vpColVector) vec_t[i];
+  }
+  meanT /= static_cast<double>(vec_t.size());
+
+  vpTranslationVector t(meanT);
+  return t;
+}
