@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,7 +61,7 @@ vpException::vpException(int id, const char *format, ...) : code(id), message()
   va_end(args);
 }
 
-vpException::vpException(const int id, const char *format, va_list args) : code(id), message()
+vpException::vpException(int id, const char *format, va_list args) : code(id), message()
 {
   setMessage(format, args);
 }
