@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -116,7 +116,7 @@ vpPoint &vpPolygon3D::getPoint(const unsigned int _index)
 
   \param nb : The number of corners.
 */
-void vpPolygon3D::setNbPoint(const unsigned int nb)
+void vpPolygon3D::setNbPoint(unsigned int nb)
 {
   nbpt = nb;
   if (p != NULL)
@@ -130,7 +130,7 @@ void vpPolygon3D::setNbPoint(const unsigned int nb)
   \param n : The index of the corner.
   \param P : The point to add.
 */
-void vpPolygon3D::addPoint(const unsigned int n, const vpPoint &P)
+void vpPolygon3D::addPoint(unsigned int n, const vpPoint &P)
 {
   // if( p!NULL && n < nbpt )
   p[n] = P;
