@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -248,9 +248,10 @@ bool vpRect::operator!=(const vpRect &r) const
 }
 
 /*!
- Intersection operator.
- \return Intersection rectangle or null rectangle if the two rectangles do not
- intersect.
+  Intersection operator.
+  \param r : Rectangle to insert.
+  \return Intersection rectangle or null rectangle if the two rectangles do not
+  intersect.
  */
 vpRect &vpRect::operator&=(const vpRect &r)
 {
@@ -269,9 +270,9 @@ vpRect &vpRect::operator&=(const vpRect &r)
 }
 
 /*!
- Intersection operator.
- \return Intersection rectangle or null rectangle if the two rectangles do not
- intersect.
+  Intersection operator.
+  \return Intersection rectangle or null rectangle if the two rectangles do not
+  intersect.
  */
 vpRect vpRect::operator&(const vpRect &r) const
 {
