@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -155,7 +155,7 @@ bool getOptions(int argc, const char **argv, std::string &ipath, std::string &op
 }
 
 void regularImageAdd(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2, vpImage<unsigned char> &Ires,
-                     const bool saturate)
+                     bool saturate)
 {
   if ((I1.getHeight() != I2.getHeight()) || (I1.getWidth() != I2.getWidth())) {
     throw(vpException(vpException::dimensionError, "The two images do not have the same size"));
@@ -175,7 +175,7 @@ void regularImageAdd(const vpImage<unsigned char> &I1, const vpImage<unsigned ch
 }
 
 void regularImageSubtract(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2,
-                          vpImage<unsigned char> &Ires, const bool saturate)
+                          vpImage<unsigned char> &Ires, bool saturate)
 {
   if ((I1.getHeight() != I2.getHeight()) || (I1.getWidth() != I2.getWidth())) {
     throw(vpException(vpException::dimensionError, "The two images do not have the same size"));
