@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -286,7 +286,7 @@ void generalDilatation(vpImage<unsigned char> &I,
 }
 
 // Generate a magic square matrix to get a consistent grayscale image
-void magicSquare(vpImage<unsigned char> &magic_square, const int N)
+void magicSquare(vpImage<unsigned char> &magic_square, int N)
 {
   magic_square.resize((unsigned int)N, (unsigned int)N, 0);
 
@@ -871,7 +871,7 @@ int main(int argc, const char **argv)
 
 #endif
 
-  } catch (vpException &e) {
+  } catch (const vpException &e) {
     std::cout << "\nCatch an exception: " << e << std::endl;
     return EXIT_FAILURE;
   }
