@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -166,7 +166,7 @@ bool getOptions(int argc, const char **argv, std::string &ipath, std::string &op
   \param alpha : Gain.
   \param beta: Offset.
 */
-void iterate_method1(vpImage<vpRGBa> &I, const double alpha, const double beta)
+void iterate_method1(vpImage<vpRGBa> &I, double alpha, double beta)
 {
   unsigned int size = I.getWidth() * I.getHeight();
   unsigned char *ptrStart = (unsigned char *)I.bitmap;
@@ -187,7 +187,7 @@ void iterate_method1(vpImage<vpRGBa> &I, const double alpha, const double beta)
   \param alpha : Gain.
   \param beta: Offset.
 */
-void iterate_method1(vpImage<unsigned char> &I, const double alpha, const double beta)
+void iterate_method1(vpImage<unsigned char> &I, double alpha, double beta)
 {
   unsigned int size = I.getWidth() * I.getHeight();
   unsigned char *ptrStart = (unsigned char *)I.bitmap;
@@ -208,7 +208,7 @@ void iterate_method1(vpImage<unsigned char> &I, const double alpha, const double
   \param alpha : Gain.
   \param beta: Offset.
 */
-void iterate_method2(vpImage<vpRGBa> &I, const double alpha, const double beta)
+void iterate_method2(vpImage<vpRGBa> &I, double alpha, double beta)
 {
   for (unsigned int i = 0; i < I.getHeight(); i++) {
     for (unsigned int j = 0; j < I.getWidth(); j++) {
@@ -526,7 +526,7 @@ int main(int argc, const char **argv)
     I_test_color.performLut(lut_color, nbThreads);
 
     return 0;
-  } catch (vpException &e) {
+  } catch (const vpException &e) {
     std::cerr << "Catch an exception: " << e.what() << std::endl;
     return 1;
   }
