@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -73,12 +73,12 @@ bool test(const std::string &s, const vpMatrix &M, const std::vector<double> &be
   return true;
 }
 
-double getRandomValues(const double min, const double max)
+double getRandomValues(double min, double max)
 {
   return (max - min) * ((double)rand() / (double)RAND_MAX) + min;
 }
 
-bool equalMatrix(const vpMatrix &A, const vpMatrix &B, const double tol = std::numeric_limits<double>::epsilon())
+bool equalMatrix(const vpMatrix &A, const vpMatrix &B, double tol = std::numeric_limits<double>::epsilon())
 {
   if (A.getRows() != B.getRows() || A.getCols() != B.getCols()) {
     return false;
@@ -95,8 +95,7 @@ bool equalMatrix(const vpMatrix &A, const vpMatrix &B, const double tol = std::n
   return true;
 }
 
-#if defined(VISP_HAVE_LAPACK) && !defined(VISP_HAVE_LAPACK_BUILT_IN)
-vpMatrix generateRandomMatrix(const unsigned int rows, const unsigned int cols, const double min, const double max)
+vpMatrix generateRandomMatrix(unsigned int rows, unsigned int cols, double min, double max)
 {
   vpMatrix M(rows, cols);
 
@@ -108,149 +107,61 @@ vpMatrix generateRandomMatrix(const unsigned int rows, const unsigned int cols,
 
   return M;
 }
-
-vpColVector generateRandomVector(const unsigned int rows, const double min, const double max)
-{
-  vpColVector v(rows);
-
-  for (unsigned int i = 0; i < v.getRows(); i++) {
-    v[i] = getRandomValues(min, max);
-  }
-
-  return v;
 }
 
-// Copy of vpMatrix::mult2Matrices
-vpMatrix dgemm_regular(const vpMatrix &A, const vpMatrix &B)
-{
-  vpMatrix C;
-
-  if ((A.getRows() != C.getRows()) || (B.getCols() != C.getCols()))
-    C.resize(A.getRows(), B.getCols(), false);
-
-  if (A.getCols() != B.getRows()) {
-    throw(vpException(vpException::dimensionError, "Cannot multiply (%dx%d) matrix by (%dx%d) matrix", A.getRows(),
-                      A.getCols(), B.getRows(), B.getCols()));
-  }
-
-  // 5/12/06 some "very" simple optimization to avoid indexation
-  unsigned int BcolNum = B.getCols();
-  unsigned int BrowNum = B.getRows();
-  unsigned int i, j, k;
-  for (i = 0; i < A.getRows(); i++) {
-    double *rowptri = A[i];
-    double *ci = C[i];
-    for (j = 0; j < BcolNum; j++) {
-      double s = 0;
-      for (k = 0; k < BrowNum; k++)
-        s += rowptri[k] * B[k][j];
-      ci[j] = s;
-    }
-  }
-
-  return C;
-}
-
-// Copy of vpMatrix::AtA
-vpMatrix AtA_regular(const vpMatrix &A)
-{
-  vpMatrix B;
-  B.resize(A.getCols(), A.getCols(), false);
-
-  unsigned int i, j, k;
-  double s;
-  double *ptr;
-  for (i = 0; i < A.getCols(); i++) {
-    double *Bi = B[i];
-    for (j = 0; j < i; j++) {
-      ptr = A.data;
-      s = 0;
-      for (k = 0; k < A.getRows(); k++) {
-        s += (*(ptr + i)) * (*(ptr + j));
-        ptr += A.getCols();
-      }
-      *Bi++ = s;
-      B[j][i] = s;
-    }
-    ptr = A.data;
-    s = 0;
-    for (k = 0; k < A.getRows(); k++) {
-      s += (*(ptr + i)) * (*(ptr + i));
-      ptr += A.getCols();
-    }
-    *Bi = s;
-  }
-
-  return B;
-}
-
-// Copy of vpMatrix::multMatrixVector
-vpMatrix dgemv_regular(const vpMatrix &A, const vpColVector &v)
-{
-  vpColVector w;
-
-  if (A.getCols() != v.getRows()) {
-    throw(vpException(vpException::dimensionError, "Cannot multiply a (%dx%d) matrix by a (%d) column vector",
-                      A.getRows(), A.getCols(), v.getRows()));
-  }
-
-  w.resize(A.getRows(), true);
-
-  for (unsigned int j = 0; j < A.getCols(); j++) {
-    double vj = v[j]; // optimization em 5/12/2006
-    for (unsigned int i = 0; i < A.getRows(); i++) {
-      w[i] += A[i][j] * vj;
-    }
-  }
-
-  return w;
-}
-
-// Copy of vpMatrix::operator*(const vpVelocityTwistMatrix &V)
-vpMatrix mat_mul_twist_matrix(const vpMatrix &A, const vpVelocityTwistMatrix &V)
-{
-  vpMatrix M;
-
-  if (A.getCols() != V.getRows()) {
-    throw(vpException(vpException::dimensionError, "Cannot multiply (%dx%d) matrix by (6x6) velocity twist matrix",
-                      A.getRows(), A.getCols()));
-  }
-
-  M.resize(A.getRows(), 6, false);
-
-  unsigned int VcolNum = V.getCols();
-  unsigned int VrowNum = V.getRows();
-
-  for (unsigned int i = 0; i < A.getRows(); i++) {
-    double *rowptri = A[i];
-    double *ci = M[i];
-    for (unsigned int j = 0; j < VcolNum; j++) {
-      double s = 0;
-      for (unsigned int k = 0; k < VrowNum; k++)
-        s += rowptri[k] * V[k][j];
-      ci[j] = s;
-    }
-  }
-
-  return M;
-}
-#endif
-}
-
-int main()
+int main(int argc, char *argv[])
 {
   try {
-    int err = 1;
+    bool ctest = true;
+    for (int i = 1; i < argc; i++) {
+      if (std::string(argv[i]) == "--benchmark") {
+        ctest = false;
+      }
+    }
+
+    {
+      const double val = 10.0;
+      vpMatrix M, M2(5, 5, val);
+      M.resize(5, 5, false, false);
+      M = val;
+      for (unsigned int i = 0; i < M.getRows(); i++) {
+        for (unsigned int j = 0; j < M.getCols(); j++) {
+          if (!vpMath::equal(M[i][j], val, std::numeric_limits<double>::epsilon())) {
+            std::cerr << "Issue with matrix assignment with value." << std::endl;
+            return EXIT_FAILURE;
+          }
+
+          if (!vpMath::equal(M2[i][j], val, std::numeric_limits<double>::epsilon())) {
+            std::cerr << "Issue with matrix constructor initialized with value." << std::endl;
+            return EXIT_FAILURE;
+          }
+        }
+      }
+    }
+    {
+      // Test vpRotationMatrix construction
+      std::vector<double> bench(9, 0);
+      bench[2] = bench[4] = bench[6] = 1.;
+      vpMatrix M(3, 3);
+      M[2][0] = M[1][1] = M[0][2] = 1.;
+      vpRotationMatrix R1(M);
+      if (test("R1", R1, bench) == false)
+        return EXIT_FAILURE;
+      vpRotationMatrix R2;
+      R2 = M;
+      if (test("R2", R2, bench) == false)
+        return EXIT_FAILURE;
+    }
     {
       vpColVector c(6, 1);
       vpRowVector r(6, 1);
       std::vector<double> bench(6, 1);
       vpMatrix M1(c);
       if (test("M1", M1, bench) == false)
-        return err;
+        return EXIT_FAILURE;
       vpMatrix M2(r);
       if (test("M2", M2, bench) == false)
-        return err;
+        return EXIT_FAILURE;
     }
     {
       vpMatrix M(4, 5);
@@ -273,7 +184,7 @@ int main()
       if (vpMatrix::saveMatrix("matrix.mat", M, false, header.c_str()))
         std::cout << "Matrix saved in matrix.mat file" << std::endl;
       else
-        return err;
+        return EXIT_FAILURE;
 
       // Load matrix in text format
       vpMatrix M1;
@@ -281,43 +192,43 @@ int main()
       if (vpMatrix::loadMatrix("matrix.mat", M1, false, header_))
         std::cout << "Matrix loaded from matrix.mat file with header \"" << header_ << "\": \n" << M1 << std::endl;
       else
-        return err;
+        return EXIT_FAILURE;
       if (header != std::string(header_)) {
         std::cout << "Bad header in matrix.mat" << std::endl;
-        return err;
+        return EXIT_FAILURE;
       }
 
       // Save matrix in binary format
       if (vpMatrix::saveMatrix("matrix.bin", M, true, header.c_str()))
         std::cout << "Matrix saved in matrix.bin file" << std::endl;
       else
-        return err;
+        return EXIT_FAILURE;
 
       // Load matrix in binary format
       if (vpMatrix::loadMatrix("matrix.bin", M1, true, header_))
         std::cout << "Matrix loaded from matrix.bin file with header \"" << header_ << "\": \n" << M1 << std::endl;
       else
-        return err;
+        return EXIT_FAILURE;
       if (header != std::string(header_)) {
         std::cout << "Bad header in matrix.bin" << std::endl;
-        return err;
+        return EXIT_FAILURE;
       }
 
       // Save matrix in YAML format
       if (vpMatrix::saveMatrixYAML("matrix.yml", M, header.c_str()))
         std::cout << "Matrix saved in matrix.yml file" << std::endl;
       else
-        return err;
+        return EXIT_FAILURE;
 
       // Read matrix in YAML format
       vpMatrix M2;
       if (vpMatrix::loadMatrixYAML("matrix.yml", M2, header_))
         std::cout << "Matrix loaded from matrix.yml file with header \"" << header_ << "\": \n" << M2 << std::endl;
       else
-        return err;
+        return EXIT_FAILURE;
       if (header != std::string(header_)) {
         std::cout << "Bad header in matrix.mat" << std::endl;
-        return err;
+        return EXIT_FAILURE;
       }
     }
 
@@ -471,9 +382,8 @@ int main()
       std::cout << "------------------------" << std::endl;
       std::cout << "--- TEST vpMatrix insert() with same colNum " << std::endl;
       std::cout << "------------------------" << std::endl;
-      const unsigned int nb = 100; // 10000; //for ctest otherwise takes too
-                                   // long time with static call
-      const unsigned int size = 100;
+      unsigned int nb = ctest ? 10 : 100; // 10000;
+      const unsigned int size = ctest ? 10 : 100;
 
       vpMatrix m_big(nb * size, 6);
       std::vector<vpMatrix> submatrices(nb);
@@ -518,7 +428,7 @@ int main()
       std::cout << "m2:\n" << m2 << std::endl;
       std::cout << "m3:\n" << m3 << std::endl;
 
-      std::cout << "------------------------" << std::endl;
+      std::cout << "\n------------------------" << std::endl;
       std::cout << "--- TEST vpMatrix stack()" << std::endl;
       std::cout << "------------------------" << std::endl;
 
@@ -533,37 +443,59 @@ int main()
         std::cout << "L:\n" << L << std::endl;
       }
 
-      vpMatrix m_big_stack;
-      t = vpTime::measureTimeMs();
-      for (unsigned int i = 0; i < nb; i++) {
-        m_big_stack.stack(submatrices[(size_t)i]);
-      }
-      t = vpTime::measureTimeMs() - t;
-      std::cout << "\nMatrix stack(): " << t << " ms" << std::endl;
+      {
+        vpMatrix m_big_stack;
+        t = vpTime::measureTimeMs();
+        for (unsigned int i = 0; i < nb; i++) {
+          m_big_stack.stack(submatrices[(size_t)i]);
+        }
+        t = vpTime::measureTimeMs() - t;
+        std::cout << "Matrix stack(): " << t << " ms" << std::endl;
 
-      if (!equalMatrix(m_big, m_big_stack)) {
-        std::cerr << "Problem with vpMatrix stack()!" << std::endl;
-        return EXIT_FAILURE;
+        if (!equalMatrix(m_big, m_big_stack)) {
+          std::cerr << "Problem with vpMatrix stack()!" << std::endl;
+          return EXIT_FAILURE;
+        }
       }
 
-      std::cout << "------------------------" << std::endl;
+      std::cout << "\n------------------------" << std::endl;
       std::cout << "--- TEST vpMatrix stack(vpRowVector)" << std::endl;
       std::cout << "------------------------" << std::endl;
 
+      vpMatrix m_big_stack = generateRandomMatrix(10000, ctest ? 10 : 100, -1000.0, 1000.0);
+      std::cout << "m_big_stack: " << m_big_stack.getRows() << "x" << m_big_stack.getCols() << std::endl;
+
       vpMatrix m_big_stack_row;
       t = vpTime::measureTimeMs();
       for (unsigned int i = 0; i < m_big_stack.getRows(); i++) {
         m_big_stack_row.stack(m_big_stack.getRow(i));
       }
       t = vpTime::measureTimeMs() - t;
-      std::cout << "\nMatrix stack(vpRowVector): " << t << " ms" << std::endl;
+      std::cout << "Matrix stack(vpRowVector): " << t << " ms" << std::endl;
 
       if (!equalMatrix(m_big_stack, m_big_stack_row)) {
         std::cerr << "Problem with vpMatrix stack(vpRowVector)!" << std::endl;
         return EXIT_FAILURE;
       }
 
+      std::cout << "\n------------------------" << std::endl;
+      std::cout << "--- TEST vpMatrix stack(vpColVector)" << std::endl;
       std::cout << "------------------------" << std::endl;
+
+      vpMatrix m_big_stack_col;
+      t = vpTime::measureTimeMs();
+      for (unsigned int j = 0; j < m_big_stack.getCols(); j++) {
+        m_big_stack_col.stack(m_big_stack.getCol(j));
+      }
+      t = vpTime::measureTimeMs() - t;
+      std::cout << "Matrix stack(vpColVector): " << t << " ms" << std::endl;
+
+      if (!equalMatrix(m_big_stack, m_big_stack_col)) {
+        std::cerr << "Problem with vpMatrix stack(vpColVector)!" << std::endl;
+        return EXIT_FAILURE;
+      }
+
+      std::cout << "\n------------------------" << std::endl;
       std::cout << "--- TEST vpMatrix::stack()" << std::endl;
       std::cout << "------------------------" << std::endl;
 
@@ -579,24 +511,29 @@ int main()
         std::cout << "L:\n" << L << std::endl;
       }
 
-      vpMatrix m_big_stack_static, m_big_stack_static_tmp;
-      t = vpTime::measureTimeMs();
-      for (unsigned int i = 0; i < nb; i++) {
-        vpMatrix::stack(m_big_stack_static_tmp, submatrices[(size_t)i], m_big_stack_static);
-        m_big_stack_static_tmp = m_big_stack_static;
-      }
-      t = vpTime::measureTimeMs() - t;
-      std::cout << "\nMatrix::stack(): " << t << " ms" << std::endl;
+      {
+        vpMatrix m_big_stack_static, m_big_stack_static_tmp;
+        t = vpTime::measureTimeMs();
+        for (unsigned int i = 0; i < nb; i++) {
+          vpMatrix::stack(m_big_stack_static_tmp, submatrices[(size_t)i], m_big_stack_static);
+          m_big_stack_static_tmp = m_big_stack_static;
+        }
+        t = vpTime::measureTimeMs() - t;
+        std::cout << "Matrix::stack(): " << t << " ms" << std::endl;
 
-      if (!equalMatrix(m_big, m_big_stack_static)) {
-        std::cerr << "Problem with vpMatrix::stack()!" << std::endl;
-        return EXIT_FAILURE;
+        if (!equalMatrix(m_big, m_big_stack_static)) {
+          std::cerr << "Problem with vpMatrix::stack()!" << std::endl;
+          return EXIT_FAILURE;
+        }
       }
 
-      std::cout << "------------------------" << std::endl;
+      std::cout << "\n------------------------" << std::endl;
       std::cout << "--- TEST vpMatrix::stack(vpMatrix, vpRowVector, vpMatrix)" << std::endl;
       std::cout << "------------------------" << std::endl;
 
+      vpMatrix m_big_stack_static = generateRandomMatrix(ctest ? 100 : 1000, ctest ? 10 : 100, -1000.0, 1000.0);
+      std::cout << "m_big_stack_static: " << m_big_stack_static.getRows() << "x" << m_big_stack_static.getCols() << std::endl;
+
       vpMatrix m_big_stack_static_row, m_big_stack_static_row_tmp;
       t = vpTime::measureTimeMs();
       for (unsigned int i = 0; i < m_big_stack_static.getRows(); i++) {
@@ -604,7 +541,7 @@ int main()
         m_big_stack_static_row_tmp = m_big_stack_static_row;
       }
       t = vpTime::measureTimeMs() - t;
-      std::cout << "\nMatrix::stack(vpMatrix, vpRowVector, vpMatrix): " << t << " ms" << std::endl;
+      std::cout << "Matrix::stack(vpMatrix, vpRowVector, vpMatrix): " << t << " ms" << std::endl;
 
       if (!equalMatrix(m_big_stack_static, m_big_stack_static_row)) {
         std::cerr << "Problem with vpMatrix::stack(vpMatrix, vpRowVector, "
@@ -612,6 +549,26 @@ int main()
                   << std::endl;
         return EXIT_FAILURE;
       }
+
+      std::cout << "\n------------------------" << std::endl;
+      std::cout << "--- TEST vpMatrix::stack(vpMatrix, vpColVector, vpMatrix)" << std::endl;
+      std::cout << "------------------------" << std::endl;
+
+      vpMatrix m_big_stack_static_col, m_big_stack_static_col_tmp;
+      t = vpTime::measureTimeMs();
+      for (unsigned int j = 0; j < m_big_stack_static.getCols(); j++) {
+        vpMatrix::stack(m_big_stack_static_col_tmp, m_big_stack_static.getCol(j), m_big_stack_static_col);
+        m_big_stack_static_col_tmp = m_big_stack_static_col;
+      }
+      t = vpTime::measureTimeMs() - t;
+      std::cout << "Matrix::stack(vpMatrix, vpColVector, vpMatrix): " << t << " ms" << std::endl;
+
+      if (!equalMatrix(m_big_stack_static, m_big_stack_static_col)) {
+        std::cerr << "Problem with vpMatrix::stack(vpMatrix, vpColVector, "
+                     "vpMatrix)!"
+                  << std::endl;
+        return EXIT_FAILURE;
+      }
     }
 
     {
@@ -634,7 +591,8 @@ int main()
         }
       }
 
-      offset_i = 4, offset_j = 5;
+      offset_i = 4;
+      offset_j = 5;
       m1.insert(m2, offset_i, offset_j);
 
       for (unsigned int i = 0; i < m2.getRows(); i++) {
@@ -646,7 +604,8 @@ int main()
         }
       }
 
-      offset_i = 8, offset_j = 5;
+      offset_i = 8;
+      offset_j = 5;
       m1.insert(m2, offset_i, offset_j);
 
       for (unsigned int i = 0; i < m2.getRows(); i++) {
@@ -660,7 +619,7 @@ int main()
     }
 
     {
-      std::cout << "------------------------" << std::endl;
+      std::cout << "\n------------------------" << std::endl;
       std::cout << "--- TEST vpMatrix::juxtaposeMatrices()" << std::endl;
       std::cout << "------------------------" << std::endl;
 
@@ -680,147 +639,7 @@ int main()
       std::cout << "juxtaposeM:\n" << juxtaposeM << std::endl;
     }
 
-#if defined(VISP_HAVE_LAPACK) && !defined(VISP_HAVE_LAPACK_BUILT_IN)
-    {
-      std::cout << "------------------------" << std::endl;
-      std::cout << "--- BENCHMARK dgemm/dgemv" << std::endl;
-      std::cout << "------------------------" << std::endl;
-
-      size_t nb_matrices = 10000;
-      unsigned int rows = 200, cols = 6;
-      double min = -1.0, max = 1.0;
-      std::vector<vpMatrix> vec_A, vec_B, vec_C, vec_C_regular;
-      vec_C.reserve(nb_matrices);
-      vec_C_regular.reserve(nb_matrices);
-
-      for (size_t i = 0; i < nb_matrices; i++) {
-        vec_A.push_back(generateRandomMatrix(cols, rows, min, max));
-        vec_B.push_back(generateRandomMatrix(rows, cols, min, max));
-      }
-
-      double t = vpTime::measureTimeMs();
-      for (size_t i = 0; i < nb_matrices; i++) {
-        vec_C.push_back(vec_A[i] * vec_B[i]);
-      }
-      t = vpTime::measureTimeMs() - t;
-      std::cout << nb_matrices << " matrix multiplication: (6x200) x (200x6)" << std::endl;
-      std::cout << "Lapack: " << t << " ms" << std::endl;
-      std::cout << "vec_C:\n" << vec_C.back() << std::endl;
-
-      t = vpTime::measureTimeMs();
-      for (size_t i = 0; i < nb_matrices; i++) {
-        vec_C_regular.push_back(dgemm_regular(vec_A[i], vec_B[i]));
-      }
-      t = vpTime::measureTimeMs() - t;
-      std::cout << "\nRegular: " << t << " ms" << std::endl;
-      std::cout << "vec_C_regular:\n" << vec_C_regular.back() << std::endl;
-
-      vpMatrix A = generateRandomMatrix(480, 640, min, max), B = generateRandomMatrix(640, 480, min, max);
-      vpMatrix AB, AB_regular;
-
-      t = vpTime::measureTimeMs();
-      AB = A * B;
-      t = vpTime::measureTimeMs() - t;
-      std::cout << "\nMatrix multiplication: (480x640) x (640x480)" << std::endl;
-      std::cout << "Lapack: " << t << " ms" << std::endl;
-      std::cout << "Min=" << AB.getMinValue() << " ; Max=" << AB.getMaxValue() << std::endl;
-
-      t = vpTime::measureTimeMs();
-      AB_regular = dgemm_regular(A, B);
-      t = vpTime::measureTimeMs() - t;
-      std::cout << "Regular: " << t << " ms" << std::endl;
-      std::cout << "Min=" << AB_regular.getMinValue() << " ; Max=" << AB_regular.getMaxValue() << std::endl;
-      bool res = equalMatrix(AB, AB_regular, 1e-9);
-      std::cout << "Check result: " << res << std::endl;
-      if (!res) {
-        std::cerr << "Problem with matrix multiplication!" << std::endl;
-        return EXIT_FAILURE;
-      }
-
-      int nb_iterations = 1000;
-      vpMatrix L = generateRandomMatrix(1000, 6, min, max);
-      vpMatrix LTL, LTL_regular;
-
-      t = vpTime::measureTimeMs();
-      for (int i = 0; i < nb_iterations; i++)
-        LTL = L.AtA();
-      t = vpTime::measureTimeMs() - t;
-      std::cout << "\n" << nb_iterations << " iterations of AtA for size: (1000x6)" << std::endl;
-      std::cout << "Lapack: " << t << " ms" << std::endl;
-      std::cout << "LTL:\n" << LTL << std::endl;
-
-      t = vpTime::measureTimeMs();
-      for (int i = 0; i < nb_iterations; i++)
-        LTL_regular = AtA_regular(L);
-      t = vpTime::measureTimeMs() - t;
-      std::cout << "\nRegular: " << t << " ms" << std::endl;
-      std::cout << "LTL_regular:\n" << LTL_regular << std::endl;
-      res = equalMatrix(LTL, LTL_regular, 1e-9);
-      std::cout << "Check result: " << res << std::endl;
-      if (!res) {
-        std::cerr << "Problem with vpMatrix::AtA()!" << std::endl;
-        return EXIT_FAILURE;
-      }
-
-      vpMatrix LT = generateRandomMatrix(6, 1000, min, max);
-      vpColVector R = generateRandomVector(1000, min, max);
-      vpMatrix LTR, LTR_regular;
-
-      t = vpTime::measureTimeMs();
-      for (int i = 0; i < nb_iterations; i++)
-        LTR = LT * R;
-      t = vpTime::measureTimeMs() - t;
-      std::cout << "\n"
-                << nb_iterations
-                << " iterations of matrix vector multiplication: (6x1000) x "
-                   "(1000x1)"
-                << std::endl;
-      std::cout << "Lapack: " << t << " ms" << std::endl;
-      std::cout << "LTR:\n" << LTR.t() << std::endl;
-
-      t = vpTime::measureTimeMs();
-      for (int i = 0; i < nb_iterations; i++)
-        LTR_regular = dgemv_regular(LT, R);
-      t = vpTime::measureTimeMs() - t;
-      std::cout << "\nRegular: " << t << " ms" << std::endl;
-      std::cout << "LTR_regular:\n" << LTR_regular.t() << std::endl;
-      res = equalMatrix(LTR, LTR_regular, 1e-9);
-      std::cout << "Check result: " << res << std::endl;
-      if (!res) {
-        std::cerr << "Problem with dgemv!" << std::endl;
-        return EXIT_FAILURE;
-      }
-
-      vpVelocityTwistMatrix V(getRandomValues(min, max), getRandomValues(min, max), getRandomValues(min, max),
-                              getRandomValues(min, max), getRandomValues(min, max), getRandomValues(min, max));
-      vpMatrix LV, LV_regular;
-
-      t = vpTime::measureTimeMs();
-      for (int i = 0; i < nb_iterations; i++)
-        LV = L * V;
-      t = vpTime::measureTimeMs() - t;
-      std::cout << "\n"
-                << nb_iterations
-                << " iterations of matrix velocity twist matrix "
-                   "multiplication: (1000x6) x (6x6)"
-                << std::endl;
-      std::cout << "Lapack: " << t << " ms" << std::endl;
-
-      t = vpTime::measureTimeMs();
-      for (int i = 0; i < nb_iterations; i++)
-        LV_regular = mat_mul_twist_matrix(L, V);
-      t = vpTime::measureTimeMs() - t;
-      std::cout << "Regular: " << t << " ms" << std::endl;
-      res = equalMatrix(LV, LV_regular, 1e-9);
-      std::cout << "Check result: " << res << std::endl;
-      if (!res) {
-        std::cerr << "Problem with matrix and velocity twist matrix multiplication!" << std::endl;
-        return EXIT_FAILURE;
-      }
-    }
-#endif
-
-#ifdef VISP_HAVE_CPP11_COMPATIBILITY
+#if (VISP_CXX_STANDARD >= VISP_CXX_STANDARD_11)
     {
       std::vector<vpMatrix> vec_mat;
       vec_mat.emplace_back(5, 5);
@@ -838,7 +657,7 @@ int main()
 #endif
 
     {
-      std::cout << "------------------------" << std::endl;
+      std::cout << "\n------------------------" << std::endl;
       std::cout << "--- TEST vpMatrix::hadamard()" << std::endl;
       std::cout << "------------------------" << std::endl;
 
@@ -854,9 +673,135 @@ int main()
       std::cout << "\nRes:\n" << M2 << std::endl;
     }
 
-    std::cout << "\nAll tests succeed" << std::endl;
+    {
+      std::cout << "\n------------------------" << std::endl;
+      std::cout << "--- TEST vpMatrix::stackColums()" << std::endl;
+      std::cout << "------------------------" << std::endl;
+      vpMatrix M(3, 5);
+      for (unsigned int j = 0; j < M.getCols(); j++) {
+        for (unsigned int i = 0; i < M.getRows(); i++) {
+          M[i][j] = i + j*M.getRows();
+        }
+      }
+      std::cout << "M:\n" << M << std::endl;
+      vpColVector v = M.stackColumns();
+      std::cout << "Column stack: " << v.t() << std::endl;
+      if (M.size() != v.size()) {
+        std::cerr << "Problem in vpMatrix::stackColumns(): size differ" << std::endl;
+        return EXIT_FAILURE;
+      }
+      for (unsigned int i=0; i < v.size(); i++) {
+        if (std::fabs(v[i]-static_cast<double>(i)) > std::numeric_limits<double>::epsilon()) {
+          std::cerr << "Problem in vpMatrix::stackColumns(): content differ" << std::endl;
+          return EXIT_FAILURE;
+        }
+      }
+    }
+
+    {
+      std::cout << "\n------------------------" << std::endl;
+      std::cout << "--- TEST vpMatrix::stackRows()" << std::endl;
+      std::cout << "------------------------" << std::endl;
+      vpMatrix M(3, 5);
+      for (unsigned int i = 0; i < M.getRows(); i++) {
+        for (unsigned int j = 0; j < M.getCols(); j++) {
+          M[i][j] = i*M.getCols() + j;
+        }
+      }
+      std::cout << "M:\n" << M << std::endl;
+      vpRowVector v = M.stackRows();
+      std::cout << "Rows stack: " << v << std::endl;
+      if (M.size() != v.size()) {
+        std::cerr << "Problem in vpMatrix::stackRows(): size differ" << std::endl;
+        return EXIT_FAILURE;
+      }
+      for (unsigned int i=0; i < v.size(); i++) {
+        if (std::fabs(v[i]-static_cast<double>(i)) > std::numeric_limits<double>::epsilon()) {
+          std::cerr << "Problem in vpMatrix::stackRows(): content differ" << std::endl;
+          return EXIT_FAILURE;
+        }
+      }
+    }
+
+    {
+      std::cout << "\n------------------------" << std::endl;
+      std::cout << "--- TEST vpMatrix::getCol()" << std::endl;
+      std::cout << "------------------------" << std::endl;
+      vpMatrix A(4,4);
+      for(unsigned int i=0; i < A.getRows(); i++)
+        for(unsigned int j=0; j < A.getCols(); j++)
+          A[i][j] = i*A.getCols()+j;
+
+      {
+        vpColVector cv = A.getCol(1, 1, 3);
+        vpColVector ref;
+        ref << 5, 9, 13;
+        if (cv != ref) {
+          std::cerr << "Problem in vpMatrix::getCol(): values are different" << std::endl;
+          return EXIT_FAILURE;
+        }
+      }
+      {
+        vpColVector cv = A.getCol(1);
+        vpColVector ref;
+        ref << 1, 5, 9, 13;
+        if (cv != ref) {
+          std::cerr << "Problem in vpMatrix::getCol(): values are different" << std::endl;
+          return EXIT_FAILURE;
+        }
+      }
+    }
+
+    {
+      std::cout << "\n------------------------" << std::endl;
+      std::cout << "--- TEST vpMatrix::getRow()" << std::endl;
+      std::cout << "------------------------" << std::endl;
+      vpMatrix A(4,4);
+      for(unsigned int i=0; i < A.getRows(); i++)
+        for(unsigned int j=0; j < A.getCols(); j++)
+          A[i][j] = i*A.getCols()+j;
+
+      {
+        vpRowVector rv = A.getRow(1, 1, 3);
+        vpRowVector ref;
+        ref << 5, 6, 7;
+        if (rv != ref) {
+          std::cerr << "Problem in vpMatrix::getRow(): values are different" << std::endl;
+          return EXIT_FAILURE;
+        }
+      }
+      {
+        vpRowVector rv = A.getRow(1);
+        vpRowVector ref;
+        ref << 4, 5, 6, 7;
+        if (rv != ref) {
+          std::cerr << "Problem in vpMatrix::getRow(): values are different" << std::endl;
+          return EXIT_FAILURE;
+        }
+      }
+    }
+
+    {
+      std::cout << "\n------------------------" << std::endl;
+      std::cout << "--- TEST vpMatrix::getDiag()" << std::endl;
+      std::cout << "------------------------" << std::endl;
+      vpMatrix A(3,4);
+      for(unsigned int i=0; i < A.getRows(); i++)
+        for(unsigned int j=0; j < A.getCols(); j++)
+          A[i][j] = i*A.getCols()+j;
+
+      vpColVector diag = A.getDiag();
+      vpColVector ref;
+      ref << 0.0, 5.0, 10.0;
+      if (diag != ref) {
+        std::cerr << "Problem in vpMatrix::getDiag(): values are different" << std::endl;
+        return EXIT_FAILURE;
+      }
+    }
+
+    std::cout << "\nAll tests succeeded" << std::endl;
     return EXIT_SUCCESS;
-  } catch (vpException &e) {
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e << std::endl;
     return EXIT_FAILURE;
   }
