@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -149,7 +149,7 @@ bool getOptions(int argc, const char **argv, std::string &ipath, unsigned int &n
   \param nbBins : Number of histogram bins.
   \param nbThreads : Number of computation threads.
 */
-unsigned int histogramSum(const vpImage<unsigned char> &I, const unsigned int nbBins, const unsigned int nbThreads)
+unsigned int histogramSum(const vpImage<unsigned char> &I, unsigned int nbBins, unsigned int nbThreads)
 {
   unsigned int sum = 0;
 
@@ -169,7 +169,7 @@ unsigned int histogramSum(const vpImage<unsigned char> &I, const unsigned int nb
   \param I : Input color image.
   \param nbBins : Number of histogram bins.
 */
-bool compareHistogram(const vpImage<unsigned char> &I, const unsigned int nbBins)
+bool compareHistogram(const vpImage<unsigned char> &I, unsigned int nbBins)
 {
   vpHistogram histogram_single_threaded;
   histogram_single_threaded.calculate(I, nbBins, 1);
@@ -359,7 +359,7 @@ int main(int argc, const char **argv)
 
     std::cout << "testHistogram is OK!" << std::endl;
     return 0;
-  } catch (vpException &e) {
+  } catch (const vpException &e) {
     std::cerr << "Catch an exception: " << e.what() << std::endl;
     return 1;
   }
