@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,8 +36,8 @@
  *
  *****************************************************************************/
 
-#ifndef __vpDetectorFace_h__
-#define __vpDetectorFace_h__
+#ifndef _vpDetectorFace_h_
+#define _vpDetectorFace_h_
 
 #include <visp3/core/vpConfig.h>
 
@@ -97,7 +97,7 @@ public:
   /*!
     Default destructor.
    */
-  virtual ~vpDetectorFace(){};
+  virtual ~vpDetectorFace(){}
 
   bool detect(const vpImage<unsigned char> &I);
   bool detect(const cv::Mat &frame_gray);
