@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,22 +52,6 @@
 #include <iostream>
 #include <stdio.h>
 
-#if defined(_WIN32)
-// Include WinSock2.h before windows.h to ensure that winsock.h is not
-// included by windows.h since winsock.h and winsock2.h are incompatible
-#include <WinSock2.h>
-#include <windows.h>
-#endif
-
-#if defined(VISP_HAVE_JPEG)
-#include <jerror.h>
-#include <jpeglib.h>
-#endif
-
-#if defined(VISP_HAVE_PNG)
-#include <png.h>
-#endif
-
 /*!
   \class vpImageIo
 
@@ -79,15 +63,15 @@
 
   This class may benefit from optional 3rd parties:
   - libpng: If installed this optional 3rd party is used to read/write PNG
-images. Installation instructions are provided here
-https://visp.inria.fr/3rd_png.
+    images. Installation instructions are provided here
+    https://visp.inria.fr/3rd_png.
   - libjpeg: If installed this optional 3rd party is used to read/write JPEG
-images. Installation instructions are provided here
-https://visp.inria.fr/3rd_jpeg.
+    images. Installation instructions are provided here
+    https://visp.inria.fr/3rd_jpeg.
   - OpenCV: If installed this optional 3rd party is used to read/write other
-image formats TIFF, BMP, DIB, PBM, RASTER, JPEG2000. If libpng or libjpeg is
-not installed OpenCV is also used to consider these image formats.
-Installation instructions are provided here https://visp.inria.fr/3rd_opencv.
+    image formats TIFF, BMP, DIB, PBM, RASTER, JPEG2000. If libpng or libjpeg is
+    not installed OpenCV is also used to consider these image formats.
+    Installation instructions are provided here https://visp.inria.fr/3rd_opencv.
 
   The code below shows how to convert an PPM P6 image file format into
   a PGM P5 image file format. The extension of the filename is here
@@ -95,7 +79,6 @@ Installation instructions are provided here https://visp.inria.fr/3rd_opencv.
   (".pgm" for PGM P5 and ".ppm" for PPM P6).
 
   \code
-#include <visp3/core/vpImage.h>
 #include <visp3/io/vpImageIo.h>
 
 int main()
@@ -113,8 +96,9 @@ int main()
   \endcode
 
   This other example available in tutorial-image-reader.cpp shows how to
-read/write jpeg images. It supposes that \c libjpeg is installed. \include
-tutorial-image-reader.cpp
+  read/write jpeg images. It supposes that `libjpeg` is installed.
+
+  \include tutorial-image-reader.cpp
 */
 
 class VISP_EXPORT vpImageIo
@@ -154,15 +138,11 @@ public:
   static void readPPM(vpImage<unsigned char> &I, const std::string &filename);
   static void readPPM(vpImage<vpRGBa> &I, const std::string &filename);
 
-#if (defined(VISP_HAVE_JPEG) || defined(VISP_HAVE_OPENCV))
   static void readJPEG(vpImage<unsigned char> &I, const std::string &filename);
   static void readJPEG(vpImage<vpRGBa> &I, const std::string &filename);
-#endif
 
-#if (defined(VISP_HAVE_PNG) || defined(VISP_HAVE_OPENCV))
   static void readPNG(vpImage<unsigned char> &I, const std::string &filename);
   static void readPNG(vpImage<vpRGBa> &I, const std::string &filename);
-#endif
 
   static void writePFM(const vpImage<float> &I, const std::string &filename);
 
@@ -173,14 +153,10 @@ public:
   static void writePPM(const vpImage<unsigned char> &I, const std::string &filename);
   static void writePPM(const vpImage<vpRGBa> &I, const std::string &filename);
 
-#if (defined(VISP_HAVE_JPEG) || defined(VISP_HAVE_OPENCV))
   static void writeJPEG(const vpImage<unsigned char> &I, const std::string &filename);
   static void writeJPEG(const vpImage<vpRGBa> &I, const std::string &filename);
-#endif
 
-#if (defined(VISP_HAVE_PNG) || defined(VISP_HAVE_OPENCV))
   static void writePNG(const vpImage<unsigned char> &I, const std::string &filename);
   static void writePNG(const vpImage<vpRGBa> &I, const std::string &filename);
-#endif
 };
 #endif
