@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -131,9 +131,12 @@ int main()
   // Set up the framerate to 30Hz. Default is 25Hz.
   writer.setFramerate(30);
 
-#if defined VISP_HAVE_OPENCV
+#if VISP_HAVE_OPENCV_VERSION >= 0x030000
+  writer.setCodec( cv::VideoWriter::fourcc('P','I','M','1') );
+#elif VISP_HAVE_OPENCV_VERSION >= 0x020100
   writer.setCodec( CV_FOURCC('P','I','M','1') );
 #endif
+
   writer.setFileName("./test.mpeg");
 
   writer.open(I);
@@ -198,7 +201,7 @@ private:
 
 public:
   vpVideoWriter();
-  ~vpVideoWriter();
+  virtual ~vpVideoWriter();
 
   void close();
 
@@ -232,7 +235,7 @@ public:
 
     \param first_frame : The first frame index.
   */
-  inline void setFirstFrameIndex(const unsigned int first_frame) { this->firstFrame = first_frame; }
+  inline void setFirstFrameIndex(unsigned int first_frame) { this->firstFrame = first_frame; }
 #if VISP_HAVE_OPENCV_VERSION >= 0x020100
   /*!
       Sets the framerate in Hz of the video when encoding.
