@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -349,7 +349,7 @@ public:
     }
   }
   /*! Set the parameter which enable or disable the singularity mangement */
-  void setSingularityManagement(const bool sm) { singularityManagement = sm; }
+  void setSingularityManagement(bool sm) { singularityManagement = sm; }
 
   /*!
     Activates extra printings when the robot reaches joint limits...
