@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -485,7 +485,6 @@ vpHomogeneousMatrix vpAfma6::getForwardKinematics(const vpColVector &q) const
 
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/core/vpHomogeneousMatrix.h>
 #include <visp3/robot/vpRobotAfma6.h>
 
@@ -1130,14 +1129,14 @@ void vpAfma6::set_eMc(const vpHomogeneousMatrix &eMc)
 
   \warning This method needs XML library to parse the file defined in
   vpAfma6::CONST_CAMERA_AFMA6_FILENAME and containing the camera
-  parameters. If XML is detected by ViSP, VISP_HAVE_XML2 macro is
+  parameters. If XML is detected by ViSP, VISP_HAVE_PUGIXML macro is
   defined in include/visp3/core/vpConfig.h file.
 
   \warning Thid method needs also an access to the files containing the
   camera parameters in XML format. This access is available if
   VISP_HAVE_AFMA6_DATA macro is defined in include/visp3/core/vpConfig.h file.
 
-  - If VISP_HAVE_AFMA6_DATA and VISP_HAVE_XML2 macros are defined,
+  - If VISP_HAVE_AFMA6_DATA and VISP_HAVE_PUGIXML macros are defined,
   this method gets the camera parameters from const_camera_Afma6.xml
   config file.
 
@@ -1171,7 +1170,7 @@ int main()
   // Get the intrinsic camera parameters depending on the image size
   // Camera parameters are read from
   // /udd/fspindle/robot/Afma6/current/include/const_camera_Afma6.xml
-  // if VISP_HAVE_AFMA6_DATA and VISP_HAVE_XML2 macros are defined in vpConfig.h file
+  // if VISP_HAVE_AFMA6_DATA and VISP_HAVE_PUGIXML macros are defined in vpConfig.h file
   try {
     robot.getCameraParameters (cam, I.getWidth(), I.getHeight());
   }
@@ -1190,7 +1189,7 @@ parameters are not found.
 void vpAfma6::getCameraParameters(vpCameraParameters &cam, const unsigned int &image_width,
                                   const unsigned int &image_height) const
 {
-#if defined(VISP_HAVE_XML2) && defined(VISP_HAVE_AFMA6_DATA)
+#if defined(VISP_HAVE_PUGIXML) && defined(VISP_HAVE_AFMA6_DATA)
   vpXmlParserCamera parser;
   switch (getToolType()) {
   case vpAfma6::TOOL_CCMOP: {
@@ -1390,7 +1389,6 @@ void vpAfma6::getCameraParameters(vpCameraParameters &cam, const vpImage<unsigne
 
   \code
 #include <visp3/core/vpCameraParameters.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/core/vpImage.h>
 #include <visp3/robot/vpRobotAfma6.h>
 #include <visp3/sensor/vp1394TwoGrabber.h>
