@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -916,7 +916,7 @@ void vpRobotAfma6::get_fJe(vpMatrix &fJe)
 
   \sa getPositioningVelocity()
 */
-void vpRobotAfma6::setPositioningVelocity(const double velocity) { positioningVelocity = velocity; }
+void vpRobotAfma6::setPositioningVelocity(double velocity) { positioningVelocity = velocity; }
 
 /*!
   Get the maximal velocity percentage used for a position control.
@@ -947,14 +947,13 @@ three last parameters are the rotations expressed as a theta u vector in
 
   - Mixt frame of joint frame is not implemented.
 
-  \exception vpRobotException::lowLevelError : vpRobot::MIXT_FRAME
-  and vpRobot::ARTICULAR_FRAME not implemented.
+  \exception vpRobotException::lowLevelError : vpRobot::MIXT_FRAME,
+  vpRobot::END_EFFECTOR_FRAME and vpRobot::ARTICULAR_FRAME not implemented.
 
   \exception vpRobotException::positionOutOfRangeError : The requested
   position is out of range.
 
   \code
-#include <visp3/core/vpConfig.h>
 #include <visp3/core/vpPoseVector.h>
 #include <visp3/robot/vpRobotAfma6.h>
 
@@ -1048,15 +1047,14 @@ void vpRobotAfma6::setPosition(const vpRobot::vpControlFrameType frame, const vp
   expressed in the reference frame, and rotations in the camera
   frame.
 
-  \exception vpRobotException::lowLevelError : vpRobot::MIXT_FRAME not
-  implemented.
+  \exception vpRobotException::lowLevelError : vpRobot::MIXT_FRAME
+  and vpRobot::END_EFFECTOR_FRAME not implemented.
 
   \exception vpRobotException::positionOutOfRangeError : The requested
   position is out of range.
 
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/robot/vpRobotAfma6.h>
 #include <visp3/robot/vpRobotException.h>
 
@@ -1199,10 +1197,13 @@ void vpRobotAfma6::setPosition(const vpRobot::vpControlFrameType frame, const vp
     break;
   }
   case vpRobot::MIXT_FRAME: {
-    vpERROR_TRACE("Positionning error. Mixt frame not implemented");
     throw vpRobotException(vpRobotException::lowLevelError, "Positionning error: "
                                                             "Mixt frame not implemented.");
   }
+  case vpRobot::END_EFFECTOR_FRAME: {
+    throw vpRobotException(vpRobotException::lowLevelError, "Positionning error: "
+                                                            "end-effector frame not implemented.");
+  }
   }
 
   CatchPrint();
@@ -1250,14 +1251,13 @@ void vpRobotAfma6::setPosition(const vpRobot::vpControlFrameType frame, const vp
   expressed in the reference frame, and rotations in the camera
   frame.
 
-  \exception vpRobotException::lowLevelError : vpRobot::MIXT_FRAME not
-  implemented.
+  \exception vpRobotException::lowLevelError : vpRobot::MIXT_FRAME
+  and vpRobot::END_EFFECTOR_FRAME not implemented.
 
   \exception vpRobotException::positionOutOfRangeError : The requested
   position is out of range.
 
   \code
-#include <visp3/core/vpConfig.h>
 #include <visp3/robot/vpRobotAfma6.h>
 
 int main()
@@ -1288,8 +1288,8 @@ int main()
 
   \sa setPosition()
 */
-void vpRobotAfma6::setPosition(const vpRobot::vpControlFrameType frame, const double pos1, const double pos2,
-                               const double pos3, const double pos4, const double pos5, const double pos6)
+void vpRobotAfma6::setPosition(const vpRobot::vpControlFrameType frame, double pos1, double pos2,
+                               double pos3, double pos4, double pos5, double pos6)
 {
   try {
     vpColVector position(6);
@@ -1321,7 +1321,6 @@ void vpRobotAfma6::setPosition(const vpRobot::vpControlFrameType frame, const do
   This method has the same behavior than the sample code given below;
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/robot/vpRobotAfma6.h>
 
 int main()
@@ -1339,8 +1338,8 @@ int main()
 }
   \endcode
 
-  \exception vpRobotException::lowLevelError : vpRobot::MIXT_FRAME not
-  implemented.
+  \exception vpRobotException::lowLevelError : vpRobot::MIXT_FRAME
+  and vpRobot::END_EFFECTOR_FRAME not implemented.
 
   \exception vpRobotException::positionOutOfRangeError : The requested
   position is out of range.
@@ -1463,10 +1462,13 @@ void vpRobotAfma6::getPosition(const vpRobot::vpControlFrameType frame, vpColVec
     break;
   }
   case vpRobot::MIXT_FRAME: {
-    vpERROR_TRACE("Cannot get position in mixt frame: not implemented");
     throw vpRobotException(vpRobotException::lowLevelError, "Cannot get position in mixt frame: "
                                                             "not implemented");
   }
+  case vpRobot::END_EFFECTOR_FRAME: {
+    throw vpRobotException(vpRobotException::lowLevelError, "Cannot get position in end-effector frame: "
+                                                            "not implemented");
+  }
   }
 
   CatchPrint();
@@ -1636,6 +1638,16 @@ void vpRobotAfma6::setVelocity(const vpRobot::vpControlFrameType frame, const vp
     Try(PrimitiveMOVESPEED_CART_Afma6(vel_sat.data, REPCAM_AFMA6));
     break;
   }
+  case vpRobot::END_EFFECTOR_FRAME: {
+    // Tranform in camera frame
+    vpHomogeneousMatrix cMe;
+    this->get_cMe(cMe);
+    vpVelocityTwistMatrix cVe(cMe);
+    vpColVector v_c = cVe * vel_sat;
+    // Send velocities in m/s and rad/s
+    Try(PrimitiveMOVESPEED_CART_Afma6(v_c.data, REPCAM_AFMA6));
+    break;
+  }
   case vpRobot::ARTICULAR_FRAME: {
     // Try( PrimitiveMOVESPEED_CART(vel_sat.data, REPART_AFMA6) );
     Try(PrimitiveMOVESPEED_Afma6(vel_sat.data));
@@ -1809,6 +1821,10 @@ void vpRobotAfma6::getVelocity(const vpRobot::vpControlFrameType frame, vpColVec
       velocity /= (time_cur - time_prev_getvel);
       break;
     }
+    default: {
+      throw(vpException(vpException::functionNotImplementedError,
+                        "vpRobotAfma6::getVelocity() not implemented in end-effector"));
+    }
     }
   } else {
     first_time_getvel = false;
@@ -2100,7 +2116,7 @@ void vpRobotAfma6::move(const std::string &filename)
   \sa readPosFile(), move(const char *)
 
 */
-void vpRobotAfma6::move(const std::string &filename, const double velocity)
+void vpRobotAfma6::move(const std::string &filename, double velocity)
 {
   vpColVector q;
 
@@ -2221,6 +2237,10 @@ void vpRobotAfma6::getDisplacement(vpRobot::vpControlFrameType frame, vpColVecto
       std::cout << "getDisplacement() MIXT_FRAME not implemented\n";
       return;
     }
+    case vpRobot::END_EFFECTOR_FRAME: {
+      std::cout << "getDisplacement() END_EFFECTOR_FRAME not implemented\n";
+      return;
+    }
     }
   } else {
     first_time_getdis = false;
