@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,12 +42,11 @@
 #include <signal.h>
 #include <string.h>
 
-#include <visp3/core/vpTime.h>
-
 #include <visp3/core/vpConfig.h>
 
 #ifdef VISP_HAVE_BICLOPS
 
+#include <visp3/core/vpTime.h>
 #include <visp3/core/vpExponentialMap.h>
 #include <visp3/core/vpIoTools.h>
 #include <visp3/robot/vpBiclops.h>
@@ -78,13 +77,12 @@ static pthread_mutex_t vpMeasure_mutex;
   Default constructor.
 
   Does nothing more than setting the default configuration file
-  to /usr/share/BiclopsDefault.cfg.
+  to `/usr/share/BiclopsDefault.cfg`.
 
-  As shown in the following example,the turret need to be initialized
+  As shown in the following example, the turret need to be initialized
   using init() function.
 
   \code
-#include <visp3/core/vpConfig.h>
 #include <visp3/robot/vpRobotBiclops.h>
 
 int main()
@@ -130,16 +128,13 @@ vpRobotBiclops::vpRobotBiclops()
 
 /*!
 
-  Default constructor.
-
-  Initialize the biclops pan, tilt head by reading the
+  Constructor that initialize the biclops pan, tilt head by reading the
   configuration file provided by Traclabs
   and do the homing sequence.
 
   The following example shows how to use the constructor.
 
   \code
-#include <visp3/core/vpConfig.h>
 #include <visp3/robot/vpRobotBiclops.h>
 
 int main()
@@ -187,7 +182,7 @@ vpRobotBiclops::vpRobotBiclops(const std::string &filename)
 
 */
 
-vpRobotBiclops::~vpRobotBiclops(void)
+vpRobotBiclops::~vpRobotBiclops()
 {
 
   vpDEBUG_TRACE(12, "Start vpRobotBiclops::~vpRobotBiclops()");
@@ -684,7 +679,7 @@ void vpRobotBiclops::get_fJe(vpMatrix &_fJe)
   \param velocity : Velocity in % of the maximum velocity between [0,100]. The
   maximum velocity is given vpBiclops::speedLimit.
 */
-void vpRobotBiclops::setPositioningVelocity(const double velocity)
+void vpRobotBiclops::setPositioningVelocity(double velocity)
 {
   if (velocity < 0 || velocity > 100) {
     vpERROR_TRACE("Bad positionning velocity");
@@ -728,23 +723,21 @@ void vpRobotBiclops::setPosition(const vpRobot::vpControlFrameType frame, const
 
   switch (frame) {
   case vpRobot::CAMERA_FRAME:
-    vpERROR_TRACE("Cannot move the robot in camera frame: "
-                  "not implemented");
     throw vpRobotException(vpRobotException::wrongStateError, "Cannot move the robot in camera frame: "
                                                               "not implemented");
     break;
   case vpRobot::REFERENCE_FRAME:
-    vpERROR_TRACE("Cannot move the robot in reference frame: "
-                  "not implemented");
     throw vpRobotException(vpRobotException::wrongStateError, "Cannot move the robot in reference frame: "
                                                               "not implemented");
     break;
   case vpRobot::MIXT_FRAME:
-    vpERROR_TRACE("Cannot move the robot in mixt frame: "
-                  "not implemented");
     throw vpRobotException(vpRobotException::wrongStateError, "Cannot move the robot in mixt frame: "
                                                               "not implemented");
     break;
+  case vpRobot::END_EFFECTOR_FRAME:
+    throw vpRobotException(vpRobotException::wrongStateError, "Cannot move the robot in end-effector frame: "
+                                                              "not implemented");
+    break;
   case vpRobot::ARTICULAR_FRAME:
     break;
   }
@@ -837,22 +830,21 @@ void vpRobotBiclops::getPosition(const vpRobot::vpControlFrameType frame, vpColV
 {
   switch (frame) {
   case vpRobot::CAMERA_FRAME:
-    vpERROR_TRACE("Cannot get position in camera frame: not implemented");
     throw vpRobotException(vpRobotException::wrongStateError, "Cannot get position in camera frame: "
                                                               "not implemented");
     break;
   case vpRobot::REFERENCE_FRAME:
-    vpERROR_TRACE("Cannot get position in reference frame: "
-                  "not implemented");
     throw vpRobotException(vpRobotException::wrongStateError, "Cannot get position in reference frame: "
                                                               "not implemented");
     break;
   case vpRobot::MIXT_FRAME:
-    vpERROR_TRACE("Cannot get position in mixt frame: "
-                  "not implemented");
     throw vpRobotException(vpRobotException::wrongStateError, "Cannot get position in mixt frame: "
                                                               "not implemented");
     break;
+  case vpRobot::END_EFFECTOR_FRAME:
+    throw vpRobotException(vpRobotException::wrongStateError, "Cannot get position in end-effector frame: "
+                                                              "not implemented");
+    break;
   case vpRobot::ARTICULAR_FRAME:
     break;
   }
@@ -903,8 +895,8 @@ void vpRobotBiclops::getPosition(const vpRobot::vpControlFrameType frame, vpColV
 
   \param frame : Control frame. This biclops head can only be controlled in
   articular. Be aware, the camera frame (vpRobot::CAMERA_FRAME), the reference
-  frame (vpRobot::REFERENCE_FRAME) and the mixt frame (vpRobot::MIXT_FRAME)
-  are not implemented.
+  frame (vpRobot::REFERENCE_FRAME), end-effector frame (vpRobot::END_EFFECTOR_FRAME)
+  and the mixt frame (vpRobot::MIXT_FRAME) are not implemented.
 
   \param q_dot : The desired articular velocity of the axis in rad/s. \f$ \dot
   {r} = [\dot{q}_1, \dot{q}_2]^t \f$ with \f$ \dot{q}_1 \f$ the pan of the
@@ -916,8 +908,8 @@ void vpRobotBiclops::getPosition(const vpRobot::vpControlFrameType frame, vpColV
   vpRobot::STATE_VELOCITY_CONTROL) before setVelocity().
 
   \exception vpRobotException::wrongStateError : If a not supported frame type
-  (vpRobot::CAMERA_FRAME, vpRobot::REFERENCE_FRAME or vpRobot::MIXT_FRAME) is
-  given.
+  (vpRobot::CAMERA_FRAME, vpRobot::REFERENCE_FRAME, vpRobot::END_EFFECTOR_FRAME
+  or vpRobot::MIXT_FRAME) is given.
 
   \warning Velocities could be saturated if one of them exceed the maximal
   autorized speed (see vpRobot::maxRotationVelocity).
@@ -951,24 +943,21 @@ void vpRobotBiclops::setVelocity(const vpRobot::vpControlFrameType frame, const
     break;
   }
   case vpRobot::REFERENCE_FRAME: {
-    vpERROR_TRACE("Cannot send a velocity to the robot "
-                  "in the reference frame: "
-                  "functionality not implemented");
     throw vpRobotException(vpRobotException::wrongStateError, "Cannot send a velocity to the robot "
                                                               "in the reference frame:"
                                                               "functionality not implemented");
   }
   case vpRobot::MIXT_FRAME: {
-    vpERROR_TRACE("Cannot send a velocity to the robot "
-                  "in the mixt frame: "
-                  "functionality not implemented");
     throw vpRobotException(vpRobotException::wrongStateError, "Cannot send a velocity to the robot "
                                                               "in the mixt frame:"
                                                               "functionality not implemented");
   }
+  case vpRobot::END_EFFECTOR_FRAME: {
+    throw vpRobotException(vpRobotException::wrongStateError, "Cannot send a velocity to the robot "
+                                                              "in the end-effector frame:"
+                                                              "functionality not implemented");
+  }
   default: {
-    vpERROR_TRACE("Error in spec of vpRobot. "
-                  "Case not taken in account.");
     throw vpRobotException(vpRobotException::wrongStateError, "Cannot send a velocity to the robot ");
   }
   }
@@ -1041,22 +1030,21 @@ void vpRobotBiclops::getVelocity(const vpRobot::vpControlFrameType frame, vpColV
 {
   switch (frame) {
   case vpRobot::CAMERA_FRAME:
-    vpERROR_TRACE("Cannot get position in camera frame: not implemented");
     throw vpRobotException(vpRobotException::wrongStateError, "Cannot get position in camera frame: "
                                                               "not implemented");
     break;
   case vpRobot::REFERENCE_FRAME:
-    vpERROR_TRACE("Cannot get position in reference frame: "
-                  "not implemented");
     throw vpRobotException(vpRobotException::wrongStateError, "Cannot get position in reference frame: "
                                                               "not implemented");
     break;
   case vpRobot::MIXT_FRAME:
-    vpERROR_TRACE("Cannot get position in mixt frame: "
-                  "not implemented");
     throw vpRobotException(vpRobotException::wrongStateError, "Cannot get position in mixt frame: "
                                                               "not implemented");
     break;
+  case vpRobot::END_EFFECTOR_FRAME:
+    throw vpRobotException(vpRobotException::wrongStateError, "Cannot get position in end-effector frame: "
+                                                              "not implemented");
+    break;
   case vpRobot::ARTICULAR_FRAME:
     break;
   }
@@ -1248,17 +1236,17 @@ void vpRobotBiclops::getDisplacement(vpRobot::vpControlFrameType frame, vpColVec
   }
 
   case vpRobot::REFERENCE_FRAME:
-    vpERROR_TRACE("Cannot get a velocity in the reference frame: "
-                  "functionality not implemented");
     throw vpRobotException(vpRobotException::wrongStateError, "Cannot get a velocity in the reference frame:"
                                                               "functionality not implemented");
     break;
   case vpRobot::MIXT_FRAME:
-    vpERROR_TRACE("Cannot get a velocity in the mixt frame: "
-                  "functionality not implemented");
     throw vpRobotException(vpRobotException::wrongStateError, "Cannot get a velocity in the mixt frame:"
                                                               "functionality not implemented");
     break;
+  case vpRobot::END_EFFECTOR_FRAME:
+    throw vpRobotException(vpRobotException::wrongStateError, "Cannot get a velocity in the end-effector frame:"
+                                                              "functionality not implemented");
+    break;
   }
 
   q_previous = q_current; // Update for next call of this method
