@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -136,7 +136,6 @@ void emergencyStopViper650(int signo)
 
   \code
 #include <visp3/core/vpCameraParameters.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/core/vpImage.h>
 #include <visp3/robot/vpRobotViper650.h>
 #include <visp3/sensor/vp1394TwoGrabber.h>
@@ -367,7 +366,6 @@ void vpRobotViper650::init(void)
 
   \code
 #include <visp3/core/vpCameraParameters.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/core/vpImage.h>
 #include <visp3/robot/vpRobotViper650.h>
 #include <visp3/sensor/vp1394TwoGrabber.h>
@@ -1000,7 +998,6 @@ void vpRobotViper650::get_fJe(vpMatrix &fJe)
 
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/robot/vpRobotViper650.h>
 
 int main()
@@ -1024,7 +1021,7 @@ int main()
 
   \sa getPositioningVelocity()
 */
-void vpRobotViper650::setPositioningVelocity(const double velocity) { positioningVelocity = velocity; }
+void vpRobotViper650::setPositioningVelocity(double velocity) { positioningVelocity = velocity; }
 
 /*!
   Get the maximal velocity percentage used for a position control.
@@ -1061,15 +1058,14 @@ double vpRobotViper650::getPositioningVelocity(void) const { return positioningV
   expressed in the reference frame, and rotations in the camera
   frame.
 
-  \exception vpRobotException::lowLevelError : vpRobot::MIXT_FRAME not
-  implemented.
+  \exception vpRobotException::lowLevelError : vpRobot::MIXT_FRAME and
+  vpRobot::END_EFFECTOR_FRAME not implemented.
 
   \exception vpRobotException::positionOutOfRangeError : The requested
   position is out of range.
 
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/robot/vpRobotViper650.h>
 
 int main()
@@ -1198,7 +1194,10 @@ void vpRobotViper650::setPosition(const vpRobot::vpControlFrameType frame, const
     break;
   }
   case vpRobot::MIXT_FRAME: {
-    vpERROR_TRACE("Positionning error. Mixt frame not implemented");
+    throw vpRobotException(vpRobotException::lowLevelError, "Positionning error: "
+                                                            "Mixt frame not implemented.");
+  }
+  case vpRobot::END_EFFECTOR_FRAME: {
     throw vpRobotException(vpRobotException::lowLevelError, "Positionning error: "
                                                             "Mixt frame not implemented.");
   }
@@ -1256,14 +1255,13 @@ void vpRobotViper650::setPosition(const vpRobot::vpControlFrameType frame, const
   expressed in the reference frame, and rotations in the camera
   frame.
 
-  \exception vpRobotException::lowLevelError : vpRobot::MIXT_FRAME not
-  implemented.
+  \exception vpRobotException::lowLevelError : vpRobot::MIXT_FRAME
+  and vpRobot::END_EFFECTOR_FRAME not implemented.
 
   \exception vpRobotException::positionOutOfRangeError : The requested
   position is out of range.
 
   \code
-#include <visp3/core/vpConfig.h>
 #include <visp3/robot/vpRobotViper650.h>
 
 int main()
@@ -1292,8 +1290,8 @@ int main()
 
   \sa setPosition()
 */
-void vpRobotViper650::setPosition(const vpRobot::vpControlFrameType frame, const double pos1, const double pos2,
-                                  const double pos3, const double pos4, const double pos5, const double pos6)
+void vpRobotViper650::setPosition(const vpRobot::vpControlFrameType frame, double pos1, double pos2,
+                                  double pos3, double pos4, double pos5, double pos6)
 {
   try {
     vpColVector position(6);
@@ -1325,7 +1323,6 @@ void vpRobotViper650::setPosition(const vpRobot::vpControlFrameType frame, const
   This method has the same behavior than the sample code given below;
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/robot/vpRobotViper650.h>
 
 int main()
@@ -1341,8 +1338,8 @@ int main()
 }
   \endcode
 
-  \exception vpRobotException::lowLevelError : vpRobot::MIXT_FRAME not
-  implemented.
+  \exception vpRobotException::lowLevelError : vpRobot::MIXT_FRAME
+  and vpRobot::END_EFFECTOR_FRAME not implemented.
 
   \exception vpRobotException::positionOutOfRangeError : The requested
   position is out of range.
@@ -1391,7 +1388,6 @@ void vpRobotViper650::setPosition(const std::string &filename)
 
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/core/vpHomogeneousMatrix.h>
 #include <visp3/core/vpRotationMatrix.h>
 #include <visp3/core/vpRxyzVector.h>
@@ -1453,33 +1449,33 @@ void vpRobotViper650::getPosition(const vpRobot::vpControlFrameType frame, vpCol
     return;
   }
   case vpRobot::REFERENCE_FRAME: {
-    Try(PrimitiveACQ_POS_C_Viper650(position.data, &timestamp));
-    //    vpCTRACE << "Get cartesian position " << position.t() << std::endl;
-    // 1=tx, 2=ty, 3=tz in meters; 4=Rz 5=Ry 6=Rz in deg
-    // Convert Euler Rzyz angles from deg to rad
-    for (unsigned int i = 3; i < 6; i++)
-      position[i] = vpMath::rad(position[i]);
-    // Convert Rzyz angles into Rxyz representation
-    vpRzyzVector rzyz(position[3], position[4], position[5]);
-    vpRotationMatrix R(rzyz);
-    vpRxyzVector rxyz(R);
+    vpColVector q(njoint);
+    Try(PrimitiveACQ_POS_J_Viper650(q.data, &timestamp));
 
-    // Update the position using Rxyz representation
-    for (unsigned int i = 0; i < 3; i++)
-      position[i + 3] = rxyz[i];
-    //     vpCTRACE << "Cartesian position Rxyz (deg)"
-    // 	     << position[0] << " " << position[1] << " " << position[2] << " "
-    // 	     << vpMath::deg(position[3]) << " "
-    // 	     << vpMath::deg(position[4]) << " "
-    // 	     << vpMath::deg(position[5]) << std::endl;
+    // Compute fMc
+    vpHomogeneousMatrix fMc = vpViper650::get_fMc(q);
+
+    // From fMc extract the pose
+    vpRotationMatrix fRc;
+    fMc.extract(fRc);
+    vpRxyzVector rxyz;
+    rxyz.buildFrom(fRc);
+
+    for (unsigned int i = 0; i < 3; i++) {
+      position[i] = fMc[i][3];   // translation x,y,z
+      position[i + 3] = rxyz[i]; // Euler rotation x,y,z
+    }
 
     break;
   }
   case vpRobot::MIXT_FRAME: {
-    vpERROR_TRACE("Cannot get position in mixt frame: not implemented");
     throw vpRobotException(vpRobotException::lowLevelError, "Cannot get position in mixt frame: "
                                                             "not implemented");
   }
+  case vpRobot::END_EFFECTOR_FRAME: {
+    throw vpRobotException(vpRobotException::lowLevelError, "Cannot get position in end-effector frame: "
+                                                            "not implemented");
+  }
   }
 
   CatchPrint();
@@ -1609,7 +1605,6 @@ reference frame in m/s and rotations \f$ ^{c} \omega_x, ^{c} \omega_y, ^{c}
 
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/core/vpMath.h>
 #include <visp3/robot/vpRobotViper650.h>
 
@@ -1660,6 +1655,7 @@ void vpRobotViper650::setVelocity(const vpRobot::vpControlFrameType frame, const
   // saturation in cartesian space
   case vpRobot::CAMERA_FRAME:
   case vpRobot::REFERENCE_FRAME:
+  case vpRobot::END_EFFECTOR_FRAME:
   case vpRobot::MIXT_FRAME: {
     vpColVector vel_max(6);
 
@@ -1699,6 +1695,16 @@ void vpRobotViper650::setVelocity(const vpRobot::vpControlFrameType frame, const
     Try(PrimitiveMOVESPEED_CART_Viper650(vel_sat.data, REPCAM_VIPER650));
     break;
   }
+  case vpRobot::END_EFFECTOR_FRAME: {
+    // Transform in camera frame
+    vpHomogeneousMatrix cMe;
+    this->get_cMe(cMe);
+    vpVelocityTwistMatrix cVe(cMe);
+    vpColVector v_c = cVe * vel_sat;
+    // Send velocities in m/s and rad/s
+    Try(PrimitiveMOVESPEED_CART_Viper650(v_c.data, REPCAM_VIPER650));
+    break;
+  }
   case vpRobot::ARTICULAR_FRAME: {
     // Convert all the velocities from rad/s into deg/s
     vel_sat.rad2deg();
@@ -1709,7 +1715,7 @@ void vpRobotViper650::setVelocity(const vpRobot::vpControlFrameType frame, const
   }
   case vpRobot::REFERENCE_FRAME: {
     // Send velocities in m/s and rad/s
-    std::cout << "Vitesse ref appliquee: " << vel_sat.t();
+    //std::cout << "Vitesse ref appliquee: " << vel_sat.t();
     Try(PrimitiveMOVESPEED_CART_Viper650(vel_sat.data, REPFIX_VIPER650));
     break;
   }
@@ -1772,7 +1778,6 @@ u}_z]\f$.
 
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/robot/vpRobotViper650.h>
 
 int main()
@@ -1882,6 +1887,10 @@ void vpRobotViper650::getVelocity(const vpRobot::vpControlFrameType frame, vpCol
       velocity /= (time_cur - time_prev_getvel);
       break;
     }
+    default: {
+      throw(vpException(vpException::functionNotImplementedError,
+                        "vpRobotViper650::getVelocity() not implemented in end-effector"));
+    }
     }
   } else {
     first_time_getvel = false;
@@ -1930,7 +1939,6 @@ void vpRobotViper650::getVelocity(const vpRobot::vpControlFrameType frame, vpCol
 
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/robot/vpRobotViper650.h>
 
 int main()
@@ -2028,7 +2036,6 @@ The code below shows how to read a position from a file and move the robot to
 this position.
 \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/robot/vpRobotViper650.h>
 
 int main()
@@ -2243,6 +2250,10 @@ void vpRobotViper650::getDisplacement(vpRobot::vpControlFrameType frame, vpColVe
       std::cout << "getDisplacement() MIXT_FRAME not implemented\n";
       return;
     }
+    case vpRobot::END_EFFECTOR_FRAME: {
+      std::cout << "getDisplacement() END_EFFECTOR_FRAME not implemented\n";
+      return;
+    }
     }
   } else {
     first_time_getdis = false;
@@ -2298,7 +2309,6 @@ bias.
 
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/core/vpTime.h>
 #include <visp3/robot/vpRobotViper650.h>
 
@@ -2353,7 +2363,6 @@ bias.
 
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/core/vpTime.h>
 #include <visp3/robot/vpRobotViper650.h>
 
@@ -2472,7 +2481,7 @@ void vpRobotViper650::setMaxRotationVelocity(double w_max)
   \param w6_max : Maximum rotation velocity expressed in rad/s on joint 6.
 */
 
-void vpRobotViper650::setMaxRotationVelocityJoint6(const double w6_max)
+void vpRobotViper650::setMaxRotationVelocityJoint6(double w6_max)
 {
   maxRotationVelocity_joint6 = w6_max;
   return;
