@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -848,7 +848,6 @@ reference frame in m/s and rotations \f$ ^{c} \omega_x, ^{c} \omega_y, ^{c}
 
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/core/vpMath.h>
 #include <visp3/robot/vpSimulatorAfma6.h>
 
@@ -965,7 +964,12 @@ void vpSimulatorAfma6::setVelocity(const vpRobot::vpControlFrameType frame, cons
     break;
   }
   case vpRobot::MIXT_FRAME: {
-    break;
+    throw vpRobotException(vpRobotException::wrongStateError, "Cannot set a velocity in MIXT_FRAME frame:"
+                                                              "functionality not implemented");
+  }
+  case vpRobot::END_EFFECTOR_FRAME: {
+    throw vpRobotException(vpRobotException::wrongStateError, "Cannot set a velocity in END_EFFECTOT_FRAME frame:"
+                                                              "functionality not implemented");
   }
   }
 
@@ -1014,6 +1018,7 @@ void vpSimulatorAfma6::computeArticularVelocity()
     set_artVel(articularVelocity);
     break;
   }
+  case vpRobot::END_EFFECTOR_FRAME:
   case vpRobot::MIXT_FRAME: {
     break;
   }
@@ -1042,6 +1047,7 @@ void vpSimulatorAfma6::computeArticularVelocity()
     break;
   }
   case vpRobot::ARTICULAR_FRAME:
+  case vpRobot::END_EFFECTOR_FRAME:
   case vpRobot::MIXT_FRAME: {
     break;
   }
@@ -1062,7 +1068,6 @@ void vpSimulatorAfma6::computeArticularVelocity()
 
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/robot/vpSimulatorAfma6.h>
 
 int main()
@@ -1119,6 +1124,7 @@ void vpSimulatorAfma6::getVelocity(const vpRobot::vpControlFrameType frame, vpCo
     vel = fJe_ * articularVelocity;
     break;
   }
+  case vpRobot::END_EFFECTOR_FRAME:
   case vpRobot::MIXT_FRAME: {
     break;
   }
@@ -1162,7 +1168,6 @@ void vpSimulatorAfma6::getVelocity(const vpRobot::vpControlFrameType frame, vpCo
 
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/robot/vpSimulatorAfma6.h>
 
 int main()
@@ -1271,7 +1276,6 @@ void vpSimulatorAfma6::findHighestPositioningSpeed(vpColVector &q)
 
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/robot/vpSimulatorAfma6.h>
 
 int main()
@@ -1427,7 +1431,12 @@ void vpSimulatorAfma6::setPosition(const vpRobot::vpControlFrameType frame, cons
   case vpRobot::MIXT_FRAME: {
     vpERROR_TRACE("Positionning error. Mixt frame not implemented");
     throw vpRobotException(vpRobotException::lowLevelError, "Positionning error: "
-                                                            "Mixt frame not implemented.");
+                                                            "MIXT_FRAME not implemented.");
+  }
+  case vpRobot::END_EFFECTOR_FRAME: {
+    vpERROR_TRACE("Positionning error. Mixt frame not implemented");
+    throw vpRobotException(vpRobotException::lowLevelError, "Positionning error: "
+                                                            "END_EFFECTOR_FRAME not implemented.");
   }
   }
 }
@@ -1468,7 +1477,6 @@ void vpSimulatorAfma6::setPosition(const vpRobot::vpControlFrameType frame, cons
   position is out of range.
 
   \code
-#include <visp3/core/vpConfig.h>
 #include <visp3/robot/vpSimulatorAfma6.h>
 
 int main()
@@ -1495,8 +1503,8 @@ int main()
 
   \sa setPosition()
 */
-void vpSimulatorAfma6::setPosition(const vpRobot::vpControlFrameType frame, const double pos1, const double pos2,
-                                   const double pos3, const double pos4, const double pos5, const double pos6)
+void vpSimulatorAfma6::setPosition(const vpRobot::vpControlFrameType frame, double pos1, double pos2,
+                                   double pos3, double pos4, double pos5, double pos6)
 {
   try {
     vpColVector position(6);
@@ -1527,7 +1535,6 @@ void vpSimulatorAfma6::setPosition(const vpRobot::vpControlFrameType frame, cons
   This method has the same behavior than the sample code given below;
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/robot/vpSimulatorAfma6.h>
 
 int main()
@@ -1587,7 +1594,6 @@ void vpSimulatorAfma6::setPosition(const char *filename)
 
   \code
 #include <visp3/core/vpColVector.h>
-#include <visp3/core/vpConfig.h>
 #include <visp3/core/vpHomogeneousMatrix.h>
 #include <visp3/core/vpRotationMatrix.h>
 #include <visp3/core/vpRxyzVector.h>
@@ -1658,10 +1664,13 @@ void vpSimulatorAfma6::getPosition(const vpRobot::vpControlFrameType frame, vpCo
   }
 
   case vpRobot::MIXT_FRAME: {
-    vpERROR_TRACE("Positionning error. Mixt frame not implemented");
     throw vpRobotException(vpRobotException::lowLevelError, "Positionning error: "
                                                             "Mixt frame not implemented.");
   }
+  case vpRobot::END_EFFECTOR_FRAME: {
+    throw vpRobotException(vpRobotException::lowLevelError, "Positionning error: "
+                                                            "End-effector frame not implemented.");
+  }
   }
 }
 
@@ -1868,21 +1877,22 @@ void vpSimulatorAfma6::getDisplacement(vpRobot::vpControlFrameType frame, vpColV
       std::cout << "getDisplacement() CAMERA_FRAME not implemented\n";
       return;
     }
-
     case vpRobot::ARTICULAR_FRAME: {
       displacement = q_cur - q_prev_getdis;
       break;
     }
-
     case vpRobot::REFERENCE_FRAME: {
       std::cout << "getDisplacement() REFERENCE_FRAME not implemented\n";
       return;
     }
-
     case vpRobot::MIXT_FRAME: {
       std::cout << "getDisplacement() MIXT_FRAME not implemented\n";
       return;
     }
+    case vpRobot::END_EFFECTOR_FRAME: {
+      std::cout << "getDisplacement() END_EFFECTOR_FRAME not implemented\n";
+      return;
+    }
     }
   } else {
     first_time_getdis = false;
@@ -2434,7 +2444,7 @@ bool vpSimulatorAfma6::setPosition(const vpHomogeneousMatrix &cdMo_, vpImage<uns
   vpVelocityTwistMatrix cVe;
 
   unsigned int i, iter = 0;
-  while ((iter++ < 300) & (err.euclideanNorm() > errMax)) {
+  while ((iter++ < 300) & (err.frobeniusNorm() > errMax)) {
     double t = vpTime::measureTimeMs();
 
     // update image
@@ -2473,7 +2483,7 @@ bool vpSimulatorAfma6::setPosition(const vpHomogeneousMatrix &cdMo_, vpImage<uns
   setMaxRotationVelocity(wMax);
 
   // std::cout << "setPosition: final error " << err.t() << std::endl;
-  return (err.euclideanNorm() <= errMax);
+  return (err.frobeniusNorm() <= errMax);
 }
 
 #elif !defined(VISP_BUILD_SHARED_LIBS)
