@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -67,6 +67,6 @@ Keyword keyword_tbl[] = {/* tableau des mots cles termine par NULL*/
                          {"vrp", T_VRP},
                          {"vup", T_VUP},
                          {"window", T_WINDOW},
-                         {NULL, NULL}};
+                         {NULL, T_NULL}};
 
 #endif
