@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -100,7 +100,7 @@ int main()
     float period = 0.001f;
     vpVirtuose virtuose;
     virtuose.setTimeStep(period);
-    virtuose.setIpAddress("localhost#5000");
+    virtuose.setIpAddress("localhost#53210");
     virtuose.setVerbose(true);
     virtuose.setPowerOn();
 
@@ -158,7 +158,7 @@ int main()
     }
 
     std::cout << "The end" << std::endl;
-  } catch (vpException &e) {
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e.getStringMessage() << std::endl;
   }
 }
