@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,7 +42,7 @@
 
 #include <visp3/core/vpConfig.h>
 
-#if defined(VISP_HAVE_REALSENSE2) && defined(VISP_HAVE_CPP11_COMPATIBILITY) &&                                         \
+#if defined(VISP_HAVE_REALSENSE2) && (VISP_CXX_STANDARD >= VISP_CXX_STANDARD_11) &&                                         \
     (defined(VISP_HAVE_X11) || defined(VISP_HAVE_GDI))
 
 #include <visp3/core/vpImage.h>
@@ -55,6 +55,7 @@
 #include <pcl/visualization/cloud_viewer.h>
 #include <pcl/visualization/pcl_visualizer.h>
 #include <thread>
+#include <mutex>
 #endif
 
 #if VISP_HAVE_OPENCV_VERSION >= 0x030000
@@ -73,7 +74,7 @@ bool cancelled = false, update_pointcloud = false;
 class ViewerWorker
 {
 public:
-  explicit ViewerWorker(const bool color_mode, std::mutex &mutex) : m_colorMode(color_mode), m_mutex(mutex) {}
+  explicit ViewerWorker(bool color_mode, std::mutex &mutex) : m_colorMode(color_mode), m_mutex(mutex) {}
 
   void run()
   {
@@ -143,12 +144,12 @@ private:
   std::mutex &m_mutex;
 };
 
-void getPointcloud(const rs2::depth_frame &depth_frame, std::vector<vpColVector> &pointcloud)
+void getPointcloud(const rs2::depth_frame &depth_frame, std::vector<vpColVector> &point_cloud)
 {
   auto vf = depth_frame.as<rs2::video_frame>();
   const int width = vf.get_width();
   const int height = vf.get_height();
-  pointcloud.resize((size_t)(width * height));
+  point_cloud.resize((size_t)(width * height));
 
   rs2::pointcloud pc;
   rs2::points points = pc.calculate(depth_frame);
@@ -167,7 +168,7 @@ void getPointcloud(const rs2::depth_frame &depth_frame, std::vector<vpColVector>
       v[3] = 1.0;
     }
 
-    pointcloud[i] = v;
+    point_cloud[i] = v;
   }
 }
 #endif
@@ -211,7 +212,7 @@ void frame_to_mat(const rs2::frame &f, cv::Mat &img)
   const int size = w * h;
 
   if (f.get_profile().format() == RS2_FORMAT_BGR8) {
-    memcpy(img.ptr<cv::Vec3b>(), f.get_data(), size * 3);
+    memcpy(static_cast<void*>(img.ptr<cv::Vec3b>()), f.get_data(), size * 3);
   } else if (f.get_profile().format() == RS2_FORMAT_RGB8) {
     cv::Mat tmp(h, w, CV_8UC3, (void *)f.get_data(), cv::Mat::AUTO_STEP);
     cv::cvtColor(tmp, img, cv::COLOR_RGB2BGR);
@@ -224,15 +225,29 @@ void frame_to_mat(const rs2::frame &f, cv::Mat &img)
 
 int main(int argc, char *argv[])
 {
+#ifdef VISP_HAVE_PCL
   bool pcl_color = false;
+#endif
   bool show_info = false;
 
   for (int i = 1; i < argc; i++) {
-    if (std::string(argv[i]) == "--pcl_color") {
-      pcl_color = true;
-    } else if (std::string(argv[i]) == "--show_info") {
+    if (std::string(argv[i]) == "--show_info") {
       show_info = true;
     }
+#ifdef VISP_HAVE_PCL
+    else if (std::string(argv[i]) == "--pcl_color") {
+      pcl_color = true;
+    }
+#endif
+    else if (std::string(argv[i]) == "--help" || std::string(argv[i]) == "-h") {
+      std::cout << argv[0] << " [--show_info]"
+#ifdef VISP_HAVE_PCL
+                           << " [--pcl_color]"
+#endif
+                           << " [--help] [-h]"
+                           << "\n";
+      return EXIT_SUCCESS;
+    }
   }
 
   if (show_info) {
@@ -499,7 +514,11 @@ int main(int argc, char *argv[])
 
     auto data = pipe.wait_for_frames();
     frame_to_mat(data.get_color_frame(), mat_color);
+#if (RS2_API_VERSION >= ((2 * 10000) + (16 * 100) + 0))
+    frame_to_mat(data.get_depth_frame().apply_filter(color_map), mat_depth);
+#else
     frame_to_mat(color_map(data.get_depth_frame()), mat_depth);
+#endif
     frame_to_mat(data.first(RS2_STREAM_INFRARED), mat_infrared);
 
     cv::imshow("OpenCV color", mat_color);
@@ -515,6 +534,55 @@ int main(int argc, char *argv[])
             << " ms ; Median time: " << vpMath::getMedian(time_vector) << " ms" << std::endl;
 #endif
 
+#ifdef VISP_HAVE_PCL
+  //Pointcloud acquisition using std::vector<vpColVector> + visualization
+  //See issue #355
+  ViewerWorker viewer_colvector2(false, mutex);
+  std::thread viewer_colvector_thread2(&ViewerWorker::run, &viewer_colvector2);
+  cancelled = false;
+
+  rs.close();
+  config.disable_all_streams();
+  config.enable_stream(RS2_STREAM_COLOR, 640, 480, RS2_FORMAT_RGBA8, 60);
+  config.enable_stream(RS2_STREAM_DEPTH, 640, 480, RS2_FORMAT_Z16, 60);
+  config.enable_stream(RS2_STREAM_INFRARED, 640, 480, RS2_FORMAT_Y8, 60);
+  rs.open(config);
+
+  time_vector.clear();
+  t_begin = vpTime::measureTimeMs();
+  while (vpTime::measureTimeMs() - t_begin < 10000) {
+    double t = vpTime::measureTimeMs();
+    rs.acquire(NULL, NULL, &pointcloud_colvector, NULL, NULL);
+
+    {
+      std::lock_guard<std::mutex> lock(mutex);
+      pointcloud->width = 640;
+      pointcloud->height = 480;
+      pointcloud->points.resize(pointcloud_colvector.size());
+      for (size_t i = 0; i < pointcloud_colvector.size(); i++) {
+        pointcloud->points[(size_t)i].x = pointcloud_colvector[i][0];
+        pointcloud->points[(size_t)i].y = pointcloud_colvector[i][1];
+        pointcloud->points[(size_t)i].z = pointcloud_colvector[i][2];
+      }
+
+      update_pointcloud = true;
+    }
+
+    time_vector.push_back(vpTime::measureTimeMs() - t);
+    vpTime::wait(t, 30.0);
+  }
+
+  {
+    std::lock_guard<std::mutex> lock(mutex);
+    cancelled = true;
+  }
+
+  viewer_colvector_thread2.join();
+
+  std::cout << "Acquisition4 - Mean time: " << vpMath::getMean(time_vector)
+            << " ms ; Median time: " << vpMath::getMedian(time_vector) << " ms" << std::endl;
+#endif
+
   return EXIT_SUCCESS;
 }
 
@@ -524,8 +592,8 @@ int main()
 #if !defined(VISP_HAVE_REALSENSE2)
   std::cout << "Install librealsense2 to make this test work." << std::endl;
 #endif
-#if !defined(VISP_HAVE_CPP11_COMPATIBILITY)
-  std::cout << "Build ViSP with C++11 compiler flag (cmake -DUSE_CPP11=ON) "
+#if !(VISP_CXX_STANDARD >= VISP_CXX_STANDARD_11)
+  std::cout << "Build ViSP with c++11 or higher compiler flag (cmake -DUSE_CXX_STANDARD=11) "
                "to make this test work"
             << std::endl;
 #endif
