@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@
 #include <visp3/io/vpImageIo.h>
 #include <visp3/sensor/vpRealSense.h>
 
-#if defined(VISP_HAVE_REALSENSE) && defined(VISP_HAVE_CPP11_COMPATIBILITY) &&                                          \
+#if defined(VISP_HAVE_REALSENSE) && (VISP_CXX_STANDARD >= VISP_CXX_STANDARD_11) &&                                          \
     (defined(VISP_HAVE_X11) || defined(VISP_HAVE_GDI))
 #include <mutex>
 #include <thread>
@@ -68,7 +68,7 @@ bool cancelled = false, update_pointcloud = false;
 class ViewerWorker
 {
 public:
-  explicit ViewerWorker(const bool color_mode, std::mutex &mutex) : m_colorMode(color_mode), m_mutex(mutex) {}
+  explicit ViewerWorker(bool color_mode, std::mutex &mutex) : m_colorMode(color_mode), m_mutex(mutex) {}
 
   void run()
   {
@@ -142,7 +142,7 @@ private:
 void test_R200(vpRealSense &rs, const std::map<rs::stream, bool> &enables,
                const std::map<rs::stream, vpRealSense::vpRsStreamParams> &params,
                const std::map<rs::option, double> &options, const std::string &title,
-               const bool depth_color_visualization = false, const rs::stream &color_stream = rs::stream::color,
+               bool depth_color_visualization = false, const rs::stream &color_stream = rs::stream::color,
                const rs::stream &depth_stream = rs::stream::depth,
                const rs::stream &infrared2_stream = rs::stream::infrared2, bool display_pcl = false,
                bool pcl_color = false)
@@ -568,8 +568,8 @@ int main()
   std::cout << "Install RealSense SDK to make this test working. X11 or GDI "
                "are needed also."
             << std::endl;
-#elif !defined(VISP_HAVE_CPP11_COMPATIBILITY)
-  std::cout << "Build ViSP with c++11 compiler flag (cmake -DUSE_CPP11=ON) "
+#elif !(VISP_CXX_STANDARD >= VISP_CXX_STANDARD_11)
+  std::cout << "Build ViSP with c++11 or higher compiler flag (cmake -DUSE_CXX_STANDARD=11) "
                "to make this test working"
             << std::endl;
 #elif !defined(VISP_HAVE_X11) && !defined(VISP_HAVE_GDI)
