@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2016 by INRIA. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -11,25 +11,23 @@
  * distribution for additional information about the GNU GPL.
  *
  * For using ViSP with software that can not be combined with the GNU
- * GPL, please contact INRIA about acquiring a ViSP Professional
+ * GPL, please contact Inria about acquiring a ViSP Professional
  * Edition License.
  *
- * See https://visp.inria.fr/download/ for more information.
+ * See http://visp.inria.fr for more information.
  *
  * This software was developed at:
- * INRIA Rennes - Bretagne Atlantique
+ * Inria Rennes - Bretagne Atlantique
  * Campus Universitaire de Beaulieu
  * 35042 Rennes Cedex
  * France
- * http://www.irisa.fr/lagadic
  *
  * If you have questions regarding the use of this file, please contact
- * INRIA at visp@inria.fr
+ * Inria at visp@inria.fr
  *
  * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
  * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  *
- *
  * Description:
  * Model-based edge klt tracker with multiple cameras.
  *
@@ -43,14 +41,16 @@
  \brief Model-based edge klt tracker with multiple cameras.
 */
 
-#ifndef __vpMbEdgeKltMultiTracker_h__
-#define __vpMbEdgeKltMultiTracker_h__
+#ifndef _vpMbEdgeKltMultiTracker_h_
+#define _vpMbEdgeKltMultiTracker_h_
 
 #include <iostream>
 #include <vector>
 
 #include <visp3/core/vpConfig.h>
 
+#if defined(VISP_BUILD_DEPRECATED_FUNCTIONS)
+
 #if defined(VISP_HAVE_MODULE_KLT) && defined(VISP_HAVE_OPENCV) && (VISP_HAVE_OPENCV_VERSION >= 0x020100)
 
 #include <visp3/mbt/vpMbEdgeMultiTracker.h>
@@ -59,12 +59,14 @@
 /*!
   \class vpMbEdgeKltMultiTracker
   \ingroup group_mbt_trackers
+  \warning This class is deprecated for user usage. You should rather use the high level
+  vpMbGenericTracker class.
   \warning This class is only available if OpenCV is installed, and used.
 
   \brief Hybrid stereo (or more) tracker based on moving-edges and keypoints
   tracked using KLT tracker.
 
-  The \ref tutorial-tracking-mb-stereo is a good starting point to use this
+  The \ref tutorial-tracking-mb-stereo-deprecated is a good starting point to use this
   class.
 
   The tracker requires the knowledge of the 3D model that could be provided in
@@ -72,7 +74,7 @@
   may also use an xml file used to tune the behavior of the tracker and an
   init file used to compute the pose at the very first image.
 */
-class VISP_EXPORT vpMbEdgeKltMultiTracker : public vpMbEdgeMultiTracker, public vpMbKltMultiTracker
+class VISP_EXPORT vp_deprecated vpMbEdgeKltMultiTracker : public vpMbEdgeMultiTracker, public vpMbKltMultiTracker
 {
 protected:
   //! Factor for KLT trackers.
@@ -80,9 +82,9 @@ protected:
   //! Factor for edge trackers.
   double m_factorMBT;
   //! The threshold used in the robust estimation of KLT.
-  double thresholdKLT;
+  double m_thresholdKLT;
   //! The threshold used in the robust estimation of MBT.
-  double thresholdMBT;
+  double m_thresholdMBT;
 
   //! Map of camera transformation matrix between the current camera frame to
   //! the reference camera frame (cCurrent_M_cRef)
@@ -103,35 +105,44 @@ protected:
 
 public:
   vpMbEdgeKltMultiTracker();
-  vpMbEdgeKltMultiTracker(const unsigned int nbCameras);
+  vpMbEdgeKltMultiTracker(unsigned int nbCameras);
   vpMbEdgeKltMultiTracker(const std::vector<std::string> &cameraNames);
 
   virtual ~vpMbEdgeKltMultiTracker();
 
-  virtual void display(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo_, const vpCameraParameters &cam_,
-                       const vpColor &col, const unsigned int thickness = 1, const bool displayFullModel = false);
+  virtual void display(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo, const vpCameraParameters &cam,
+                       const vpColor &col, unsigned int thickness = 1, bool displayFullModel = false);
 
-  virtual void display(const vpImage<vpRGBa> &I, const vpHomogeneousMatrix &cMo_, const vpCameraParameters &cam_,
-                       const vpColor &col, const unsigned int thickness = 1, const bool displayFullModel = false);
+  virtual void display(const vpImage<vpRGBa> &I, const vpHomogeneousMatrix &cMo, const vpCameraParameters &cam,
+                       const vpColor &col, unsigned int thickness = 1, bool displayFullModel = false);
 
   virtual void display(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2,
                        const vpHomogeneousMatrix &c1Mo, const vpHomogeneousMatrix &c2Mo, const vpCameraParameters &cam1,
-                       const vpCameraParameters &cam2, const vpColor &color, const unsigned int thickness = 1,
-                       const bool displayFullModel = false);
+                       const vpCameraParameters &cam2, const vpColor &color, unsigned int thickness = 1,
+                       bool displayFullModel = false);
 
   virtual void display(const vpImage<vpRGBa> &I1, const vpImage<vpRGBa> &I2, const vpHomogeneousMatrix &c1Mo,
                        const vpHomogeneousMatrix &c2Mo, const vpCameraParameters &cam1, const vpCameraParameters &cam2,
-                       const vpColor &color, const unsigned int thickness = 1, const bool displayFullModel = false);
+                       const vpColor &color, unsigned int thickness = 1,  bool displayFullModel = false);
 
   virtual void display(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
                        const std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses,
                        const std::map<std::string, vpCameraParameters> &mapOfCameraParameters, const vpColor &col,
-                       const unsigned int thickness = 1, const bool displayFullModel = false);
+                       unsigned int thickness = 1, bool displayFullModel = false);
 
   virtual void display(const std::map<std::string, const vpImage<vpRGBa> *> &mapOfImages,
                        const std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses,
                        const std::map<std::string, vpCameraParameters> &mapOfCameraParameters, const vpColor &col,
-                       const unsigned int thickness = 1, const bool displayFullModel = false);
+                       unsigned int thickness = 1, bool displayFullModel = false);
+
+  virtual std::vector<std::vector<double> > getModelForDisplay(unsigned int, unsigned int,
+                                                               const vpHomogeneousMatrix &,
+                                                               const vpCameraParameters &,
+                                                               bool =false)
+  {
+    std::cerr << "Not implemented. Deprecated class." << std::endl;
+    return std::vector<std::vector<double> >();
+  }
 
   virtual std::vector<std::string> getCameraNames() const;
 
@@ -176,7 +187,7 @@ public:
 
   using vpMbKltMultiTracker::getPose;
   virtual void getPose(vpHomogeneousMatrix &c1Mo, vpHomogeneousMatrix &c2Mo) const;
-  virtual void getPose(const std::string &cameraName, vpHomogeneousMatrix &cMo_) const;
+  virtual void getPose(const std::string &cameraName, vpHomogeneousMatrix &cMo) const;
   virtual void getPose(std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses) const;
 
   virtual inline vpColVector getError() const { return m_error_hybridMulti; }
@@ -189,17 +200,18 @@ public:
   virtual void initClick(const vpImage<unsigned char> &I, const std::vector<vpPoint> &points3D_list,
                          const std::string &displayFile = "");
 
-  virtual void initClick(const vpImage<unsigned char> &I, const std::string &initFile, const bool displayHelp = false);
+  virtual void initClick(const vpImage<unsigned char> &I, const std::string &initFile, bool displayHelp = false,
+                         const vpHomogeneousMatrix &T=vpHomogeneousMatrix());
 
   virtual void initClick(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2,
-                         const std::string &initFile1, const std::string &initFile2, const bool displayHelp = false,
-                         const bool firstCameraIsReference = true);
+                         const std::string &initFile1, const std::string &initFile2, bool displayHelp = false,
+                         bool firstCameraIsReference = true);
 
   virtual void initClick(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
-                         const std::string &initFile, const bool displayHelp = false);
+                         const std::string &initFile, bool displayHelp = false);
 
   virtual void initClick(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
-                         const std::map<std::string, std::string> &mapOfInitFiles, const bool displayHelp = false);
+                         const std::map<std::string, std::string> &mapOfInitFiles, bool displayHelp = false);
 #endif
 
   virtual void initCircle(const vpPoint &, const vpPoint &, const vpPoint &, const double, const int,
@@ -212,9 +224,9 @@ public:
 
   virtual void initFromPose(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2,
                             const vpHomogeneousMatrix &c1Mo, const vpHomogeneousMatrix &c2Mo,
-                            const bool firstCameraIsReference = true);
+                            bool firstCameraIsReference = true);
   virtual void initFromPose(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
-                            const vpHomogeneousMatrix &cMo_);
+                            const vpHomogeneousMatrix &cMo);
   virtual void initFromPose(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
                             const std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses);
 
@@ -224,22 +236,24 @@ public:
   virtual void loadConfigFile(const std::string &configFile);
 
   virtual void loadConfigFile(const std::string &configFile1, const std::string &configFile2,
-                              const bool firstCameraIsReference = true);
+                              bool firstCameraIsReference = true);
 
   virtual void loadConfigFile(const std::map<std::string, std::string> &mapOfConfigFiles);
 
-  virtual void loadModel(const std::string &modelFile, const bool verbose = false);
+  virtual void loadModel(const std::string &modelFile, bool verbose = false,
+                         const vpHomogeneousMatrix &T=vpHomogeneousMatrix());
 
   virtual void reInitModel(const vpImage<unsigned char> &I, const std::string &cad_name,
-                           const vpHomogeneousMatrix &cMo_, const bool verbose = false);
+                           const vpHomogeneousMatrix &cMo, bool verbose = false,
+                           const vpHomogeneousMatrix &T=vpHomogeneousMatrix());
   virtual void reInitModel(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2,
                            const std::string &cad_name, const vpHomogeneousMatrix &c1Mo,
-                           const vpHomogeneousMatrix &c2Mo, const bool verbose = false,
-                           const bool firstCameraIsReference = true);
+                           const vpHomogeneousMatrix &c2Mo, bool verbose = false,
+                           bool firstCameraIsReference = true);
   virtual void reInitModel(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
                            const std::string &cad_name,
                            const std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses,
-                           const bool verbose = false);
+                           bool verbose = false);
 
   virtual void resetTracker();
 
@@ -249,7 +263,7 @@ public:
   virtual void setCameraParameters(const vpCameraParameters &camera);
 
   virtual void setCameraParameters(const vpCameraParameters &camera1, const vpCameraParameters &camera2,
-                                   const bool firstCameraIsReference = true);
+                                   bool firstCameraIsReference = true);
 
   virtual void setCameraParameters(const std::string &cameraName, const vpCameraParameters &camera);
 
@@ -266,7 +280,7 @@ public:
 
   virtual void setCovarianceComputation(const bool &flag);
 
-  virtual void setDisplayFeatures(const bool displayF);
+  virtual void setDisplayFeatures(bool displayF);
 
   virtual void setFarClippingDistance(const double &dist);
   virtual void setFarClippingDistance(const std::string &cameraName, const double &dist);
@@ -309,34 +323,35 @@ public:
   void setNbRayCastingAttemptsForVisibility(const unsigned int &attempts);
 #endif
 
-  virtual void setLod(const bool useLod, const std::string &name = "");
-  virtual void setLod(const bool useLod, const std::string &cameraName, const std::string &name);
+  virtual void setLod(bool useLod, const std::string &name = "");
+  virtual void setLod(bool useLod, const std::string &cameraName, const std::string &name);
 
-  virtual void setMinLineLengthThresh(const double minLineLengthThresh, const std::string &name = "");
-  virtual void setMinLineLengthThresh(const double minLineLengthThresh, const std::string &cameraName,
+  virtual void setMinLineLengthThresh(double minLineLengthThresh, const std::string &name = "");
+  virtual void setMinLineLengthThresh(double minLineLengthThresh, const std::string &cameraName,
                                       const std::string &name);
 
-  virtual void setMinPolygonAreaThresh(const double minPolygonAreaThresh, const std::string &name = "");
-  virtual void setMinPolygonAreaThresh(const double minPolygonAreaThresh, const std::string &cameraName,
+  virtual void setMinPolygonAreaThresh(double minPolygonAreaThresh, const std::string &name = "");
+  virtual void setMinPolygonAreaThresh(double minPolygonAreaThresh, const std::string &cameraName,
                                        const std::string &name);
 
   virtual void setNearClippingDistance(const double &dist);
   virtual void setNearClippingDistance(const std::string &cameraName, const double &dist);
 
-  virtual void setOgreShowConfigDialog(const bool showConfigDialog);
+  virtual void setOgreShowConfigDialog(bool showConfigDialog);
 
   virtual void setOgreVisibilityTest(const bool &v);
 
   virtual void setOptimizationMethod(const vpMbtOptimizationMethod &opt);
 
   virtual void setPose(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo);
+  virtual void setPose(const vpImage<vpRGBa> &I_color, const vpHomogeneousMatrix &cMo);
 
   virtual void setPose(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2,
                        const vpHomogeneousMatrix &c1Mo, const vpHomogeneousMatrix &c2Mo,
-                       const bool firstCameraIsReference = true);
+                       bool firstCameraIsReference = true);
 
   virtual void setPose(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
-                       const vpHomogeneousMatrix &cMo_);
+                       const vpHomogeneousMatrix &cMo);
 
   virtual void setPose(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
                        const std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses);
@@ -347,11 +362,12 @@ public:
 
   virtual void setScanLineVisibilityTest(const bool &v);
 
-  virtual void setThresholdAcceptation(const double th);
+  virtual void setThresholdAcceptation(double th);
 
   virtual void testTracking();
 
   virtual void track(const vpImage<unsigned char> &I);
+  virtual void track(const vpImage<vpRGBa> &I_color);
   virtual void track(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2);
   virtual void track(std::map<std::string, const vpImage<unsigned char> *> &mapOfImages);
 
@@ -363,7 +379,7 @@ protected:
   using vpMbKltMultiTracker::computeVVS;
 
   virtual void computeVVS(std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
-                          const unsigned int lvl = 0);
+                          unsigned int lvl = 0);
   virtual void computeVVSInit();
   virtual void computeVVSInteractionMatrixAndResidu();
   using vpMbEdgeMultiTracker::computeVVSInteractionMatrixAndResidu;
@@ -379,7 +395,7 @@ protected:
   // Same thing as computeVVS
   using vpMbKltMultiTracker::postTracking;
 
-  virtual void postTracking(std::map<std::string, const vpImage<unsigned char> *> &mapOfImages, const unsigned int lvl);
+  virtual void postTracking(std::map<std::string, const vpImage<unsigned char> *> &mapOfImages);
 
   virtual void reinit(/*const vpImage<unsigned char>& I*/);
 
@@ -387,4 +403,5 @@ protected:
 };
 
 #endif // VISP_HAVE_OPENCV
-#endif //__vpMbEdgeKltMultiTracker_h__
+#endif //#if defined(VISP_BUILD_DEPRECATED_FUNCTIONS)
+#endif //_vpMbEdgeKltMultiTracker_h_
