@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2016 by INRIA. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -11,25 +11,23 @@
  * distribution for additional information about the GNU GPL.
  *
  * For using ViSP with software that can not be combined with the GNU
- * GPL, please contact INRIA about acquiring a ViSP Professional
+ * GPL, please contact Inria about acquiring a ViSP Professional
  * Edition License.
  *
- * See https://visp.inria.fr/download/ for more information.
+ * See http://visp.inria.fr for more information.
  *
  * This software was developed at:
- * INRIA Rennes - Bretagne Atlantique
+ * Inria Rennes - Bretagne Atlantique
  * Campus Universitaire de Beaulieu
  * 35042 Rennes Cedex
  * France
- * http://www.irisa.fr/lagadic
  *
  * If you have questions regarding the use of this file, please contact
- * INRIA at visp@inria.fr
+ * Inria at visp@inria.fr
  *
  * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
  * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  *
- *
  * Description:
  * Model-based edge tracker with multiple cameras.
  *
@@ -43,22 +41,28 @@
  \brief Model-based edge tracker with multiple cameras.
 */
 
-#ifndef __vpMbEdgeMultiTracker_h__
-#define __vpMbEdgeMultiTracker_h__
+#ifndef _vpMbEdgeMultiTracker_h_
+#define _vpMbEdgeMultiTracker_h_
+
+#include <visp3/core/vpConfig.h>
 
 #include <iostream>
 #include <vector>
 
 #include <visp3/mbt/vpMbEdgeTracker.h>
 
+#if defined(VISP_BUILD_DEPRECATED_FUNCTIONS)
+
 /*!
   \class vpMbEdgeMultiTracker
   \ingroup group_mbt_trackers
   \brief Make the complete stereo (or more) tracking of an object by using its
   CAD model.
+  \warning This class is deprecated for user usage. You should rather use the high level
+  vpMbGenericTracker class.
 
   This class allows to track an object or a scene given its 3D model.
-  The \ref tutorial-tracking-mb-stereo is also a good starting point to use
+  The \ref tutorial-tracking-mb-stereo-deprecated is also a good starting point to use
   this class.
 
   The tracker requires the knowledge of the 3D model that could be provided in
@@ -66,7 +70,7 @@
   may also use an xml file used to tune the behavior of the tracker and an
   init file used to compute the pose at the very first image.
 */
-class VISP_EXPORT vpMbEdgeMultiTracker : public vpMbEdgeTracker
+class VISP_EXPORT vp_deprecated vpMbEdgeMultiTracker : public vpMbEdgeTracker
 {
 protected:
   //! Map of camera transformation matrix between the current camera frame to
@@ -95,7 +99,7 @@ public:
   vpMbEdgeMultiTracker();
   // Constructor with a specified number of cameras, camera names are
   // generated
-  explicit vpMbEdgeMultiTracker(const unsigned int nbCameras);
+  explicit vpMbEdgeMultiTracker(unsigned int nbCameras);
   // Constructor with a specified list of camera names
   explicit vpMbEdgeMultiTracker(const std::vector<std::string> &cameraNames);
 
@@ -103,30 +107,39 @@ public:
 
   /** @name Inherited functionalities from vpMbEdgeMultiTracker */
   //@{
-  virtual void display(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo_, const vpCameraParameters &cam_,
-                       const vpColor &col, const unsigned int thickness = 1, const bool displayFullModel = false);
+  virtual void display(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo, const vpCameraParameters &cam,
+                       const vpColor &col, unsigned int thickness = 1, bool displayFullModel = false);
 
-  virtual void display(const vpImage<vpRGBa> &I, const vpHomogeneousMatrix &cMo_, const vpCameraParameters &cam_,
-                       const vpColor &col, const unsigned int thickness = 1, const bool displayFullModel = false);
+  virtual void display(const vpImage<vpRGBa> &I, const vpHomogeneousMatrix &cMo, const vpCameraParameters &cam,
+                       const vpColor &col, unsigned int thickness = 1, bool displayFullModel = false);
 
   virtual void display(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2,
                        const vpHomogeneousMatrix &c1Mo, const vpHomogeneousMatrix &c2Mo, const vpCameraParameters &cam1,
-                       const vpCameraParameters &cam2, const vpColor &col, const unsigned int thickness = 1,
-                       const bool displayFullModel = false);
+                       const vpCameraParameters &cam2, const vpColor &col, unsigned int thickness = 1,
+                       bool displayFullModel = false);
 
   virtual void display(const vpImage<vpRGBa> &I1, const vpImage<vpRGBa> &I2, const vpHomogeneousMatrix &c1Mo,
                        const vpHomogeneousMatrix &c2Mo, const vpCameraParameters &cam1, const vpCameraParameters &cam2,
-                       const vpColor &col, const unsigned int thickness = 1, const bool displayFullModel = false);
+                       const vpColor &col, unsigned int thickness = 1, bool displayFullModel = false);
 
   virtual void display(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
                        const std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses,
                        const std::map<std::string, vpCameraParameters> &mapOfCameraParameters, const vpColor &col,
-                       const unsigned int thickness = 1, const bool displayFullModel = false);
+                       unsigned int thickness = 1, bool displayFullModel = false);
 
   virtual void display(const std::map<std::string, const vpImage<vpRGBa> *> &mapOfImages,
                        const std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses,
                        const std::map<std::string, vpCameraParameters> &mapOfCameraParameters, const vpColor &col,
-                       const unsigned int thickness = 1, const bool displayFullModel = false);
+                       unsigned int thickness = 1, bool displayFullModel = false);
+
+  virtual std::vector<std::vector<double> > getModelForDisplay(unsigned int, unsigned int,
+                                                               const vpHomogeneousMatrix &,
+                                                               const vpCameraParameters &,
+                                                               bool =false)
+  {
+    std::cerr << "Not implemented. Deprecated class." << std::endl;
+    return std::vector<std::vector<double> >();
+  }
 
   virtual std::vector<std::string> getCameraNames() const;
 
@@ -142,25 +155,25 @@ public:
   virtual vpMbHiddenFaces<vpMbtPolygon> &getFaces(const std::string &cameraName);
   virtual std::map<std::string, vpMbHiddenFaces<vpMbtPolygon> > getFaces() const;
 
-  void getLcircle(std::list<vpMbtDistanceCircle *> &circlesList, const unsigned int level = 0) const;
+  void getLcircle(std::list<vpMbtDistanceCircle *> &circlesList, unsigned int level = 0) const;
   virtual void getLcircle(const std::string &cameraName, std::list<vpMbtDistanceCircle *> &circlesList,
-                          const unsigned int level = 0) const;
+                          unsigned int level = 0) const;
 
-  void getLcylinder(std::list<vpMbtDistanceCylinder *> &cylindersList, const unsigned int level = 0) const;
+  void getLcylinder(std::list<vpMbtDistanceCylinder *> &cylindersList, unsigned int level = 0) const;
   virtual void getLcylinder(const std::string &cameraName, std::list<vpMbtDistanceCylinder *> &cylindersList,
-                            const unsigned int level = 0) const;
+                            unsigned int level = 0) const;
 
-  void getLline(std::list<vpMbtDistanceLine *> &linesList, const unsigned int level = 0) const;
+  void getLline(std::list<vpMbtDistanceLine *> &linesList, unsigned int level = 0) const;
   virtual void getLline(const std::string &cameraName, std::list<vpMbtDistanceLine *> &linesList,
-                        const unsigned int level = 0) const;
+                        unsigned int level = 0) const;
 
   virtual void getMovingEdge(vpMe &p_me) const;
   virtual vpMe getMovingEdge() const;
   virtual void getMovingEdge(const std::string &cameraName, vpMe &p_me) const;
   virtual vpMe getMovingEdge(const std::string &cameraName) const;
 
-  virtual unsigned int getNbPoints(const unsigned int level = 0) const;
-  virtual unsigned int getNbPoints(const std::string &cameraName, const unsigned int level = 0) const;
+  virtual unsigned int getNbPoints(unsigned int level = 0) const;
+  virtual unsigned int getNbPoints(const std::string &cameraName, unsigned int level = 0) const;
 
   virtual unsigned int getNbPolygon() const;
   virtual unsigned int getNbPolygon(const std::string &cameraName) const;
@@ -175,7 +188,7 @@ public:
 
   using vpMbTracker::getPose;
   virtual void getPose(vpHomogeneousMatrix &c1Mo, vpHomogeneousMatrix &c2Mo) const;
-  virtual void getPose(const std::string &cameraName, vpHomogeneousMatrix &cMo_) const;
+  virtual void getPose(const std::string &cameraName, vpHomogeneousMatrix &cMo) const;
   virtual void getPose(std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses) const;
 
   virtual inline vpColVector getError() const { return m_error_edgeMulti; }
@@ -185,53 +198,58 @@ public:
   void init(const vpImage<unsigned char> &I);
 
 #ifdef VISP_HAVE_MODULE_GUI
+  using vpMbTracker::initClick;
   virtual void initClick(const vpImage<unsigned char> &I, const std::vector<vpPoint> &points3D_list,
                          const std::string &displayFile = "");
 
-  virtual void initClick(const vpImage<unsigned char> &I, const std::string &initFile, const bool displayHelp = false);
+  virtual void initClick(const vpImage<unsigned char> &I, const std::string &initFile, bool displayHelp = false,
+                         const vpHomogeneousMatrix &T=vpHomogeneousMatrix());
 
   virtual void initClick(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2,
-                         const std::string &initFile1, const std::string &initFile2, const bool displayHelp = false,
-                         const bool firstCameraIsReference = true);
+                         const std::string &initFile1, const std::string &initFile2, bool displayHelp = false,
+                         bool firstCameraIsReference = true);
 
   virtual void initClick(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
-                         const std::string &initFile, const bool displayHelp = false);
+                         const std::string &initFile, bool displayHelp = false);
 
   virtual void initClick(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
-                         const std::map<std::string, std::string> &mapOfInitFiles, const bool displayHelp = false);
+                         const std::map<std::string, std::string> &mapOfInitFiles, bool displayHelp = false);
 #endif
 
+  using vpMbTracker::initFromPose;
   virtual void initFromPose(const vpImage<unsigned char> &I, const std::string &initFile);
   virtual void initFromPose(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo);
   virtual void initFromPose(const vpImage<unsigned char> &I, const vpPoseVector &cPo);
 
   virtual void initFromPose(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2,
                             const vpHomogeneousMatrix &c1Mo, const vpHomogeneousMatrix &c2Mo,
-                            const bool firstCameraIsReference = true);
+                            bool firstCameraIsReference = true);
   virtual void initFromPose(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
-                            const vpHomogeneousMatrix &cMo_);
+                            const vpHomogeneousMatrix &cMo);
   virtual void initFromPose(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
                             const std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses);
 
   virtual void loadConfigFile(const std::string &configFile);
 
   virtual void loadConfigFile(const std::string &configFile1, const std::string &configFile2,
-                              const bool firstCameraIsReference = true);
+                              bool firstCameraIsReference = true);
 
   virtual void loadConfigFile(const std::map<std::string, std::string> &mapOfConfigFiles);
 
-  virtual void loadModel(const std::string &modelFile, const bool verbose = false);
+  virtual void loadModel(const std::string &modelFile, bool verbose = false,
+                         const vpHomogeneousMatrix &T=vpHomogeneousMatrix());
 
   virtual void reInitModel(const vpImage<unsigned char> &I, const std::string &cad_name,
-                           const vpHomogeneousMatrix &cMo_, const bool verbose = false);
+                           const vpHomogeneousMatrix &cMo, bool verbose = false,
+                           const vpHomogeneousMatrix &T=vpHomogeneousMatrix());
   virtual void reInitModel(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2,
                            const std::string &cad_name, const vpHomogeneousMatrix &c1Mo,
-                           const vpHomogeneousMatrix &c2Mo, const bool verbose = false,
-                           const bool firstCameraIsReference = true);
+                           const vpHomogeneousMatrix &c2Mo, bool verbose = false,
+                           bool firstCameraIsReference = true);
   virtual void reInitModel(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
                            const std::string &cad_name,
                            const std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses,
-                           const bool verbose = false);
+                           bool verbose = false);
 
   virtual void resetTracker();
 
@@ -241,7 +259,7 @@ public:
   virtual void setCameraParameters(const vpCameraParameters &camera);
 
   virtual void setCameraParameters(const vpCameraParameters &camera1, const vpCameraParameters &camera2,
-                                   const bool firstCameraIsReference = true);
+                                   bool firstCameraIsReference = true);
 
   virtual void setCameraParameters(const std::string &cameraName, const vpCameraParameters &camera);
 
@@ -258,12 +276,12 @@ public:
 
   virtual void setCovarianceComputation(const bool &flag);
 
-  virtual void setDisplayFeatures(const bool displayF);
+  virtual void setDisplayFeatures(bool displayF);
 
   virtual void setFarClippingDistance(const double &dist);
   virtual void setFarClippingDistance(const std::string &cameraName, const double &dist);
 
-  virtual void setGoodMovingEdgesRatioThreshold(const double threshold);
+  virtual void setGoodMovingEdgesRatioThreshold(double threshold);
 
 #ifdef VISP_HAVE_OGRE
   /*!
@@ -289,19 +307,19 @@ public:
   void setNbRayCastingAttemptsForVisibility(const unsigned int &attempts);
 #endif
 
-  virtual void setLod(const bool useLod, const std::string &name = "");
-  virtual void setLod(const bool useLod, const std::string &cameraName, const std::string &name);
+  virtual void setLod(bool useLod, const std::string &name = "");
+  virtual void setLod(bool useLod, const std::string &cameraName, const std::string &name);
 
-  virtual void setMinLineLengthThresh(const double minLineLengthThresh, const std::string &name = "");
-  virtual void setMinLineLengthThresh(const double minLineLengthThresh, const std::string &cameraName,
+  virtual void setMinLineLengthThresh(double minLineLengthThresh, const std::string &name = "");
+  virtual void setMinLineLengthThresh(double minLineLengthThresh, const std::string &cameraName,
                                       const std::string &name);
 
-  virtual void setMinPolygonAreaThresh(const double minPolygonAreaThresh, const std::string &name = "");
-  virtual void setMinPolygonAreaThresh(const double minPolygonAreaThresh, const std::string &cameraName,
+  virtual void setMinPolygonAreaThresh(double minPolygonAreaThresh, const std::string &name = "");
+  virtual void setMinPolygonAreaThresh(double minPolygonAreaThresh, const std::string &cameraName,
                                        const std::string &name);
 
-  virtual void setMovingEdge(const vpMe &me);
-  virtual void setMovingEdge(const std::string &cameraName, const vpMe &me);
+  virtual void setMovingEdge(const vpMe &moving_edge);
+  virtual void setMovingEdge(const std::string &cameraName, const vpMe &moving_edge);
 
   virtual void setNearClippingDistance(const double &dist);
   virtual void setNearClippingDistance(const std::string &cameraName, const double &dist);
@@ -316,20 +334,21 @@ public:
     Ogre rendering options) when Ogre visibility is enabled. By default, this
     functionality is turned off.
   */
-  virtual void setOgreShowConfigDialog(const bool showConfigDialog);
+  virtual void setOgreShowConfigDialog(bool showConfigDialog);
 
   virtual void setOgreVisibilityTest(const bool &v);
 
   virtual void setOptimizationMethod(const vpMbtOptimizationMethod &opt);
 
   virtual void setPose(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo);
+  virtual void setPose(const vpImage<vpRGBa> &I_color, const vpHomogeneousMatrix &cMo);
 
   virtual void setPose(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2,
                        const vpHomogeneousMatrix &c1Mo, const vpHomogeneousMatrix &c2Mo,
-                       const bool firstCameraIsReference = true);
+                       bool firstCameraIsReference = true);
 
   virtual void setPose(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
-                       const vpHomogeneousMatrix &cMo_);
+                       const vpHomogeneousMatrix &cMo);
 
   virtual void setPose(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
                        const std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses);
@@ -344,7 +363,9 @@ public:
 
   virtual void setUseEdgeTracking(const std::string &name, const bool &useEdgeTracking);
 
+  using vpMbEdgeTracker::track;
   virtual void track(const vpImage<unsigned char> &I);
+  virtual void track(const vpImage<vpRGBa> &I_color);
   virtual void track(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2);
   virtual void track(std::map<std::string, const vpImage<unsigned char> *> &mapOfImages);
   //@}
@@ -358,8 +379,8 @@ protected:
 
   virtual void computeProjectionError();
 
-  virtual void computeVVS(std::map<std::string, const vpImage<unsigned char> *> &mapOfImages, const unsigned int lvl);
-  virtual void computeVVSFirstPhasePoseEstimation(const unsigned int iter, bool &isoJoIdentity_);
+  virtual void computeVVS(std::map<std::string, const vpImage<unsigned char> *> &mapOfImages, unsigned int lvl);
+  virtual void computeVVSFirstPhasePoseEstimation(unsigned int iter, bool &isoJoIdentity_);
   virtual void computeVVSInit();
   virtual void computeVVSInteractionMatrixAndResidu();
   using vpMbEdgeTracker::computeVVSInteractionMatrixAndResidu;
@@ -373,4 +394,5 @@ protected:
   //@}
 };
 
-#endif
+#endif //#if defined(VISP_BUILD_DEPRECATED_FUNCTIONS)
+#endif //_vpMbEdgeMultiTracker_h_
