@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,7 +47,6 @@
 #define vpMbEdgeTracker_HH
 
 #include <visp3/core/vpPoint.h>
-#include <visp3/core/vpXmlParser.h>
 #include <visp3/mbt/vpMbTracker.h>
 #include <visp3/mbt/vpMbtDistanceCircle.h>
 #include <visp3/mbt/vpMbtDistanceCylinder.h>
@@ -90,10 +89,12 @@
   \class vpMbEdgeTracker
   \ingroup group_mbt_trackers
   \brief Make the complete tracking of an object by using its CAD model.
+  \warning This class is deprecated for user usage. You should rather use the high level
+  vpMbGenericTracker class.
 
   This class allows to track an object or a scene given its 3D model. A
   video can be found on YouTube \e https://www.youtube.com/watch?v=UK10KMMJFCI
-. The \ref tutorial-tracking-mb is also a good starting point to use this
+. The \ref tutorial-tracking-mb-deprecated is also a good starting point to use this
 class.
 
   The tracker requires the knowledge of the 3D model that could be provided in
@@ -104,13 +105,12 @@ used to compute the pose at the very first image.
   The following code shows the simplest way to use the tracker.
 
 \code
-#include <visp/vpCameraParameters.h>
-#include <visp/vpDisplayX.h>
-#include <visp/vpException.h>
-#include <visp/vpHomogeneousMatrix.h>
-#include <visp/vpImage.h>
-#include <visp/vpImageIo.h>
-#include <visp/vpMbEdgeTracker.h>
+#include <visp3/core/vpCameraParameters.h>
+#include <visp3/core/vpHomogeneousMatrix.h>
+#include <visp3/core/vpImage.h>
+#include <visp3/gui/vpDisplayX.h>
+#include <visp3/io/vpImageIo.h>
+#include <visp3/mbt/vpMbEdgeTracker.h>
 
 int main()
 {
@@ -127,7 +127,7 @@ int main()
   display.init(I,100,100,"Mb Edge Tracker");
 #endif
 
-#if defined VISP_HAVE_XML2
+#if defined VISP_HAVE_PUGIXML
   tracker.loadConfigFile("cube.xml"); // Load the configuration of the tracker
 #endif
   tracker.getCameraParameters(cam);   // Get the camera parameters used by the tracker (from the configuration file).
@@ -146,9 +146,6 @@ int main()
    vpDisplay::flush(I);
   }
 
-  // Cleanup memory allocated by xml library used to parse the xml config file in vpMbEdgeTracker::loadConfigFile()
-  vpXmlParser::cleanup();
-
   return 0;
 }
 \endcode
@@ -171,8 +168,6 @@ not) them using a vector of booleans, as presented in the following code:
   tracker.loadConfigFile("cube.xml"); // Load the configuration of the tracker
   tracker.getCameraParameters(cam); // Get the camera parameters used by the tracker (from the configuration file).
   ...
-  // Cleanup memory allocated by xml library used to parse the xml config file in vpMbEdgeTracker::loadConfigFile()
-  vpXmlParser::cleanup();
 \endcode
 
   The tracker can also be used without display, in that case the initial pose
@@ -180,11 +175,11 @@ not) them using a vector of booleans, as presented in the following code:
 computed using another method:
 
 \code
-#include <visp/vpCameraParameters.h>
-#include <visp/vpHomogeneousMatrix.h>
-#include <visp/vpImage.h>
-#include <visp/vpImageIo.h>
-#include <visp/vpMbEdgeTracker.h>
+#include <visp3/core/vpCameraParameters.h>
+#include <visp3/core/vpHomogeneousMatrix.h>
+#include <visp3/core/vpImage.h>
+#include <visp3/io/vpImageIo.h>
+#include <visp3/mbt/vpMbEdgeTracker.h>
 
 int main()
 {
@@ -195,7 +190,7 @@ int main()
   //acquire an image
   vpImageIo::read(I, "cube.pgm"); // Example of acquisition
 
-#if defined VISP_HAVE_XML2
+#if defined VISP_HAVE_PUGIXML
   tracker.loadConfigFile("cube.xml"); // Load the configuration of the tracker
 #endif
   // load the 3d model, to read .wrl model coin is required, if coin is not installed .cao file can be used.
@@ -208,9 +203,6 @@ int main()
     tracker.getPose(cMo); // get the pose
   }
 
-  // Cleanup memory allocated by xml library used to parse the xml config file in vpMbEdgeTracker::loadConfigFile()
-  vpXmlParser::cleanup();
-
   return 0;
 }
 \endcode
@@ -219,12 +211,12 @@ int main()
 a given pose:
 
 \code
-#include <visp/vpCameraParameters.h>
-#include <visp/vpDisplayX.h>
-#include <visp/vpHomogeneousMatrix.h>
-#include <visp/vpImage.h>
-#include <visp/vpImageIo.h>
-#include <visp/vpMbEdgeTracker.h>
+#include <visp3/core/vpCameraParameters.h>
+#include <visp3/core/vpHomogeneousMatrix.h>
+#include <visp3/core/vpImage.h>
+#include <visp3/gui/vpDisplayX.h>
+#include <visp3/io/vpImageIo.h>
+#include <visp3/mbt/vpMbEdgeTracker.h>
 
 int main()
 {
@@ -241,7 +233,7 @@ int main()
   display.init(I,100,100,"Mb Edge Tracker");
 #endif
 
-#if defined VISP_HAVE_XML2
+#if defined VISP_HAVE_PUGIXML
   tracker.loadConfigFile("cube.xml"); // Load the configuration of the tracker
 #endif
   tracker.getCameraParameters(cam); // Get the camera parameters used by the tracker (from the configuration file).
@@ -257,9 +249,6 @@ int main()
     vpDisplay::flush(I);
   }
 
-  // Cleanup memory allocated by xml library used to parse the xml config file in vpMbEdgeTracker::loadConfigFile()
-  vpXmlParser::cleanup();
-
   return 0;
 }
 \endcode
@@ -340,6 +329,8 @@ protected:
   vpColVector m_weightedError_edge;
   //! Robust
   vpRobust m_robust_edge;
+  //! Display features
+  std::vector<std::vector<double> > m_featuresToBeDisplayedEdge;
 
 public:
   vpMbEdgeTracker();
@@ -348,14 +339,19 @@ public:
   /** @name Inherited functionalities from vpMbEdgeTracker */
   //@{
 
-  void display(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo, const vpCameraParameters &cam,
-               const vpColor &col, const unsigned int thickness = 1, const bool displayFullModel = false);
-  void display(const vpImage<vpRGBa> &I, const vpHomogeneousMatrix &cMo, const vpCameraParameters &cam,
-               const vpColor &col, const unsigned int thickness = 1, const bool displayFullModel = false);
+  virtual void display(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo, const vpCameraParameters &cam,
+                       const vpColor &col, unsigned int thickness = 1, bool displayFullModel = false);
+  virtual void display(const vpImage<vpRGBa> &I, const vpHomogeneousMatrix &cMo, const vpCameraParameters &cam,
+                       const vpColor &col, unsigned int thickness = 1, bool displayFullModel = false);
 
-  void getLline(std::list<vpMbtDistanceLine *> &linesList, const unsigned int level = 0) const;
-  void getLcircle(std::list<vpMbtDistanceCircle *> &circlesList, const unsigned int level = 0) const;
-  void getLcylinder(std::list<vpMbtDistanceCylinder *> &cylindersList, const unsigned int level = 0) const;
+  void getLline(std::list<vpMbtDistanceLine *> &linesList, unsigned int level = 0) const;
+  void getLcircle(std::list<vpMbtDistanceCircle *> &circlesList, unsigned int level = 0) const;
+  void getLcylinder(std::list<vpMbtDistanceCylinder *> &cylindersList, unsigned int level = 0) const;
+
+  virtual std::vector<std::vector<double> > getModelForDisplay(unsigned int width, unsigned int height,
+                                                               const vpHomogeneousMatrix &cMo,
+                                                               const vpCameraParameters &cam,
+                                                               bool displayFullModel=false);
 
   /*!
     Get the moving edge parameters.
@@ -371,7 +367,7 @@ public:
   */
   virtual inline vpMe getMovingEdge() const { return this->me; }
 
-  virtual unsigned int getNbPoints(const unsigned int level = 0) const;
+  virtual unsigned int getNbPoints(unsigned int level = 0) const;
 
   /*!
     Return the scales levels used for the tracking.
@@ -395,35 +391,34 @@ public:
   virtual inline vpColVector getRobustWeights() const { return m_w_edge; }
 
   void loadConfigFile(const std::string &configFile);
-  void loadConfigFile(const char *configFile);
+
   virtual void reInitModel(const vpImage<unsigned char> &I, const std::string &cad_name,
-                           const vpHomogeneousMatrix &cMo_, const bool verbose = false);
-  void reInitModel(const vpImage<unsigned char> &I, const char *cad_name, const vpHomogeneousMatrix &cMo,
-                   const bool verbose = false);
+                           const vpHomogeneousMatrix &cMo, bool verbose = false,
+                           const vpHomogeneousMatrix &T=vpHomogeneousMatrix());
   void resetTracker();
 
   /*!
     Set the camera parameters.
 
-    \param camera : the new camera parameters
+    \param cam : The new camera parameters.
   */
-  virtual void setCameraParameters(const vpCameraParameters &camera)
+  virtual void setCameraParameters(const vpCameraParameters &cam)
   {
-    this->cam = camera;
+    m_cam = cam;
 
     for (unsigned int i = 0; i < scales.size(); i += 1) {
       if (scales[i]) {
         for (std::list<vpMbtDistanceLine *>::const_iterator it = lines[i].begin(); it != lines[i].end(); ++it) {
-          (*it)->setCameraParameters(cam);
+          (*it)->setCameraParameters(m_cam);
         }
 
         for (std::list<vpMbtDistanceCylinder *>::const_iterator it = cylinders[i].begin(); it != cylinders[i].end();
              ++it) {
-          (*it)->setCameraParameters(cam);
+          (*it)->setCameraParameters(m_cam);
         }
 
         for (std::list<vpMbtDistanceCircle *>::const_iterator it = circles[i].begin(); it != circles[i].end(); ++it) {
-          (*it)->setCameraParameters(cam);
+          (*it)->setCameraParameters(m_cam);
         }
       }
     }
@@ -482,49 +477,52 @@ public:
 
      \sa getGoodMovingEdgesRatioThreshold()
    */
-  void setGoodMovingEdgesRatioThreshold(const double threshold) { percentageGdPt = threshold; }
+  void setGoodMovingEdgesRatioThreshold(double threshold) { percentageGdPt = threshold; }
 
   void setMovingEdge(const vpMe &me);
 
   virtual void setPose(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cdMo);
+  virtual void setPose(const vpImage<vpRGBa> &I_color, const vpHomogeneousMatrix &cdMo);
 
   void setScales(const std::vector<bool> &_scales);
 
   void setUseEdgeTracking(const std::string &name, const bool &useEdgeTracking);
 
-  void track(const vpImage<unsigned char> &I);
+  virtual void track(const vpImage<unsigned char> &I);
+  virtual void track(const vpImage<vpRGBa> &I);
   //@}
 
 protected:
   /** @name Protected Member Functions Inherited from vpMbEdgeTracker */
   //@{
-  bool samePoint(const vpPoint &P1, const vpPoint &P2) const;
-  void addCircle(const vpPoint &P1, const vpPoint &P2, const vpPoint &P3, const double r, int idFace = -1,
+  void addCircle(const vpPoint &P1, const vpPoint &P2, const vpPoint &P3, double r, int idFace = -1,
                  const std::string &name = "");
-  void addCylinder(const vpPoint &P1, const vpPoint &P2, const double r, int idFace = -1, const std::string &name = "");
+  void addCylinder(const vpPoint &P1, const vpPoint &P2, double r, int idFace = -1, const std::string &name = "");
   void addLine(vpPoint &p1, vpPoint &p2, int polygon = -1, std::string name = "");
   void addPolygon(vpMbtPolygon &p);
 
   void cleanPyramid(std::vector<const vpImage<unsigned char> *> &_pyramid);
   void computeProjectionError(const vpImage<unsigned char> &_I);
 
-  void computeVVS(const vpImage<unsigned char> &_I, const unsigned int lvl);
-  void computeVVSFirstPhase(const vpImage<unsigned char> &I, const unsigned int iter, double &count,
-                            const unsigned int lvl = 0);
-  void computeVVSFirstPhaseFactor(const vpImage<unsigned char> &I, const unsigned int lvl = 0);
-  void computeVVSFirstPhasePoseEstimation(const unsigned int iter, bool &isoJoIdentity_);
+  void computeVVS(const vpImage<unsigned char> &_I, unsigned int lvl);
+  void computeVVSFirstPhase(const vpImage<unsigned char> &I, unsigned int iter, double &count,
+                            unsigned int lvl = 0);
+  void computeVVSFirstPhaseFactor(const vpImage<unsigned char> &I, unsigned int lvl = 0);
+  void computeVVSFirstPhasePoseEstimation(unsigned int iter, bool &isoJoIdentity_);
   virtual void computeVVSInit();
   virtual void computeVVSInteractionMatrixAndResidu();
   virtual void computeVVSInteractionMatrixAndResidu(const vpImage<unsigned char> &I);
   virtual void computeVVSWeights();
   using vpMbTracker::computeVVSWeights;
 
-  void displayFeaturesOnImage(const vpImage<unsigned char> &I, const unsigned int lvl);
+  void displayFeaturesOnImage(const vpImage<unsigned char> &I);
+  void displayFeaturesOnImage(const vpImage<vpRGBa> &I);
   void downScale(const unsigned int _scale);
-  void init(const vpImage<unsigned char> &I);
-  virtual void initCircle(const vpPoint &p1, const vpPoint &p2, const vpPoint &p3, const double radius,
-                          const int idFace = 0, const std::string &name = "");
-  virtual void initCylinder(const vpPoint &p1, const vpPoint &p2, const double radius, const int idFace = 0,
+  virtual std::vector<std::vector<double> > getFeaturesForDisplayEdge();
+  virtual void init(const vpImage<unsigned char> &I);
+  virtual void initCircle(const vpPoint &p1, const vpPoint &p2, const vpPoint &p3, double radius,
+                          int idFace = 0, const std::string &name = "");
+  virtual void initCylinder(const vpPoint &p1, const vpPoint &p2, double radius, int idFace = 0,
                             const std::string &name = "");
   virtual void initFaceFromCorners(vpMbtPolygon &polygon);
   virtual void initFaceFromLines(vpMbtPolygon &polygon);
@@ -538,7 +536,7 @@ protected:
   void removeCylinder(const std::string &name);
   void removeLine(const std::string &name);
   void resetMovingEdge();
-  void testTracking();
+  virtual void testTracking();
   void trackMovingEdge(const vpImage<unsigned char> &I);
   void updateMovingEdge(const vpImage<unsigned char> &I);
   void updateMovingEdgeWeights();
