@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -88,12 +88,12 @@ private:
 
   unsigned int setVisiblePrivate(const vpHomogeneousMatrix &cMo, const double &angleAppears,
                                  const double &angleDisappears, bool &changed, bool useOgre = false,
-                                 bool not_used = false, const vpImage<unsigned char> &I = vpImage<unsigned char>(),
+                                 bool not_used = false, unsigned int width=0, unsigned int height=0,
                                  const vpCameraParameters &cam = vpCameraParameters());
 
 public:
   vpMbHiddenFaces();
-  ~vpMbHiddenFaces();
+  virtual ~vpMbHiddenFaces();
   vpMbHiddenFaces(const vpMbHiddenFaces &copy);
   vpMbHiddenFaces &operator=(vpMbHiddenFaces other);
   friend void swap<>(vpMbHiddenFaces &first, vpMbHiddenFaces &second);
@@ -101,7 +101,7 @@ public:
   void addPolygon(PolygonType *p);
 
   bool computeVisibility(const vpHomogeneousMatrix &cMo, const double &angleAppears, const double &angleDisappears,
-                         bool &changed, bool useOgre, bool not_used, const vpImage<unsigned char> &I,
+                         bool &changed, bool useOgre, bool not_used, unsigned int width, unsigned int height,
                          const vpCameraParameters &cam, const vpTranslationVector &cameraPos, unsigned int index);
 
   void computeClippedPolygons(const vpHomogeneousMatrix &cMo, const vpCameraParameters &cam);
@@ -166,7 +166,7 @@ public:
   double getGoodNbRayCastingAttemptsRatio() { return ratioVisibleRay; }
 #endif
 
-  bool isAppearing(const unsigned int i) { return Lpol[i]->isAppearing(); }
+  bool isAppearing(unsigned int i) { return Lpol[i]->isAppearing(); }
 
 #ifdef VISP_HAVE_OGRE
   /*!
@@ -184,16 +184,16 @@ public:
 
   \return Return true if the polygon is visible.
 */
-  bool isVisible(const unsigned int i) { return Lpol[i]->isVisible(); }
+  bool isVisible(unsigned int i) { return Lpol[i]->isVisible(); }
 
 #ifdef VISP_HAVE_OGRE
   bool isVisibleOgre(const vpTranslationVector &cameraPos, const unsigned int &index);
 #endif
 
   //! operator[] as modifier.
-  inline PolygonType *operator[](const unsigned int i) { return Lpol[i]; }
+  inline PolygonType *operator[](unsigned int i) { return Lpol[i]; }
   //! operator[] as reader.
-  inline const PolygonType *operator[](const unsigned int i) const { return Lpol[i]; }
+  inline const PolygonType *operator[](unsigned int i) const { return Lpol[i]; }
 
   void reset();
 
@@ -251,19 +251,19 @@ public:
     Ogre rendering options) when Ogre visibility is enabled. By default, this
     functionality is turned off.
   */
-  inline void setOgreShowConfigDialog(const bool showConfigDialog) { ogreShowConfigDialog = showConfigDialog; }
+  inline void setOgreShowConfigDialog(bool showConfigDialog) { ogreShowConfigDialog = showConfigDialog; }
 #endif
 
-  unsigned int setVisible(const vpImage<unsigned char> &I, const vpCameraParameters &cam,
+  unsigned int setVisible(unsigned int width, unsigned int height, const vpCameraParameters &cam,
                           const vpHomogeneousMatrix &cMo, const double &angle, bool &changed);
-  unsigned int setVisible(const vpImage<unsigned char> &I, const vpCameraParameters &cam,
+  unsigned int setVisible(unsigned int width, unsigned int height, const vpCameraParameters &cam,
                           const vpHomogeneousMatrix &cMo, const double &angleAppears, const double &angleDisappears,
                           bool &changed);
   unsigned int setVisible(const vpHomogeneousMatrix &cMo, const double &angleAppears, const double &angleDisappears,
                           bool &changed);
 
 #ifdef VISP_HAVE_OGRE
-  unsigned int setVisibleOgre(const vpImage<unsigned char> &I, const vpCameraParameters &cam,
+  unsigned int setVisibleOgre(unsigned int width, unsigned int height, const vpCameraParameters &cam,
                               const vpHomogeneousMatrix &cMo, const double &angleAppears, const double &angleDisappears,
                               bool &changed);
   unsigned int setVisibleOgre(const vpHomogeneousMatrix &cMo, const double &angleAppears, const double &angleDisappears,
@@ -522,7 +522,7 @@ void vpMbHiddenFaces<PolygonType>::computeScanLineQuery(const vpPoint &a, const
 template <class PolygonType>
 unsigned int vpMbHiddenFaces<PolygonType>::setVisiblePrivate(const vpHomogeneousMatrix &cMo, const double &angleAppears,
                                                              const double &angleDisappears, bool &changed, bool useOgre,
-                                                             bool not_used, const vpImage<unsigned char> &I,
+                                                             bool not_used, unsigned int width, unsigned int height,
                                                              const vpCameraParameters &cam)
 {
   nbVisiblePolygon = 0;
@@ -541,7 +541,7 @@ unsigned int vpMbHiddenFaces<PolygonType>::setVisiblePrivate(const vpHomogeneous
 
   for (unsigned int i = 0; i < Lpol.size(); i++) {
     // std::cout << "Calling poly: " << i << std::endl;
-    if (computeVisibility(cMo, angleAppears, angleDisappears, changed, useOgre, not_used, I, cam, cameraPos, i))
+    if (computeVisibility(cMo, angleAppears, angleDisappears, changed, useOgre, not_used, width, height, cam, cameraPos, i))
       nbVisiblePolygon++;
   }
   return nbVisiblePolygon;
@@ -554,19 +554,21 @@ unsigned int vpMbHiddenFaces<PolygonType>::setVisiblePrivate(const vpHomogeneous
   \param angleAppears : Angle used to test the appearance of a face
   \param angleDisappears : Angle used to test the disappearance of a face
   \param changed : True if a face appeared, disappeared or too many points
-  have been lost. False otherwise \param useOgre : True if a Ogre is used to
-  test the visibility, False otherwise. \param not_used : Unused parameter.
-  \param I : Image used to test if a face is entirely projected in the image.
+  have been lost. False otherwise
+  \param useOgre : True if a Ogre is used to test the visibility, False otherwise.
+  \param not_used : Unused parameter.
+  \param width, height Image size.
   \param cam : Camera parameters.
   \param cameraPos : Position of the camera. Used only when Ogre is used as
-  3rd party. \param index : Index of the face to consider.
+  3rd party.
+  \param index : Index of the face to consider.
 
   \return Return true if the face is visible.
 */
 template <class PolygonType>
 bool vpMbHiddenFaces<PolygonType>::computeVisibility(const vpHomogeneousMatrix &cMo, const double &angleAppears,
                                                      const double &angleDisappears, bool &changed, bool useOgre,
-                                                     bool not_used, const vpImage<unsigned char> &I,
+                                                     bool not_used, unsigned int width, unsigned int height,
                                                      const vpCameraParameters &cam,
                                                      const vpTranslationVector &cameraPos, unsigned int index)
 {
@@ -589,15 +591,15 @@ bool vpMbHiddenFaces<PolygonType>::computeVisibility(const vpHomogeneousMatrix &
       if (!testDisappear) {
         if (useOgre)
 #ifdef VISP_HAVE_OGRE
-          testDisappear = ((!Lpol[i]->isVisible(cMo, angleDisappears, true, cam, I)) || !isVisibleOgre(cameraPos, i));
+          testDisappear = ((!Lpol[i]->isVisible(cMo, angleDisappears, true, cam, width, height)) || !isVisibleOgre(cameraPos, i));
 #else
         {
           (void)cameraPos; // Avoid warning
-          testDisappear = (!Lpol[i]->isVisible(cMo, angleDisappears, false, cam, I));
+          testDisappear = (!Lpol[i]->isVisible(cMo, angleDisappears, false, cam, width, height));
         }
 #endif
         else
-          testDisappear = (!Lpol[i]->isVisible(cMo, angleDisappears, false, cam, I));
+          testDisappear = (!Lpol[i]->isVisible(cMo, angleDisappears, false, cam, width, height));
       }
 
       // test if the face is still visible
@@ -619,12 +621,12 @@ bool vpMbHiddenFaces<PolygonType>::computeVisibility(const vpHomogeneousMatrix &
       if (testAppear) {
         if (useOgre)
 #ifdef VISP_HAVE_OGRE
-          testAppear = ((Lpol[i]->isVisible(cMo, angleAppears, true, cam, I)) && isVisibleOgre(cameraPos, i));
+          testAppear = ((Lpol[i]->isVisible(cMo, angleAppears, true, cam, width, height)) && isVisibleOgre(cameraPos, i));
 #else
-          testAppear = (Lpol[i]->isVisible(cMo, angleAppears, false, cam, I));
+          testAppear = (Lpol[i]->isVisible(cMo, angleAppears, false, cam, width, height));
 #endif
         else
-          testAppear = (Lpol[i]->isVisible(cMo, angleAppears, false, cam, I));
+          testAppear = (Lpol[i]->isVisible(cMo, angleAppears, false, cam, width, height));
       }
 
       if (testAppear) {
@@ -646,40 +648,44 @@ bool vpMbHiddenFaces<PolygonType>::computeVisibility(const vpHomogeneousMatrix &
 /*!
   Compute the number of visible polygons.
 
-  \param I : Image used to check if the region of interest is inside the
-  image. \param cam : Camera parameters. \param cMo : The pose of the camera.
-  \param angle : Angle used to test the appearance and disappearance of a
-  face. \param changed : True if a face appeared, disappeared or too many
-  points have been lost. False otherwise
+  \param width, height : Image size used to check if the region of interest is inside the
+  image.
+  \param cam : Camera parameters.
+  \param cMo : The pose of the camera.
+  \param angle : Angle used to test the appearance and disappearance of a face.
+  \param changed : True if a face appeared, disappeared or too many
+  points have been lost. False otherwise.
 
   \return Return the number of visible polygons
 */
 template <class PolygonType>
-unsigned int vpMbHiddenFaces<PolygonType>::setVisible(const vpImage<unsigned char> &I, const vpCameraParameters &cam,
+unsigned int vpMbHiddenFaces<PolygonType>::setVisible(unsigned int width, unsigned int height, const vpCameraParameters &cam,
                                                       const vpHomogeneousMatrix &cMo, const double &angle,
                                                       bool &changed)
 {
-  return setVisible(I, cam, cMo, angle, angle, changed);
+  return setVisible(width, height, cam, cMo, angle, angle, changed);
 }
 
 /*!
   Compute the number of visible polygons.
 
-  \param I : Image used to check if the region of interest is inside the
-  image. \param cam : Camera parameters. \param cMo : The pose of the camera
+  \param width, height : Image size used to check if the region of interest is inside the
+  image.
+  \param cam : Camera parameters.
+  \param cMo : The pose of the camera.
   \param changed : True if a face appeared, disappeared or too many points
-  have been lost. False otherwise \param angleAppears : Angle used to test the
-  appearance of a face \param angleDisappears : Angle used to test the
-  disappearance of a face
+  have been lost. False otherwise.
+  \param angleAppears : Angle used to test the appearance of a face.
+  \param angleDisappears : Angle used to test the disappearance of a face.
 
   \return Return the number of visible polygons
 */
 template <class PolygonType>
-unsigned int vpMbHiddenFaces<PolygonType>::setVisible(const vpImage<unsigned char> &I, const vpCameraParameters &cam,
+unsigned int vpMbHiddenFaces<PolygonType>::setVisible(unsigned int width, unsigned int height, const vpCameraParameters &cam,
                                                       const vpHomogeneousMatrix &cMo, const double &angleAppears,
                                                       const double &angleDisappears, bool &changed)
 {
-  return setVisiblePrivate(cMo, angleAppears, angleDisappears, changed, false, true, I, cam);
+  return setVisiblePrivate(cMo, angleAppears, angleDisappears, changed, false, true, width, height, cam);
 }
 
 /*!
@@ -754,22 +760,24 @@ template <class PolygonType> void vpMbHiddenFaces<PolygonType>::displayOgre(cons
 /*!
   Compute the number of visible polygons through Ogre3D.
 
-  \param I : Image used to check if the region of interest is inside the
-  image. \param cam : Camera parameters. \param cMo : The pose of the camera
+  \param width, height : Image size used to check if the region of interest is inside the
+  image.
+  \param cam : Camera parameters.
+  \param cMo : The pose of the camera.
   \param changed : True if a face appeared, disappeared or too many points
-  have been lost. False otherwise \param angleAppears : Angle used to test the
-  appearance of a face \param angleDisappears : Angle used to test the
-  disappearance of a face
+  have been lost. False otherwise.
+  \param angleAppears : Angle used to test the appearance of a face.
+  \param angleDisappears : Angle used to test the disappearance of a face.
 
   \return Return the number of visible polygons
 */
 template <class PolygonType>
-unsigned int vpMbHiddenFaces<PolygonType>::setVisibleOgre(const vpImage<unsigned char> &I,
+unsigned int vpMbHiddenFaces<PolygonType>::setVisibleOgre(unsigned int width, unsigned int height,
                                                           const vpCameraParameters &cam, const vpHomogeneousMatrix &cMo,
                                                           const double &angleAppears, const double &angleDisappears,
                                                           bool &changed)
 {
-  return setVisiblePrivate(cMo, angleAppears, angleDisappears, changed, true, true, I, cam);
+  return setVisiblePrivate(cMo, angleAppears, angleDisappears, changed, true, true, width, height, cam);
 }
 
 /*!
