@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2016 by INRIA. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -11,25 +11,23 @@
  * distribution for additional information about the GNU GPL.
  *
  * For using ViSP with software that can not be combined with the GNU
- * GPL, please contact INRIA about acquiring a ViSP Professional
+ * GPL, please contact Inria about acquiring a ViSP Professional
  * Edition License.
  *
- * See https://visp.inria.fr/download/ for more information.
+ * See http://visp.inria.fr for more information.
  *
  * This software was developed at:
- * INRIA Rennes - Bretagne Atlantique
+ * Inria Rennes - Bretagne Atlantique
  * Campus Universitaire de Beaulieu
  * 35042 Rennes Cedex
  * France
- * http://www.irisa.fr/lagadic
  *
  * If you have questions regarding the use of this file, please contact
- * INRIA at visp@inria.fr
+ * Inria at visp@inria.fr
  *
  * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
  * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  *
- *
  * Description:
  * Model-based klt tracker with multiple cameras.
  *
@@ -43,14 +41,16 @@
  \brief Model-based klt tracker with multiple cameras.
 */
 
-#ifndef __vpMbKltMultiTracker_h__
-#define __vpMbKltMultiTracker_h__
+#ifndef _vpMbKltMultiTracker_h_
+#define _vpMbKltMultiTracker_h_
 
 #include <iostream>
 #include <vector>
 
 #include <visp3/core/vpConfig.h>
 
+#if defined(VISP_BUILD_DEPRECATED_FUNCTIONS)
+
 #if defined(VISP_HAVE_MODULE_KLT) && defined(VISP_HAVE_OPENCV) && (VISP_HAVE_OPENCV_VERSION >= 0x020100)
 
 #include <visp3/mbt/vpMbKltTracker.h>
@@ -58,11 +58,13 @@
 /*!
   \class vpMbKltMultiTracker
   \ingroup group_mbt_trackers
+  \warning This class is deprecated for user usage. You should rather use the high level
+  vpMbGenericTracker class.
   \warning This class is only available if OpenCV is installed, and used.
 
   \brief Model based stereo (or more) tracker using only KLT.
 
-  The \ref tutorial-tracking-mb-stereo is a good starting point to use this
+  The \ref tutorial-tracking-mb-stereo-deprecated is a good starting point to use this
   class.
 
   The tracker requires the knowledge of the 3D model that could be provided in
@@ -70,7 +72,7 @@
   may also use an xml file used to tune the behavior of the tracker and an
   init file used to compute the pose at the very first image.
 */
-class VISP_EXPORT vpMbKltMultiTracker : public vpMbKltTracker
+class VISP_EXPORT vp_deprecated vpMbKltMultiTracker : public vpMbKltTracker
 {
 protected:
   //! Map of camera transformation matrix between the current camera frame to
@@ -93,40 +95,49 @@ protected:
 
 public:
   vpMbKltMultiTracker();
-  vpMbKltMultiTracker(const unsigned int nbCameras);
+  vpMbKltMultiTracker(unsigned int nbCameras);
   vpMbKltMultiTracker(const std::vector<std::string> &cameraNames);
 
   virtual ~vpMbKltMultiTracker();
 
   /** @name Inherited functionalities from vpMbKltMultiTracker */
   //@{
-  virtual void addCircle(const vpPoint &P1, const vpPoint &P2, const vpPoint &P3, const double r,
+  virtual void addCircle(const vpPoint &P1, const vpPoint &P2, const vpPoint &P3, double r,
                          const std::string &name = "");
 
-  virtual void display(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo_, const vpCameraParameters &cam_,
-                       const vpColor &col, const unsigned int thickness = 1, const bool displayFullModel = false);
+  virtual void display(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo, const vpCameraParameters &cam,
+                       const vpColor &col, unsigned int thickness = 1, bool displayFullModel = false);
 
-  virtual void display(const vpImage<vpRGBa> &I, const vpHomogeneousMatrix &cMo_, const vpCameraParameters &cam_,
-                       const vpColor &col, const unsigned int thickness = 1, const bool displayFullModel = false);
+  virtual void display(const vpImage<vpRGBa> &I, const vpHomogeneousMatrix &cMo, const vpCameraParameters &cam,
+                       const vpColor &col, unsigned int thickness = 1, bool displayFullModel = false);
 
   virtual void display(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2,
                        const vpHomogeneousMatrix &c1Mo, const vpHomogeneousMatrix &c2Mo, const vpCameraParameters &cam1,
-                       const vpCameraParameters &cam2, const vpColor &color, const unsigned int thickness = 1,
-                       const bool displayFullModel = false);
+                       const vpCameraParameters &cam2, const vpColor &color, unsigned int thickness = 1,
+                       bool displayFullModel = false);
 
   virtual void display(const vpImage<vpRGBa> &I1, const vpImage<vpRGBa> &I2, const vpHomogeneousMatrix &c1Mo,
                        const vpHomogeneousMatrix &c2Mo, const vpCameraParameters &cam1, const vpCameraParameters &cam2,
-                       const vpColor &color, const unsigned int thickness = 1, const bool displayFullModel = false);
+                       const vpColor &color, unsigned int thickness = 1, bool displayFullModel = false);
 
   virtual void display(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
                        const std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses,
                        const std::map<std::string, vpCameraParameters> &mapOfCameraParameters, const vpColor &col,
-                       const unsigned int thickness = 1, const bool displayFullModel = false);
+                       unsigned int thickness = 1, bool displayFullModel = false);
 
   virtual void display(const std::map<std::string, const vpImage<vpRGBa> *> &mapOfImages,
                        const std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses,
                        const std::map<std::string, vpCameraParameters> &mapOfCameraParameters, const vpColor &col,
-                       const unsigned int thickness = 1, const bool displayFullModel = false);
+                       unsigned int thickness = 1, bool displayFullModel = false);
+
+  virtual std::vector<std::vector<double> > getModelForDisplay(unsigned int, unsigned int,
+                                                               const vpHomogeneousMatrix &,
+                                                               const vpCameraParameters &,
+                                                               bool =false)
+  {
+    std::cerr << "Not implemented. Deprecated class." << std::endl;
+    return std::vector<std::vector<double> >();
+  }
 
   virtual std::vector<std::string> getCameraNames() const;
 
@@ -177,7 +188,7 @@ public:
 
   using vpMbTracker::getPose;
   virtual void getPose(vpHomogeneousMatrix &c1Mo, vpHomogeneousMatrix &c2Mo) const;
-  virtual void getPose(const std::string &cameraName, vpHomogeneousMatrix &cMo_) const;
+  virtual void getPose(const std::string &cameraName, vpHomogeneousMatrix &cMo) const;
   virtual void getPose(std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses) const;
 
   virtual inline vpColVector getError() const { return m_error_kltMulti; }
@@ -187,53 +198,58 @@ public:
   virtual void init(const vpImage<unsigned char> &I);
 
 #ifdef VISP_HAVE_MODULE_GUI
+  using vpMbTracker::initClick;
   virtual void initClick(const vpImage<unsigned char> &I, const std::vector<vpPoint> &points3D_list,
                          const std::string &displayFile = "");
 
-  virtual void initClick(const vpImage<unsigned char> &I, const std::string &initFile, const bool displayHelp = false);
+  virtual void initClick(const vpImage<unsigned char> &I, const std::string &initFile, bool displayHelp = false,
+                         const vpHomogeneousMatrix &T=vpHomogeneousMatrix());
 
   virtual void initClick(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2,
-                         const std::string &initFile1, const std::string &initFile2, const bool displayHelp = false,
-                         const bool firstCameraIsReference = true);
+                         const std::string &initFile1, const std::string &initFile2, bool displayHelp = false,
+                         bool firstCameraIsReference = true);
 
   virtual void initClick(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
-                         const std::string &initFile, const bool displayHelp = false);
+                         const std::string &initFile, bool displayHelp = false);
 
   virtual void initClick(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
-                         const std::map<std::string, std::string> &mapOfInitFiles, const bool displayHelp = false);
+                         const std::map<std::string, std::string> &mapOfInitFiles, bool displayHelp = false);
 #endif
 
+  using vpMbTracker::initFromPose;
   virtual void initFromPose(const vpImage<unsigned char> &I, const std::string &initFile);
   virtual void initFromPose(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo);
   virtual void initFromPose(const vpImage<unsigned char> &I, const vpPoseVector &cPo);
 
   virtual void initFromPose(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2,
                             const vpHomogeneousMatrix &c1Mo, const vpHomogeneousMatrix &c2Mo,
-                            const bool firstCameraIsReference = true);
+                            bool firstCameraIsReference = true);
   virtual void initFromPose(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
-                            const vpHomogeneousMatrix &cMo_);
+                            const vpHomogeneousMatrix &cMo);
   virtual void initFromPose(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
                             const std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses);
 
   virtual void loadConfigFile(const std::string &configFile);
 
   virtual void loadConfigFile(const std::string &configFile1, const std::string &configFile2,
-                              const bool firstCameraIsReference = true);
+                              bool firstCameraIsReference = true);
 
   virtual void loadConfigFile(const std::map<std::string, std::string> &mapOfConfigFiles);
 
-  virtual void loadModel(const std::string &modelFile, const bool verbose = false);
+  virtual void loadModel(const std::string &modelFile, bool verbose = false,
+                         const vpHomogeneousMatrix &T=vpHomogeneousMatrix());
 
   virtual void reInitModel(const vpImage<unsigned char> &I, const std::string &cad_name,
-                           const vpHomogeneousMatrix &cMo_, const bool verbose = false);
+                           const vpHomogeneousMatrix &cMo, bool verbose = false,
+                           const vpHomogeneousMatrix &T=vpHomogeneousMatrix());
   virtual void reInitModel(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2,
                            const std::string &cad_name, const vpHomogeneousMatrix &c1Mo,
-                           const vpHomogeneousMatrix &c2Mo, const bool verbose = false,
-                           const bool firstCameraIsReference = true);
+                           const vpHomogeneousMatrix &c2Mo, bool verbose = false,
+                           bool firstCameraIsReference = true);
   virtual void reInitModel(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
                            const std::string &cad_name,
                            const std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses,
-                           const bool verbose = false);
+                           bool verbose = false);
 
   virtual void resetTracker();
 
@@ -243,7 +259,7 @@ public:
   virtual void setCameraParameters(const vpCameraParameters &camera);
 
   virtual void setCameraParameters(const vpCameraParameters &camera1, const vpCameraParameters &camera2,
-                                   const bool firstCameraIsReference = true);
+                                   bool firstCameraIsReference = true);
 
   virtual void setCameraParameters(const std::string &cameraName, const vpCameraParameters &camera);
 
@@ -260,7 +276,7 @@ public:
 
   virtual void setCovarianceComputation(const bool &flag);
 
-  virtual void setDisplayFeatures(const bool displayF);
+  virtual void setDisplayFeatures(bool displayF);
 
   virtual void setFarClippingDistance(const double &dist);
   virtual void setFarClippingDistance(const std::string &cameraName, const double &dist);
@@ -276,34 +292,36 @@ public:
   virtual void setKltOpencv(const vpKltOpencv &t);
   virtual void setKltOpencv(const std::map<std::string, vpKltOpencv> &mapOfOpenCVTrackers);
 
-  virtual void setKltThresholdAcceptation(const double th);
+  virtual void setKltThresholdAcceptation(double th);
 
-  virtual void setLod(const bool useLod, const std::string &name = "");
-  virtual void setLod(const bool useLod, const std::string &cameraName, const std::string &name);
+  virtual void setLod(bool useLod, const std::string &name = "");
+  virtual void setLod(bool useLod, const std::string &cameraName, const std::string &name);
 
-  virtual void setMinLineLengthThresh(const double minLineLengthThresh, const std::string &name = "");
+  virtual void setMinLineLengthThresh(double minLineLengthThresh, const std::string &name = "");
 
-  virtual void setMinPolygonAreaThresh(const double minPolygonAreaThresh, const std::string &name = "");
-  virtual void setMinPolygonAreaThresh(const double minPolygonAreaThresh, const std::string &cameraName,
+  virtual void setMinPolygonAreaThresh(double minPolygonAreaThresh, const std::string &name = "");
+  virtual void setMinPolygonAreaThresh(double minPolygonAreaThresh, const std::string &cameraName,
                                        const std::string &name);
 
   virtual void setNearClippingDistance(const double &dist);
   virtual void setNearClippingDistance(const std::string &cameraName, const double &dist);
 
-  virtual void setOgreShowConfigDialog(const bool showConfigDialog);
+  virtual void setOgreShowConfigDialog(bool showConfigDialog);
 
   virtual void setOgreVisibilityTest(const bool &v);
 
   virtual void setOptimizationMethod(const vpMbtOptimizationMethod &opt);
 
+  using vpMbKltTracker::setPose;
   virtual void setPose(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo);
+  virtual void setPose(const vpImage<vpRGBa> &I_color, const vpHomogeneousMatrix &cMo);
 
   virtual void setPose(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2,
                        const vpHomogeneousMatrix &c1Mo, const vpHomogeneousMatrix &c2Mo,
-                       const bool firstCameraIsReference = true);
+                       bool firstCameraIsReference = true);
 
   virtual void setPose(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
-                       const vpHomogeneousMatrix &cMo_);
+                       const vpHomogeneousMatrix &cMo);
 
   virtual void setPose(const std::map<std::string, const vpImage<unsigned char> *> &mapOfImages,
                        const std::map<std::string, vpHomogeneousMatrix> &mapOfCameraPoses);
@@ -315,6 +333,7 @@ public:
   virtual void setUseKltTracking(const std::string &name, const bool &useKltTracking);
 
   virtual void track(const vpImage<unsigned char> &I);
+  virtual void track(const vpImage<vpRGBa> &I_color);
   virtual void track(const vpImage<unsigned char> &I1, const vpImage<unsigned char> &I2);
   virtual void track(std::map<std::string, const vpImage<unsigned char> *> &mapOfImages);
   //@}
@@ -325,7 +344,7 @@ public:
   //@{
   /* vp_deprecated */ virtual std::map<std::string, int> getNbKltPoints() const;
   /* vp_deprecated */ virtual void setMaskBorder(const unsigned int &e);
-  /* vp_deprecated */ virtual void setThresholdAcceptation(const double th);
+  /* vp_deprecated */ virtual void setThresholdAcceptation(double th);
   //@}
 
 protected:
@@ -348,4 +367,5 @@ protected:
 };
 
 #endif // VISP_HAVE_OPENCV
-#endif //__vpMbKltMultiTracker_h__
+#endif //#if defined(VISP_BUILD_DEPRECATED_FUNCTIONS)
+#endif //_vpMbKltMultiTracker_h_
