@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -116,7 +116,7 @@ void vpMbtDistanceCylinder::project(const vpHomogeneousMatrix &cMo)
   \param _p2 : The second extremity on the axis.
   \param r : Radius of the cylinder.
 */
-void vpMbtDistanceCylinder::buildFrom(const vpPoint &_p1, const vpPoint &_p2, const double r)
+void vpMbtDistanceCylinder::buildFrom(const vpPoint &_p1, const vpPoint &_p2, double r)
 {
   c = new vpCylinder;
   p1 = new vpPoint;
@@ -177,9 +177,12 @@ void vpMbtDistanceCylinder::setMovingEdge(vpMe *_me)
 
   \param I : The image.
   \param cMo : The pose of the camera used to initialize the moving edges.
+  \param doNotTrack : If true, ME are not tracked.
+  \param mask: Mask image or NULL if not wanted. Mask values that are set to true are considered in the tracking. To disable a pixel, set false.
   \return false if an error occur, true otherwise.
 */
-bool vpMbtDistanceCylinder::initMovingEdge(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo)
+bool vpMbtDistanceCylinder::initMovingEdge(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo, bool doNotTrack,
+                                           const vpImage<bool> *mask)
 {
   if (isvisible) {
     // Perspective projection
@@ -210,8 +213,10 @@ bool vpMbtDistanceCylinder::initMovingEdge(const vpImage<unsigned char> &I, cons
 
     // Create the moving edges containers
     meline1 = new vpMbtMeLine;
+    meline1->setMask(*mask);
     meline1->setMe(me);
     meline2 = new vpMbtMeLine;
+    meline1->setMask(*mask);
     meline2->setMe(me);
 
     //    meline->setDisplay(vpMeSite::RANGE_RESULT);
@@ -294,13 +299,13 @@ bool vpMbtDistanceCylinder::initMovingEdge(const vpImage<unsigned char> &I, cons
       theta2 = M_PI / 2.0 - theta2;
 
     try {
-      meline1->initTracking(I, ip11, ip12, rho1, theta1);
+      meline1->initTracking(I, ip11, ip12, rho1, theta1, doNotTrack);
     } catch (...) {
       // vpTRACE("the line can't be initialized");
       return false;
     }
     try {
-      meline2->initTracking(I, ip21, ip22, rho2, theta2);
+      meline2->initTracking(I, ip21, ip22, rho2, theta2, doNotTrack);
     } catch (...) {
       // vpTRACE("the line can't be initialized");
       return false;
@@ -479,8 +484,10 @@ void vpMbtDistanceCylinder::updateMovingEdge(const vpImage<unsigned char> &I, co
 
   \param I : the image.
   \param cMo : The pose of the camera.
+  \param mask: Mask image or NULL if not wanted. Mask values that are set to true are considered in the tracking. To disable a pixel, set false.
 */
-void vpMbtDistanceCylinder::reinitMovingEdge(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo)
+void vpMbtDistanceCylinder::reinitMovingEdge(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo,
+                                             const vpImage<bool> *mask)
 {
   if (meline1 != NULL)
     delete meline1;
@@ -490,7 +497,7 @@ void vpMbtDistanceCylinder::reinitMovingEdge(const vpImage<unsigned char> &I, co
   meline1 = NULL;
   meline2 = NULL;
 
-  if (initMovingEdge(I, cMo) == false)
+  if (!initMovingEdge(I, cMo, false, mask))
     Reinit = true;
 
   Reinit = false;
@@ -507,57 +514,17 @@ void vpMbtDistanceCylinder::reinitMovingEdge(const vpImage<unsigned char> &I, co
   \param displayFullModel : When true, display the circle even if non visible.
 */
 void vpMbtDistanceCylinder::display(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo,
-                                    const vpCameraParameters &camera, const vpColor &col, const unsigned int thickness,
-                                    const bool displayFullModel)
+                                    const vpCameraParameters &camera, const vpColor &col, unsigned int thickness,
+                                    bool displayFullModel)
 {
-  if ((isvisible && isTrackedCylinder) || displayFullModel) {
-    // Perspective projection
-    p1->changeFrame(cMo);
-    p2->changeFrame(cMo);
-    cercle1->changeFrame(cMo);
-    cercle2->changeFrame(cMo);
-    c->changeFrame(cMo);
+  std::vector<std::vector<double> > models = getModelForDisplay(I.getWidth(), I.getHeight(),
+                                                                cMo, camera, displayFullModel);
 
-    p1->projection();
-    p2->projection();
-    try {
-      cercle1->projection();
-    } catch (...) {
-      std::cout << "Problem projection circle 1";
-    }
-    try {
-      cercle2->projection();
-    } catch (...) {
-      std::cout << "Problem projection circle 2";
-    }
-    c->projection();
+  for (size_t i = 0; i < models.size(); i++) {
+    vpImagePoint ip1(models[i][1], models[i][2]);
+    vpImagePoint ip2(models[i][3], models[i][4]);
 
-    double rho1, theta1;
-    double rho2, theta2;
-
-    // Meters to pixels conversion
-    vpMeterPixelConversion::convertLine(camera, c->getRho1(), c->getTheta1(), rho1, theta1);
-    vpMeterPixelConversion::convertLine(camera, c->getRho2(), c->getTheta2(), rho2, theta2);
-
-    // Determine intersections between circles and limbos
-    double i11, i12, i21, i22, j11, j12, j21, j22;
-
-    vpCircle::computeIntersectionPoint(*cercle1, cam, rho1, theta1, i11, j11);
-    vpCircle::computeIntersectionPoint(*cercle2, cam, rho1, theta1, i12, j12);
-
-    vpCircle::computeIntersectionPoint(*cercle1, cam, rho2, theta2, i21, j21);
-    vpCircle::computeIntersectionPoint(*cercle2, cam, rho2, theta2, i22, j22);
-
-    // Create the image points
-    vpImagePoint ip11, ip12, ip21, ip22;
-    ip11.set_ij(i11, j11);
-    ip12.set_ij(i12, j12);
-    ip21.set_ij(i21, j21);
-    ip22.set_ij(i22, j22);
-
-    // Display
-    vpDisplay::displayLine(I, ip11, ip12, col, thickness);
-    vpDisplay::displayLine(I, ip21, ip22, col, thickness);
+    vpDisplay::displayLine(I, ip1, ip2, col, thickness);
   }
 }
 
@@ -572,9 +539,86 @@ void vpMbtDistanceCylinder::display(const vpImage<unsigned char> &I, const vpHom
   \param displayFullModel : When true, display the circle even if non visible.
 */
 void vpMbtDistanceCylinder::display(const vpImage<vpRGBa> &I, const vpHomogeneousMatrix &cMo,
-                                    const vpCameraParameters &camera, const vpColor &col, const unsigned int thickness,
-                                    const bool displayFullModel)
+                                    const vpCameraParameters &camera, const vpColor &col, unsigned int thickness,
+                                    bool displayFullModel)
 {
+  std::vector<std::vector<double> > models = getModelForDisplay(I.getWidth(), I.getHeight(),
+                                                                cMo, camera, displayFullModel);
+
+  for (size_t i = 0; i < models.size(); i++) {
+    vpImagePoint ip1(models[i][1], models[i][2]);
+    vpImagePoint ip2(models[i][3], models[i][4]);
+
+    vpDisplay::displayLine(I, ip1, ip2, col, thickness);
+  }
+}
+
+/*!
+  Return a list of features parameters for display.
+  - Parameters are: `<feature id (here 0 for ME)>`, `<pt.i()>`, `<pt.j()>`, `<state>`
+*/
+std::vector<std::vector<double> > vpMbtDistanceCylinder::getFeaturesForDisplay()
+{
+  std::vector<std::vector<double> > features;
+
+  if (meline1 != NULL) {
+    for (std::list<vpMeSite>::const_iterator it = meline1->getMeList().begin(); it != meline1->getMeList().end(); ++it) {
+      vpMeSite p_me = *it;
+#if (VISP_CXX_STANDARD >= VISP_CXX_STANDARD_11)
+      std::vector<double> params = {0, //ME
+                                    p_me.get_ifloat(),
+                                    p_me.get_jfloat(),
+                                    static_cast<double>(p_me.getState())};
+#else
+      std::vector<double> params;
+      params.push_back(0); //ME
+      params.push_back(p_me.get_ifloat());
+      params.push_back(p_me.get_jfloat());
+      params.push_back(static_cast<double>(p_me.getState()));
+#endif
+      features.push_back(params);
+    }
+  }
+
+  if (meline2 != NULL) {
+    for (std::list<vpMeSite>::const_iterator it = meline2->getMeList().begin(); it != meline2->getMeList().end(); ++it) {
+      vpMeSite p_me = *it;
+#if (VISP_CXX_STANDARD >= VISP_CXX_STANDARD_11)
+      std::vector<double> params = {0, //ME
+                                    p_me.get_ifloat(),
+                                    p_me.get_jfloat(),
+                                    static_cast<double>(p_me.getState())};
+#else
+      std::vector<double> params;
+      params.push_back(0); //ME
+      params.push_back(p_me.get_ifloat());
+      params.push_back(p_me.get_jfloat());
+      params.push_back(static_cast<double>(p_me.getState()));
+#endif
+      features.push_back(params);
+    }
+  }
+
+  return features;
+}
+
+/*!
+  Return a list of line parameters to display the primitive at a given pose and camera parameters.
+  - Parameters are: `<primitive id (here 0 for line)>`, `<pt_start.i()>`, `<pt_start.j()>`,
+  `<pt_end.i()>`, `<pt_end.j()>`
+
+  \param width, height Image size (unused parameters).
+  \param cMo : Pose used to project the 3D model into the image.
+  \param camera : The camera parameters.
+  \param displayFullModel : If true, the line is displayed even if it is not
+*/
+std::vector<std::vector<double> > vpMbtDistanceCylinder::getModelForDisplay(unsigned int, unsigned int,
+                                                                            const vpHomogeneousMatrix &cMo,
+                                                                            const vpCameraParameters &camera,
+                                                                            bool displayFullModel)
+{
+  std::vector<std::vector<double> > models;
+
   if ((isvisible && isTrackedCylinder) || displayFullModel) {
     // Perspective projection
     p1->changeFrame(cMo);
@@ -620,10 +664,38 @@ void vpMbtDistanceCylinder::display(const vpImage<vpRGBa> &I, const vpHomogeneou
     ip21.set_ij(i21, j21);
     ip22.set_ij(i22, j22);
 
-    // Display
-    vpDisplay::displayLine(I, ip11, ip12, col, thickness);
-    vpDisplay::displayLine(I, ip21, ip22, col, thickness);
+#if (VISP_CXX_STANDARD >= VISP_CXX_STANDARD_11)
+    std::vector<double> params1 = {0,
+                                   ip11.get_i(),
+                                   ip11.get_j(),
+                                   ip12.get_i(),
+                                   ip12.get_j()};
+
+    std::vector<double> params2 = {0,
+                                   ip21.get_i(),
+                                   ip21.get_j(),
+                                   ip22.get_i(),
+                                   ip22.get_j()};
+#else
+    std::vector<double> params1, params2;
+    params1.push_back(0);
+    params1.push_back(ip11.get_i());
+    params1.push_back(ip11.get_j());
+    params1.push_back(ip12.get_i());
+    params1.push_back(ip12.get_j());
+
+    params2.push_back(0); 
+    params2.push_back(ip11.get_i());
+    params2.push_back(ip11.get_j());
+    params2.push_back(ip12.get_i());
+    params2.push_back(ip12.get_j());
+#endif
+
+    models.push_back(params1);
+    models.push_back(params2);
   }
+
+  return models;
 }
 
 /*!
@@ -650,6 +722,16 @@ void vpMbtDistanceCylinder::displayMovingEdges(const vpImage<unsigned char> &I)
   }
 }
 
+void vpMbtDistanceCylinder::displayMovingEdges(const vpImage<vpRGBa> &I)
+{
+  if (meline1 != NULL) {
+    meline1->display(I);
+  }
+  if (meline2 != NULL) {
+    meline2->display(I);
+  }
+}
+
 /*!
   Initialize the size of the interaction matrix and the error vector.
 */
