@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -300,9 +300,12 @@ void vpMbtDistanceLine::setMovingEdge(vpMe *_me)
 
   \param I : The image.
   \param cMo : The pose of the camera used to initialize the moving edges.
+  \param doNotTrack : If true, ME are not tracked.
+  \param mask: Mask image or NULL if not wanted. Mask values that are set to true are considered in the tracking. To disable a pixel, set false.
   \return false if an error occur, true otherwise.
 */
-bool vpMbtDistanceLine::initMovingEdge(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo)
+bool vpMbtDistanceLine::initMovingEdge(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo, bool doNotTrack,
+                                       const vpImage<bool> *mask)
 {
   for (unsigned int i = 0; i < meline.size(); i++) {
     if (meline[i] != NULL)
@@ -333,26 +336,17 @@ bool vpMbtDistanceLine::initMovingEdge(const vpImage<unsigned char> &I, const vp
       }
 
       if (linesLst.size() == 0) {
-        //        isvisible = false;
         return false;
       }
 
-      // To have the exact same pose values as the old version (angle or ogre
-      // visibility test only), points should be reorganised when using
-      // scanline algorithm.
-      //      if(sqrt(vpMath::sqr(poly.polyClipped[0].first.get_X() -
-      //      linesLst[0].first.get_X())) >
-      //      sqrt(vpMath::sqr(poly.polyClipped[0].first.get_X() -
-      //      linesLst[0].second.get_X())))
-      //      {
-      //          std::vector<std::pair<vpPoint, vpPoint> > linesLstTmp;
-      //          for(int i = linesLst.size()-1 ; i >= 0 ; i--)
-      //            linesLstTmp.push_back(std::make_pair(linesLst[i].second,linesLst[i].first));
-      //          linesLst = linesLstTmp;
-      //      }
-
       line->changeFrame(cMo);
-      line->projection();
+      try {
+        line->projection();
+      }
+      catch(...) {
+        isvisible = false;
+        return false;
+      }
       double rho, theta;
       // rho theta uv
       vpMeterPixelConversion::convertLine(cam, line->getRho(), line->getTheta(), rho, theta);
@@ -379,9 +373,9 @@ bool vpMbtDistanceLine::initMovingEdge(const vpImage<unsigned char> &I, const vp
         vpMeterPixelConversion::convertPoint(cam, linesLst[i].second.get_x(), linesLst[i].second.get_y(), ip2);
 
         vpMbtMeLine *melinePt = new vpMbtMeLine;
+        melinePt->setMask(*mask);
         melinePt->setMe(me);
 
-        //    meline[i]->setDisplay(vpMeSite::RANGE_RESULT);
         melinePt->setInitRange(0);
 
         int marge = /*10*/ 5; // ou 5 normalement
@@ -401,13 +395,11 @@ bool vpMbtDistanceLine::initMovingEdge(const vpImage<unsigned char> &I, const vp
         }
 
         try {
-          melinePt->initTracking(I, ip1, ip2, rho, theta);
+          melinePt->initTracking(I, ip1, ip2, rho, theta, doNotTrack);
           meline.push_back(melinePt);
-          //        nbFeature.push_back((unsigned int)
-          //        melinePt->getMeList().size()); nbFeatureTotal +=
-          //        nbFeature.back();
+          nbFeature.push_back((unsigned int) melinePt->getMeList().size());
+          nbFeatureTotal += nbFeature.back();
         } catch (...) {
-          // vpTRACE("the line can't be initialized");
           delete melinePt;
           isvisible = false;
           return false;
@@ -415,11 +407,9 @@ bool vpMbtDistanceLine::initMovingEdge(const vpImage<unsigned char> &I, const vp
       }
     } else {
       isvisible = false;
-      //      return false;
     }
   }
 
-  //	trackMovingEdge(I,cMo);
   return true;
 }
 
@@ -427,41 +417,20 @@ bool vpMbtDistanceLine::initMovingEdge(const vpImage<unsigned char> &I, const vp
   Track the moving edges in the image.
 
   \param I : the image.
-  \param cMo : The pose of the camera.
 */
-void vpMbtDistanceLine::trackMovingEdge(const vpImage<unsigned char> &I, const vpHomogeneousMatrix & /*cMo*/)
+void vpMbtDistanceLine::trackMovingEdge(const vpImage<unsigned char> &I)
 {
-
   if (isvisible) {
-    //     p1->changeFrame(cMo);
-    //     p2->changeFrame(cMo);
-    //
-    //     p1->projection();
-    //     p2->projection();
-    //
-    //     vpImagePoint ip1, ip2;
-    //
-    //     vpMeterPixelConversion::convertPoint(*cam,p1->get_x(),p1->get_y(),ip1);
-    //     vpMeterPixelConversion::convertPoint(*cam,p2->get_x(),p2->get_y(),ip2);
-    //
-    //     int marge = /*10*/5; //ou 5 normalement
-    //     if (ip1.get_j()<ip2.get_j()) { meline->jmin = ip1.get_j()-marge ;
-    //     meline->jmax = ip2.get_j()+marge ; } else{ meline->jmin =
-    //     ip2.get_j()-marge ; meline->jmax = ip1.get_j()+marge ; } if
-    //     (ip1.get_i()<ip2.get_i()) { meline->imin = ip1.get_i()-marge ;
-    //     meline->imax = ip2.get_i()+marge ; } else{ meline->imin =
-    //     ip2.get_i()-marge ; meline->imax = ip1.get_i()+marge ; }
-
     try {
       nbFeature.clear();
       nbFeatureTotal = 0;
-      for (unsigned int i = 0; i < meline.size(); i++) {
+      for (size_t i = 0; i < meline.size(); i++) {
         meline[i]->track(I);
         nbFeature.push_back((unsigned int)meline[i]->getMeList().size());
         nbFeatureTotal += (unsigned int)meline[i]->getMeList().size();
       }
     } catch (...) {
-      for (unsigned int i = 0; i < meline.size(); i++) {
+      for (size_t i = 0; i < meline.size(); i++) {
         if (meline[i] != NULL)
           delete meline[i];
       }
@@ -503,7 +472,7 @@ void vpMbtDistanceLine::updateMovingEdge(const vpImage<unsigned char> &I, const
       }
 
       if (linesLst.size() != meline.size() || linesLst.size() == 0) {
-        for (unsigned int i = 0; i < meline.size(); i++) {
+        for (size_t i = 0; i < meline.size(); i++) {
           if (meline[i] != NULL)
             delete meline[i];
         }
@@ -514,23 +483,23 @@ void vpMbtDistanceLine::updateMovingEdge(const vpImage<unsigned char> &I, const
         isvisible = false;
         Reinit = true;
       } else {
-
-        // To have the exact same pose values as the old version (angle or
-        // ogre visibility test only), points should be reorganised when using
-        // scanline algorithm.
-        //        if(sqrt(vpMath::sqr(poly.polyClipped[0].first.get_X() -
-        //        linesLst[0].first.get_X())) >
-        //        sqrt(vpMath::sqr(poly.polyClipped[0].first.get_X() -
-        //        linesLst[0].second.get_X())))
-        //        {
-        //            std::vector<std::pair<vpPoint, vpPoint> > linesLstTmp;
-        //            for(int i = linesLst.size()-1 ; i >= 0 ; i--)
-        //              linesLstTmp.push_back(std::make_pair(linesLst[i].second,linesLst[i].first));
-        //            linesLst = linesLstTmp;
-        //        }
-
         line->changeFrame(cMo);
-        line->projection();
+        try {
+          line->projection();
+        }
+        catch(...) {
+          for (size_t j = 0; j < meline.size(); j++) {
+            if (meline[j] != NULL)
+              delete meline[j];
+          }
+
+          meline.clear();
+          nbFeature.clear();
+          nbFeatureTotal = 0;
+          isvisible = false;
+          Reinit = true;
+          return;
+        }
         double rho, theta;
         // rho theta uv
         vpMeterPixelConversion::convertLine(cam, line->getRho(), line->getTheta(), rho, theta);
@@ -578,7 +547,7 @@ void vpMbtDistanceLine::updateMovingEdge(const vpImage<unsigned char> &I, const
             nbFeatureTotal += nbFeature[i];
           }
         } catch (...) {
-          for (unsigned int j = 0; j < meline.size(); j++) {
+          for (size_t j = 0; j < meline.size(); j++) {
             if (meline[j] != NULL)
               delete meline[j];
           }
@@ -591,7 +560,7 @@ void vpMbtDistanceLine::updateMovingEdge(const vpImage<unsigned char> &I, const
         }
       }
     } else {
-      for (unsigned int i = 0; i < meline.size(); i++) {
+      for (size_t i = 0; i < meline.size(); i++) {
         if (meline[i] != NULL)
           delete meline[i];
       }
@@ -611,10 +580,11 @@ void vpMbtDistanceLine::updateMovingEdge(const vpImage<unsigned char> &I, const
 
   \param I : the image.
   \param cMo : The pose of the camera.
+  \param mask: Mask image or NULL if not wanted. Mask values that are set to true are considered in the tracking. To disable a pixel, set false.
 */
-void vpMbtDistanceLine::reinitMovingEdge(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo)
+void vpMbtDistanceLine::reinitMovingEdge(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo, const vpImage<bool> *mask)
 {
-  for (unsigned int i = 0; i < meline.size(); i++) {
+  for (size_t i = 0; i < meline.size(); i++) {
     if (meline[i] != NULL)
       delete meline[i];
   }
@@ -623,7 +593,7 @@ void vpMbtDistanceLine::reinitMovingEdge(const vpImage<unsigned char> &I, const
   meline.clear();
   nbFeatureTotal = 0;
 
-  if (initMovingEdge(I, cMo) == false)
+  if (!initMovingEdge(I, cMo, false, mask))
     Reinit = true;
 
   Reinit = false;
@@ -641,45 +611,16 @@ void vpMbtDistanceLine::reinitMovingEdge(const vpImage<unsigned char> &I, const
   visible.
 */
 void vpMbtDistanceLine::display(const vpImage<unsigned char> &I, const vpHomogeneousMatrix &cMo,
-                                const vpCameraParameters &camera, const vpColor &col, const unsigned int thickness,
-                                const bool displayFullModel)
+                                const vpCameraParameters &camera, const vpColor &col, unsigned int thickness,
+                                bool displayFullModel)
 {
-  if ((isvisible && isTrackedLine) || displayFullModel) {
-    p1->changeFrame(cMo);
-    p2->changeFrame(cMo);
+  std::vector<std::vector<double> > models =
+      getModelForDisplay(I.getWidth(), I.getHeight(), cMo, camera, displayFullModel);
 
-    vpImagePoint ip1, ip2;
-    vpCameraParameters c = camera;
-    if (poly.getClipping() > 3) // Contains at least one FOV constraint
-      c.computeFov(I.getWidth(), I.getHeight());
-
-    poly.computePolygonClipped(c);
-
-    if (poly.polyClipped.size() == 2 &&
-        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::NEAR_CLIPPING) == 0) &&
-        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::FAR_CLIPPING) == 0) &&
-        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::DOWN_CLIPPING) == 0) &&
-        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::UP_CLIPPING) == 0) &&
-        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::LEFT_CLIPPING) == 0) &&
-        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::RIGHT_CLIPPING) == 0)) {
-
-      std::vector<std::pair<vpPoint, vpPoint> > linesLst;
-      if (useScanLine && !displayFullModel) {
-        hiddenface->computeScanLineQuery(poly.polyClipped[0].first, poly.polyClipped[1].first, linesLst, true);
-      } else {
-        linesLst.push_back(std::make_pair(poly.polyClipped[0].first, poly.polyClipped[1].first));
-      }
-
-      for (unsigned int i = 0; i < linesLst.size(); i++) {
-        linesLst[i].first.project();
-        linesLst[i].second.project();
-
-        vpMeterPixelConversion::convertPoint(camera, linesLst[i].first.get_x(), linesLst[i].first.get_y(), ip1);
-        vpMeterPixelConversion::convertPoint(camera, linesLst[i].second.get_x(), linesLst[i].second.get_y(), ip2);
-
-        vpDisplay::displayLine(I, ip1, ip2, col, thickness);
-      }
-    }
+  for (size_t i = 0; i < models.size(); i++) {
+    vpImagePoint ip1(models[i][1], models[i][2]);
+    vpImagePoint ip2(models[i][3], models[i][4]);
+    vpDisplay::displayLine(I, ip1, ip2, col, thickness);
   }
 }
 
@@ -695,45 +636,16 @@ void vpMbtDistanceLine::display(const vpImage<unsigned char> &I, const vpHomogen
   visible.
 */
 void vpMbtDistanceLine::display(const vpImage<vpRGBa> &I, const vpHomogeneousMatrix &cMo,
-                                const vpCameraParameters &camera, const vpColor &col, const unsigned int thickness,
-                                const bool displayFullModel)
+                                const vpCameraParameters &camera, const vpColor &col, unsigned int thickness,
+                                bool displayFullModel)
 {
-  if ((isvisible && isTrackedLine) || displayFullModel) {
-    p1->changeFrame(cMo);
-    p2->changeFrame(cMo);
+  std::vector<std::vector<double> > models =
+      getModelForDisplay(I.getWidth(), I.getHeight(), cMo, camera, displayFullModel);
 
-    vpImagePoint ip1, ip2;
-    vpCameraParameters c = camera;
-    if (poly.getClipping() > 3) // Contains at least one FOV constraint
-      c.computeFov(I.getWidth(), I.getHeight());
-
-    poly.computePolygonClipped(c);
-
-    if (poly.polyClipped.size() == 2 &&
-        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::NEAR_CLIPPING) == 0) &&
-        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::FAR_CLIPPING) == 0) &&
-        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::DOWN_CLIPPING) == 0) &&
-        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::UP_CLIPPING) == 0) &&
-        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::LEFT_CLIPPING) == 0) &&
-        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::RIGHT_CLIPPING) == 0)) {
-
-      std::vector<std::pair<vpPoint, vpPoint> > linesLst;
-      if (useScanLine && !displayFullModel) {
-        hiddenface->computeScanLineQuery(poly.polyClipped[0].first, poly.polyClipped[1].first, linesLst, true);
-      } else {
-        linesLst.push_back(std::make_pair(poly.polyClipped[0].first, poly.polyClipped[1].first));
-      }
-
-      for (unsigned int i = 0; i < linesLst.size(); i++) {
-        linesLst[i].first.project();
-        linesLst[i].second.project();
-
-        vpMeterPixelConversion::convertPoint(camera, linesLst[i].first.get_x(), linesLst[i].first.get_y(), ip1);
-        vpMeterPixelConversion::convertPoint(camera, linesLst[i].second.get_x(), linesLst[i].second.get_y(), ip2);
-
-        vpDisplay::displayLine(I, ip1, ip2, col, thickness);
-      }
-    }
+  for (size_t i = 0; i < models.size(); i++) {
+    vpImagePoint ip1(models[i][1], models[i][2]);
+    vpImagePoint ip2(models[i][3], models[i][4]);
+    vpDisplay::displayLine(I, ip1, ip2, col, thickness);
   }
 }
 
@@ -753,10 +665,126 @@ void vpMbtDistanceLine::display(const vpImage<vpRGBa> &I, const vpHomogeneousMat
 */
 void vpMbtDistanceLine::displayMovingEdges(const vpImage<unsigned char> &I)
 {
-  for (unsigned int i = 0; i < meline.size(); i++)
+  for (size_t i = 0; i < meline.size(); i++) {
     if (meline[i] != NULL) {
       meline[i]->display(I);
     }
+  }
+}
+
+void vpMbtDistanceLine::displayMovingEdges(const vpImage<vpRGBa> &I)
+{
+  for (size_t i = 0; i < meline.size(); i++) {
+    if (meline[i] != NULL) {
+      meline[i]->display(I);
+    }
+  }
+}
+
+/*!
+  Return a list of features parameters for display.
+  - Parameters are: `<feature id (here 0 for ME)>`, `<pt.i()>`, `<pt.j()>`, `<state>`
+*/
+std::vector<std::vector<double> > vpMbtDistanceLine::getFeaturesForDisplay()
+{
+  std::vector<std::vector<double> > features;
+
+  for (size_t i = 0; i < meline.size(); i++) {
+    vpMbtMeLine *me_l = meline[i];
+    if (me_l != NULL) {
+      for (std::list<vpMeSite>::const_iterator it = me_l->getMeList().begin(); it != me_l->getMeList().end(); ++it) {
+        vpMeSite p_me_l = *it;
+#if (VISP_CXX_STANDARD >= VISP_CXX_STANDARD_11)
+        std::vector<double> params = {0, //ME
+                                      p_me_l.get_ifloat(),
+                                      p_me_l.get_jfloat(),
+                                      static_cast<double>(p_me_l.getState())};
+#else
+        std::vector<double> params;
+        params.push_back(0); // ME
+        params.push_back(p_me_l.get_ifloat());
+        params.push_back(p_me_l.get_jfloat());
+        params.push_back(static_cast<double>(p_me_l.getState()));
+#endif
+        features.push_back(params);
+      }
+    }
+  }
+
+  return features;
+}
+
+/*!
+  Return a list of line parameters to display the primitive at a given pose and camera parameters.
+  - Parameters are: `<primitive id (here 0 for line)>`, `<pt_start.i()>`, `<pt_start.j()>`,
+  `<pt_end.i()>`, `<pt_end.j()>`
+
+  \param width : Image width.
+  \param height : Image height.
+  \param cMo : Pose used to project the 3D model into the image.
+  \param camera : The camera parameters.
+  \param displayFullModel : If true, the line is displayed even if it is not
+*/
+std::vector<std::vector<double> > vpMbtDistanceLine::getModelForDisplay(unsigned int width, unsigned int height,
+                                                                        const vpHomogeneousMatrix &cMo,
+                                                                        const vpCameraParameters &camera,
+                                                                        bool displayFullModel)
+{
+  std::vector<std::vector<double> > models;
+
+  if ((isvisible && isTrackedLine) || displayFullModel) {
+    p1->changeFrame(cMo);
+    p2->changeFrame(cMo);
+
+    vpImagePoint ip1, ip2;
+    vpCameraParameters c = camera;
+    if (poly.getClipping() > 3) // Contains at least one FOV constraint
+      c.computeFov(width, height);
+
+    poly.computePolygonClipped(c);
+
+    if (poly.polyClipped.size() == 2 &&
+        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::NEAR_CLIPPING) == 0) &&
+        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::FAR_CLIPPING) == 0) &&
+        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::DOWN_CLIPPING) == 0) &&
+        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::UP_CLIPPING) == 0) &&
+        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::LEFT_CLIPPING) == 0) &&
+        ((poly.polyClipped[1].second & poly.polyClipped[0].second & vpPolygon3D::RIGHT_CLIPPING) == 0)) {
+
+      std::vector<std::pair<vpPoint, vpPoint> > linesLst;
+      if (useScanLine && !displayFullModel) {
+        hiddenface->computeScanLineQuery(poly.polyClipped[0].first, poly.polyClipped[1].first, linesLst, true);
+      } else {
+        linesLst.push_back(std::make_pair(poly.polyClipped[0].first, poly.polyClipped[1].first));
+      }
+
+      for (unsigned int i = 0; i < linesLst.size(); i++) {
+        linesLst[i].first.project();
+        linesLst[i].second.project();
+
+        vpMeterPixelConversion::convertPoint(camera, linesLst[i].first.get_x(), linesLst[i].first.get_y(), ip1);
+        vpMeterPixelConversion::convertPoint(camera, linesLst[i].second.get_x(), linesLst[i].second.get_y(), ip2);
+
+#if (VISP_CXX_STANDARD >= VISP_CXX_STANDARD_11)
+        std::vector<double> params = {0, //0 for line parameters
+                                      ip1.get_i(),
+                                      ip1.get_j(),
+                                      ip2.get_i(),
+                                      ip2.get_j()};
+#else
+        std::vector<double> params;
+        params.push_back(0); //0 for line parameters
+        params.push_back(ip1.get_i());
+        params.push_back(ip1.get_j());
+        params.push_back(ip2.get_i());
+        params.push_back(ip2.get_j());
+#endif
+        models.push_back(params);
+      }
+    }
+  }
+
+  return models;
 }
 
 /*!
@@ -764,11 +792,11 @@ void vpMbtDistanceLine::displayMovingEdges(const vpImage<unsigned char> &I)
 */
 void vpMbtDistanceLine::initInteractionMatrixError()
 {
-  if (isvisible == true) {
+  if (isvisible) {
     L.resize(nbFeatureTotal, 6);
     error.resize(nbFeatureTotal);
   } else {
-    for (unsigned int i = 0; i < meline.size(); i++) {
+    for (size_t i = 0; i < meline.size(); i++) {
       nbFeature[i] = 0;
       // To be consistent with nbFeature[i] = 0
       std::list<vpMeSite> &me_site_list = meline[i]->getMeList();
@@ -809,7 +837,7 @@ void vpMbtDistanceLine::computeInteractionMatrixError(const vpHomogeneousMatrix
       double x, y;
       unsigned int j = 0;
 
-      for (unsigned int i = 0; i < meline.size(); i++) {
+      for (size_t i = 0; i < meline.size(); i++) {
         for (std::list<vpMeSite>::const_iterator it = meline[i]->getMeList().begin();
              it != meline[i]->getMeList().end(); ++it) {
           x = (double)it->j;
@@ -830,12 +858,11 @@ void vpMbtDistanceLine::computeInteractionMatrixError(const vpHomogeneousMatrix
           j++;
         }
       }
-    } catch (const vpException &e) {
-      std::cerr << "Catch an exception: " << e.what() << std::endl;
-      std::cerr << "Set the corresponding interaction matrix part to zero." << std::endl;
-
+    } catch (...) {
+      // Handle potential exception: due to a degenerate case: the image of the straight line is a point!
+      // Set the corresponding interaction matrix part to zero
       unsigned int j = 0;
-      for (unsigned int i = 0; i < meline.size(); i++) {
+      for (size_t i = 0; i < meline.size(); i++) {
         for (std::list<vpMeSite>::const_iterator it = meline[i]->getMeList().begin();
              it != meline[i]->getMeList().end(); ++it) {
           for (unsigned int k = 0; k < 6; k++) {
@@ -865,7 +892,7 @@ bool vpMbtDistanceLine::closeToImageBorder(const vpImage<unsigned char> &I, cons
   }
   if (isvisible) {
 
-    for (unsigned int i = 0; i < meline.size(); i++) {
+    for (size_t i = 0; i < meline.size(); i++) {
       for (std::list<vpMeSite>::const_iterator it = meline[i]->getMeList().begin(); it != meline[i]->getMeList().end();
            ++it) {
         int i_ = it->i;
