@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -76,6 +76,8 @@ protected:
   vpMe *me;
   unsigned int init_range;
   int nGoodElement;
+  //! Mask used to disable tracking on a part of image
+  const vpImage<bool> *m_mask;
 
 protected:
   vpMeSite::vpMeSiteDisplayType selectDisplay;
@@ -97,19 +99,22 @@ public:
 
   virtual void display(const vpImage<unsigned char> &I, vpColor col) = 0;
   virtual void display(const vpImage<unsigned char> &I);
+  virtual void display(const vpImage<vpRGBa> &I);
   void display(const vpImage<unsigned char> &I, vpColVector &w, unsigned int &index_w);
 
   void setDisplay(vpMeSite::vpMeSiteDisplayType select) { selectDisplay = select; }
 
   vpMeTracker &operator=(vpMeTracker &f);
 
+  static bool inMask(const vpImage<bool> *mask, unsigned int i, unsigned int j);
+
   int outOfImage(int i, int j, int half, int row, int cols);
   int outOfImage(const vpImagePoint &iP, int half, int rows, int cols);
 
   void reset();
 
   //! Sample pixels at a given interval
-  virtual void sample(const vpImage<unsigned char> &image) = 0;
+  virtual void sample(const vpImage<unsigned char> &image, bool doNotTrack=false) = 0;
 
   /*!
     Set the initial range.
@@ -125,6 +130,13 @@ public:
   */
   inline unsigned int getInitRange() { return init_range; }
 
+  /*!
+    Set the mask
+
+    \param mask : Mask.
+  */
+  virtual void setMask(const vpImage<bool> &mask) { m_mask = &mask; }
+
   /*!
     Set the moving edges initialisation parameters
 
