@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -141,13 +141,15 @@ vpMeEllipse::~vpMeEllipse()
   the points with the smallest and the biggest \f$ alpha \f$ angle.
 
   \param I : Image in which the ellipse appears.
+  \param doNotTrack : Inherited parameter, not used.
 
   \exception vpTrackingException::initializationError : Moving edges not
   initialized.
 
 */
-void vpMeEllipse::sample(const vpImage<unsigned char> &I)
+void vpMeEllipse::sample(const vpImage<unsigned char> &I, bool doNotTrack)
 {
+  (void)doNotTrack;
   if (!me) {
     throw(vpException(vpException::fatalError, "Moving edges on ellipse tracking not initialized"));
   }
@@ -680,7 +682,7 @@ void vpMeEllipse::initTracking(const vpImage<unsigned char> &I)
   \param n : The number of points in the list.
   \param iP : A pointer to a list of points belonging to the ellipse edge.
 */
-void vpMeEllipse::initTracking(const vpImage<unsigned char> &I, const unsigned int n, vpImagePoint *iP)
+void vpMeEllipse::initTracking(const vpImage<unsigned char> &I, unsigned int n, vpImagePoint *iP)
 {
   vpMatrix A(n, 5);
   vpColVector b_(n);
@@ -922,7 +924,7 @@ void vpMeEllipse::computeAngle(int ip1, int jp1, int ip2, int jp2)
   computeAngle(ip1, jp1, a1, ip2, jp2, a2);
 }
 
-void vpMeEllipse::initTracking(const vpImage<unsigned char> &I, const unsigned int n, unsigned *i, unsigned *j)
+void vpMeEllipse::initTracking(const vpImage<unsigned char> &I, unsigned int n, unsigned *i, unsigned *j)
 {
   vpMatrix A(n, 5);
   vpColVector b_(n);
