@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -139,13 +139,15 @@ vpMeLine::~vpMeLine() { list.clear(); }
   step between the two extremities of the line.
 
   \param I : Image in which the line appears.
+  \param doNotTrack : Inherited parameter, not used.
 
   \exception vpTrackingException::initializationError : Moving edges not
   initialized.
 
 */
-void vpMeLine::sample(const vpImage<unsigned char> &I)
+void vpMeLine::sample(const vpImage<unsigned char> &I, bool doNotTrack)
 {
+  (void)doNotTrack;
   if (!me) {
     vpDERROR_TRACE(2, "Tracking error: Moving edges not initialized");
     throw(vpTrackingException(vpTrackingException::initializationError, "Moving edges not initialized"));
