@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@
 /*
   Compute the distance d = |Pw1-Pw2|
 */
-inline double distance(const vpImagePoint &iP1, const double w1, const vpImagePoint &iP2, const double w2)
+inline double distance(const vpImagePoint &iP1, double w1, const vpImagePoint &iP2, double w2)
 {
   double distancei = iP1.get_i() - iP2.get_i();
   double distancej = iP1.get_j() - iP2.get_j();
