@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,8 +41,7 @@
 #include <visp3/tt/vpTemplateTrackerSSDInverseCompositional.h>
 
 vpTemplateTrackerSSDInverseCompositional::vpTemplateTrackerSSDInverseCompositional(vpTemplateTrackerWarp *warp)
-  : vpTemplateTrackerSSD(warp), compoInitialised(false), HInv(), HCompInverse(), useTemplateSelect(false), evolRMS(0),
-    x_pos(), y_pos(), threshold_RMS(1e-8)
+  : vpTemplateTrackerSSD(warp), compoInitialised(false), HInv(), HCompInverse(), useTemplateSelect(false)
 {
   useInverse = true;
   HInv.resize(nbParam, nbParam);
@@ -51,7 +50,6 @@ vpTemplateTrackerSSDInverseCompositional::vpTemplateTrackerSSDInverseComposition
 
 void vpTemplateTrackerSSDInverseCompositional::initCompInverse(const vpImage<unsigned char> & /*I*/)
 {
-
   H = 0;
   int i, j;
 
@@ -59,9 +57,6 @@ void vpTemplateTrackerSSDInverseCompositional::initCompInverse(const vpImage<uns
     if ((!useTemplateSelect) || (ptTemplateSelect[point])) {
       i = ptTemplate[point].y;
       j = ptTemplate[point].x;
-      X1[0] = j;
-      X1[1] = i;
-      Warp->computeDenom(X1, p);
       ptTemplate[point].dW = new double[nbParam];
 
       Warp->getdW0(i, j, ptTemplate[point].dy, ptTemplate[point].dx, ptTemplate[point].dW);
@@ -77,18 +72,15 @@ void vpTemplateTrackerSSDInverseCompositional::initCompInverse(const vpImage<uns
 
   HCompInverse.resize(nbParam, nbParam);
   HCompInverse = HLMtemp.inverseByLU();
-  // std::cout<<Hinverse<<std::endl;
   vpColVector dWtemp(nbParam);
   vpColVector HiGtemp(nbParam);
 
   for (unsigned int point = 0; point < templateSize; point++) {
     if ((!useTemplateSelect) || (ptTemplateSelect[point])) {
-      // i=ptTemplate[point].y;
-      // j=ptTemplate[point].x;
       for (unsigned int it = 0; it < nbParam; it++)
         dWtemp[it] = ptTemplate[point].dW[it];
 
-      HiGtemp = -1. * HCompInverse * dWtemp;
+      HiGtemp = - HCompInverse * dWtemp;
       ptTemplate[point].HiG = new double[nbParam];
 
       for (unsigned int it = 0; it < nbParam; it++)
@@ -115,10 +107,14 @@ void vpTemplateTrackerSSDInverseCompositional::trackNoPyr(const vpImage<unsigned
   int i, j;
   double i2, j2;
   double alpha = 2.;
-  // vpTemplateTrackerPointtest *pt;
   initPosEvalRMS(p);
 
   vpTemplateTrackerPoint *pt;
+
+  double evolRMS_init = 0;
+  double evolRMS_prec = 0;
+  double evolRMS_delta;
+
   do {
     unsigned int Nbpoint = 0;
     double erreur = 0;
@@ -126,7 +122,6 @@ void vpTemplateTrackerSSDInverseCompositional::trackNoPyr(const vpImage<unsigned
     Warp->computeCoeff(p);
     for (unsigned int point = 0; point < templateSize; point++) {
       if ((!useTemplateSelect) || (ptTemplateSelect[point])) {
-        // pt=&ptTemplatetest[point];
         pt = &ptTemplate[point];
         i = pt->y;
         j = pt->x;
@@ -152,14 +147,10 @@ void vpTemplateTrackerSSDInverseCompositional::trackNoPyr(const vpImage<unsigned
         }
       }
     }
-    // std::cout << "npoint: " << Nbpoint << std::endl;
     if (Nbpoint == 0) {
-      // std::cout<<"plus de point dans template suivi"<<std::endl;
-      deletePosEvalRMS();
       throw(vpTrackingException(vpTrackingException::notEnoughPointError, "No points in the template"));
     }
     dp = gain * dp;
-    // std::cout<<erreur/Nbpoint<<","<<GetCost(I,p)<<std::endl;
     if (useBrent) {
       alpha = 2.;
       computeOptimalBrentGain(I, p, erreur / Nbpoint, dp, alpha);
@@ -167,62 +158,17 @@ void vpTemplateTrackerSSDInverseCompositional::trackNoPyr(const vpImage<unsigned
     }
     Warp->getParamInverse(dp, dpinv);
     Warp->pRondp(p, dpinv, p);
-    iteration++;
 
     computeEvalRMS(p);
-    // std::cout << "iteration: " << iteration << " max: " << iterationMax <<
-    // std::endl;  std::cout << "evolRMS: " <<  evolRMS << " threshold: " <<
-    // threshold_RMS << std::endl;
-  } while (/*( erreur_prec-erreur<50) &&*/ (iteration < iterationMax) && (evolRMS > threshold_RMS));
 
+    if (iteration == 0) {
+      evolRMS_init = evolRMS;
+    }
+    iteration++;
+
+    evolRMS_delta = std::fabs(evolRMS - evolRMS_prec);
+    evolRMS_prec = evolRMS;
+
+  } while ( (iteration < iterationMax) && (evolRMS_delta > std::fabs(evolRMS_init)*evolRMS_eps) );
   nbIteration = iteration;
-  deletePosEvalRMS();
 }
-
-void vpTemplateTrackerSSDInverseCompositional::initPosEvalRMS(const vpColVector &p_)
-{
-  unsigned int nb_corners = zoneTracked->getNbTriangle() * 3;
-  x_pos.resize(nb_corners);
-  y_pos.resize(nb_corners);
-
-  Warp->computeCoeff(p_);
-  vpTemplateTrackerTriangle triangle;
-
-  for (unsigned int i = 0; i < zoneTracked->getNbTriangle(); i++) {
-    zoneTracked->getTriangle(i, triangle);
-    for (unsigned int j = 0; j < 3; j++) {
-      triangle.getCorner(j, X1[0], X1[1]);
-
-      Warp->computeDenom(X1, p_);
-      Warp->warpX(X1, X2, p_);
-      x_pos[i * 3 + j] = X2[0];
-      y_pos[i * 3 + j] = X2[1];
-    }
-  }
-}
-
-void vpTemplateTrackerSSDInverseCompositional::computeEvalRMS(const vpColVector &p_)
-{
-  unsigned int nb_corners = zoneTracked->getNbTriangle() * 3;
-
-  Warp->computeCoeff(p_);
-  evolRMS = 0;
-  vpTemplateTrackerTriangle triangle;
-
-  for (unsigned int i = 0; i < zoneTracked->getNbTriangle(); i++) {
-    zoneTracked->getTriangle(i, triangle);
-    for (unsigned int j = 0; j < 3; j++) {
-      triangle.getCorner(j, X1[0], X1[1]);
-
-      Warp->computeDenom(X1, p_);
-      Warp->warpX(X1, X2, p_);
-      evolRMS += (x_pos[i * 3 + j] - X2[0]) * (x_pos[i * 3 + j] - X2[0]) +
-                 (y_pos[i * 3 + j] - X2[1]) * (y_pos[i * 3 + j] - X2[1]);
-      x_pos[i * 3 + j] = X2[0];
-      y_pos[i * 3 + j] = X2[1];
-    }
-  }
-  evolRMS = evolRMS / nb_corners;
-}
-
-void vpTemplateTrackerSSDInverseCompositional::deletePosEvalRMS() {}
