@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -443,7 +443,7 @@ vpRect vpTemplateTrackerZone::getBoundingBox() const
   triangles that define the zone. \param I : Image. \param col : Color used to
   display the triangles. \param thickness : Thickness of the triangle lines.
  */
-void vpTemplateTrackerZone::display(const vpImage<unsigned char> &I, const vpColor &col, const unsigned int thickness)
+void vpTemplateTrackerZone::display(const vpImage<unsigned char> &I, const vpColor &col, unsigned int thickness)
 {
   std::vector<vpImagePoint> ip;
   for (unsigned int i = 0; i < Zone.size(); i++) {
@@ -460,7 +460,7 @@ void vpTemplateTrackerZone::display(const vpImage<unsigned char> &I, const vpCol
   triangles that define the zone. \param I : Image. \param col : Color used to
   display the triangles. \param thickness : Thickness of the triangle lines.
  */
-void vpTemplateTrackerZone::display(const vpImage<vpRGBa> &I, const vpColor &col, const unsigned int thickness)
+void vpTemplateTrackerZone::display(const vpImage<vpRGBa> &I, const vpColor &col, unsigned int thickness)
 {
   std::vector<vpImagePoint> ip;
   for (unsigned int i = 0; i < Zone.size(); i++) {
