@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,7 +42,8 @@
 #include <visp3/tt/vpTemplateTrackerBSpline.h>
 
 vpTemplateTracker::vpTemplateTracker(vpTemplateTrackerWarp *_warp)
-  : nbLvlPyr(1), l0Pyr(0), pyrInitialised(false), ptTemplate(NULL), ptTemplatePyr(NULL), ptTemplateInit(false),
+  : nbLvlPyr(1), l0Pyr(0), pyrInitialised(false), evolRMS(0), x_pos(), y_pos(),
+    evolRMS_eps(1e-4), ptTemplate(NULL), ptTemplatePyr(NULL), ptTemplateInit(false),
     templateSize(0), templateSizePyr(NULL), ptTemplateSelect(NULL), ptTemplateSelectPyr(NULL),
     ptTemplateSelectInit(false), templateSelectSize(0), ptTemplateSupp(NULL), ptTemplateSuppPyr(NULL),
     ptTemplateCompo(NULL), ptTemplateCompoPyr(NULL), zoneTracked(NULL), zoneTrackedPyr(NULL), pyr_IDes(NULL), H(),
@@ -337,7 +338,7 @@ int main()
 }
   \endcode
  */
-void vpTemplateTracker::display(const vpImage<unsigned char> &I, const vpColor &col, const unsigned int thickness)
+void vpTemplateTracker::display(const vpImage<unsigned char> &I, const vpColor &col, unsigned int thickness)
 {
   if (I.display) { // Only if a display is associated to the image
     vpTemplateTrackerZone zoneWarped;
@@ -379,7 +380,7 @@ int main()
 }
   \endcode
  */
-void vpTemplateTracker::display(const vpImage<vpRGBa> &I, const vpColor &col, const unsigned int thickness)
+void vpTemplateTracker::display(const vpImage<vpRGBa> &I, const vpColor &col, unsigned int thickness)
 {
   if (I.display) { // Only if a display is associated to the image
     vpTemplateTrackerZone zoneWarped;
@@ -747,7 +748,7 @@ void vpTemplateTracker::initHessienDesiredPyr(const vpImage<unsigned char> &I)
     HdesirePyr[0] = Hdesire;
     HLMdesirePyr[0] = HLMdesire;
     HLMdesireInversePyr[0] = HLMdesireInverse;
-  } catch (vpException &e) {
+  } catch (const vpException &e) {
     ptTemplateSuppPyr[0] = ptTemplateSupp;
     ptTemplateCompoPyr[0] = ptTemplateCompo;
     HdesirePyr[0] = Hdesire;
@@ -774,7 +775,7 @@ void vpTemplateTracker::initHessienDesiredPyr(const vpImage<unsigned char> &I)
         HdesirePyr[i] = Hdesire;
         HLMdesirePyr[i] = HLMdesire;
         HLMdesireInversePyr[i] = HLMdesireInverse;
-      } catch (vpException &e) {
+      } catch (const vpException &e) {
         ptTemplateSuppPyr[i] = ptTemplateSupp;
         ptTemplateCompoPyr[i] = ptTemplateCompo;
         HdesirePyr[i] = Hdesire;
@@ -890,7 +891,7 @@ void vpTemplateTracker::trackPyr(const vpImage<unsigned char> &I)
       trackRobust(I);
     }
     delete[] pyr_I;
-  } catch (vpException &e) {
+  } catch (const vpException &e) {
     delete[] pyr_I;
     throw(vpTrackingException(vpTrackingException::badValue, e.getMessage()));
   }
@@ -914,3 +915,65 @@ void vpTemplateTracker::trackRobust(const vpImage<unsigned char> &I)
   } else
     trackNoPyr(I);
 }
+
+/*!
+  Compute residual. Before using this function you need to call initPosEvalRMS() once.
+  \param[in] param : Warp function parameters.
+
+  \sa initPosEvalRMS()
+ */
+void vpTemplateTracker::computeEvalRMS(const vpColVector &param)
+{
+  unsigned int nb_corners = zoneTracked->getNbTriangle() * 3;
+
+  Warp->computeCoeff(param);
+  evolRMS = 0;
+  vpTemplateTrackerTriangle triangle;
+
+  for (unsigned int i = 0; i < zoneTracked->getNbTriangle(); i++) {
+    zoneTracked->getTriangle(i, triangle);
+    for (unsigned int j = 0; j < 3; j++) {
+      triangle.getCorner(j, X1[0], X1[1]);
+
+      Warp->computeDenom(X1, param);
+      Warp->warpX(X1, X2, param);
+
+      unsigned int index = i * 3 + j;
+      double x_ = x_pos[index] - X2[0];
+      double y_ = y_pos[index] - X2[1];
+      evolRMS += x_ * x_ + y_ * y_;
+      x_pos[index] = X2[0];
+      y_pos[index] = X2[1];
+    }
+  }
+  evolRMS /= nb_corners;
+}
+
+/*!
+  Initialize residual computed using computeEvalRMS().
+  \param[in] param : Warp function parameters.
+
+  \sa computeEvalRMS()
+ */
+void vpTemplateTracker::initPosEvalRMS(const vpColVector &param)
+{
+  unsigned int nb_corners = zoneTracked->getNbTriangle() * 3;
+  x_pos.resize(nb_corners);
+  y_pos.resize(nb_corners);
+
+  Warp->computeCoeff(param);
+  vpTemplateTrackerTriangle triangle;
+
+  for (unsigned int i = 0; i < zoneTracked->getNbTriangle(); i++) {
+    unsigned int i3 = i * 3;
+    zoneTracked->getTriangle(i, triangle);
+    for (unsigned int j = 0; j < 3; j++) {
+      triangle.getCorner(j, X1[0], X1[1]);
+
+      Warp->computeDenom(X1, param);
+      Warp->warpX(X1, X2, param);
+      x_pos[i3 + j] = X2[0];
+      y_pos[i3 + j] = X2[1];
+    }
+  }
+}
