@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,8 +35,8 @@
  * Souriya Trinh
  *
  *****************************************************************************/
-#ifndef __vpKeyPoint_h__
-#define __vpKeyPoint_h__
+#ifndef _vpKeyPoint_h_
+#define _vpKeyPoint_h_
 
 #include <algorithm> // std::transform
 #include <float.h>   // DBL_MAX
@@ -57,7 +57,7 @@
 #include <visp3/vision/vpBasicKeyPoint.h>
 #include <visp3/vision/vpPose.h>
 #ifdef VISP_HAVE_MODULE_IO
-#include <visp3/io/vpImageIo.h>
+#  include <visp3/io/vpImageIo.h>
 #endif
 #include <visp3/core/vpConvert.h>
 #include <visp3/core/vpCylinder.h>
@@ -68,20 +68,21 @@
 // Require at least OpenCV >= 2.1.1
 #if (VISP_HAVE_OPENCV_VERSION >= 0x020101)
 
-#include <opencv2/calib3d/calib3d.hpp>
-#include <opencv2/features2d/features2d.hpp>
-#include <opencv2/imgproc/imgproc.hpp>
+#  include <opencv2/calib3d/calib3d.hpp>
+#  include <opencv2/features2d/features2d.hpp>
+#  include <opencv2/imgproc/imgproc.hpp>
 
-#if defined(VISP_HAVE_OPENCV_XFEATURES2D) // OpenCV >= 3.0.0
-#include <opencv2/xfeatures2d.hpp>
-#elif defined(VISP_HAVE_OPENCV_NONFREE) && (VISP_HAVE_OPENCV_VERSION >= 0x020400) &&                                   \
-    (VISP_HAVE_OPENCV_VERSION < 0x030000)
-#include <opencv2/nonfree/nonfree.hpp>
-#endif
+#  if (VISP_HAVE_OPENCV_VERSION >= 0x040000) // Require opencv >= 4.0.0
+#    include <opencv2/imgproc/imgproc_c.h>
+#    include <opencv2/imgproc.hpp>
+#  endif
 
-#ifdef VISP_HAVE_XML2
-#include <libxml/xmlwriter.h>
-#endif
+#  if defined(VISP_HAVE_OPENCV_XFEATURES2D) // OpenCV >= 3.0.0
+#    include <opencv2/xfeatures2d.hpp>
+#  elif defined(VISP_HAVE_OPENCV_NONFREE) && (VISP_HAVE_OPENCV_VERSION >= 0x020400) && \
+     (VISP_HAVE_OPENCV_VERSION < 0x030000)
+#    include <opencv2/nonfree/nonfree.hpp>
+#  endif
 
 /*!
   \class vpKeyPoint
@@ -130,7 +131,6 @@ keypoints and the index of the query keypoints.
 between current and reference keypoints.
 
   \code
-#include <visp3/core/vpConfig.h>
 #include <visp3/core/vpImage.h>
 #include <visp3/vision/vpKeyPoint.h>
 
@@ -167,7 +167,6 @@ int main()
 image. The small following example shows how to do this:
 
   \code
-#include <visp3/core/vpConfig.h>
 #include <visp3/core/vpDisplay.h>
 #include <visp3/core/vpImage.h>
 #include <visp3/vision/vpKeyPoint.h>
@@ -322,15 +321,26 @@ public:
              const vpFilterMatchingType &filterType = ratioDistanceThreshold);
 
   unsigned int buildReference(const vpImage<unsigned char> &I);
-  unsigned int buildReference(const vpImage<unsigned char> &I, const vpImagePoint &iP, const unsigned int height,
-                              const unsigned int width);
+  unsigned int buildReference(const vpImage<unsigned char> &I, const vpImagePoint &iP, unsigned int height,
+                              unsigned int width);
   unsigned int buildReference(const vpImage<unsigned char> &I, const vpRect &rectangle);
 
   void buildReference(const vpImage<unsigned char> &I, std::vector<cv::KeyPoint> &trainKeyPoint,
-                      std::vector<cv::Point3f> &points3f, const bool append = false, const int class_id = -1);
+                      std::vector<cv::Point3f> &points3f, bool append = false, int class_id = -1);
   void buildReference(const vpImage<unsigned char> &I, const std::vector<cv::KeyPoint> &trainKeyPoint,
                       const cv::Mat &trainDescriptors, const std::vector<cv::Point3f> &points3f,
-                      const bool append = false, const int class_id = -1);
+                      bool append = false, int class_id = -1);
+
+  unsigned int buildReference(const vpImage<vpRGBa> &I_color);
+  unsigned int buildReference(const vpImage<vpRGBa> &I_color, const vpImagePoint &iP, unsigned int height,
+                              unsigned int width);
+  unsigned int buildReference(const vpImage<vpRGBa> &I_color, const vpRect &rectangle);
+
+  void buildReference(const vpImage<vpRGBa> &I_color, std::vector<cv::KeyPoint> &trainKeyPoint,
+                      std::vector<cv::Point3f> &points3f, bool append = false, int class_id = -1);
+  void buildReference(const vpImage<vpRGBa> &I, const std::vector<cv::KeyPoint> &trainKeyPoint,
+                      const cv::Mat &trainDescriptors, const std::vector<cv::Point3f> &points3f,
+                      bool append = false, int class_id = -1);
 
   static void compute3D(const cv::KeyPoint &candidate, const std::vector<vpPoint> &roi, const vpCameraParameters &cam,
                         const vpHomogeneousMatrix &cMo, cv::Point3f &point);
@@ -364,24 +374,32 @@ public:
 
   bool computePose(const std::vector<cv::Point2f> &imagePoints, const std::vector<cv::Point3f> &objectPoints,
                    const vpCameraParameters &cam, vpHomogeneousMatrix &cMo, std::vector<int> &inlierIndex,
-                   double &elapsedTime, bool (*func)(vpHomogeneousMatrix *) = NULL);
+                   double &elapsedTime, bool (*func)(const vpHomogeneousMatrix &) = NULL);
 
   bool computePose(const std::vector<vpPoint> &objectVpPoints, vpHomogeneousMatrix &cMo, std::vector<vpPoint> &inliers,
-                   double &elapsedTime, bool (*func)(vpHomogeneousMatrix *) = NULL);
+                   double &elapsedTime, bool (*func)(const vpHomogeneousMatrix &) = NULL);
 
   bool computePose(const std::vector<vpPoint> &objectVpPoints, vpHomogeneousMatrix &cMo, std::vector<vpPoint> &inliers,
                    std::vector<unsigned int> &inlierIndex, double &elapsedTime,
-                   bool (*func)(vpHomogeneousMatrix *) = NULL);
+                   bool (*func)(const vpHomogeneousMatrix &) = NULL);
 
   void createImageMatching(vpImage<unsigned char> &IRef, vpImage<unsigned char> &ICurrent,
                            vpImage<unsigned char> &IMatching);
   void createImageMatching(vpImage<unsigned char> &ICurrent, vpImage<unsigned char> &IMatching);
 
+  void createImageMatching(vpImage<unsigned char> &IRef, vpImage<vpRGBa> &ICurrent,
+                           vpImage<vpRGBa> &IMatching);
+  void createImageMatching(vpImage<vpRGBa> &ICurrent, vpImage<vpRGBa> &IMatching);
+
   void detect(const vpImage<unsigned char> &I, std::vector<cv::KeyPoint> &keyPoints,
               const vpRect &rectangle = vpRect());
+  void detect(const vpImage<vpRGBa> &I_color, std::vector<cv::KeyPoint> &keyPoints,
+              const vpRect &rectangle = vpRect());
   void detect(const cv::Mat &matImg, std::vector<cv::KeyPoint> &keyPoints, const cv::Mat &mask = cv::Mat());
   void detect(const vpImage<unsigned char> &I, std::vector<cv::KeyPoint> &keyPoints, double &elapsedTime,
               const vpRect &rectangle = vpRect());
+  void detect(const vpImage<vpRGBa> &I_color, std::vector<cv::KeyPoint> &keyPoints, double &elapsedTime,
+              const vpRect &rectangle = vpRect());
   void detect(const cv::Mat &matImg, std::vector<cv::KeyPoint> &keyPoints, double &elapsedTime,
               const cv::Mat &mask = cv::Mat());
 
@@ -391,19 +409,32 @@ public:
 
   void display(const vpImage<unsigned char> &IRef, const vpImage<unsigned char> &ICurrent, unsigned int size = 3);
   void display(const vpImage<unsigned char> &ICurrent, unsigned int size = 3, const vpColor &color = vpColor::green);
+  void display(const vpImage<vpRGBa> &IRef, const vpImage<vpRGBa> &ICurrent, unsigned int size = 3);
+  void display(const vpImage<vpRGBa> &ICurrent, unsigned int size = 3, const vpColor &color = vpColor::green);
 
   void displayMatching(const vpImage<unsigned char> &IRef, vpImage<unsigned char> &IMatching, unsigned int crossSize,
                        unsigned int lineThickness = 1, const vpColor &color = vpColor::green);
   void displayMatching(const vpImage<unsigned char> &ICurrent, vpImage<unsigned char> &IMatching,
                        const std::vector<vpImagePoint> &ransacInliers = std::vector<vpImagePoint>(),
                        unsigned int crossSize = 3, unsigned int lineThickness = 1);
+  void displayMatching(const vpImage<unsigned char> &IRef, vpImage<vpRGBa> &IMatching, unsigned int crossSize,
+                       unsigned int lineThickness = 1, const vpColor &color = vpColor::green);
+  void displayMatching(const vpImage<vpRGBa> &IRef, vpImage<vpRGBa> &IMatching, unsigned int crossSize,
+                       unsigned int lineThickness = 1, const vpColor &color = vpColor::green);
+  void displayMatching(const vpImage<vpRGBa> &ICurrent, vpImage<vpRGBa> &IMatching,
+                       const std::vector<vpImagePoint> &ransacInliers = std::vector<vpImagePoint>(),
+                       unsigned int crossSize = 3, unsigned int lineThickness = 1);
 
   void extract(const vpImage<unsigned char> &I, std::vector<cv::KeyPoint> &keyPoints, cv::Mat &descriptors,
                std::vector<cv::Point3f> *trainPoints = NULL);
+  void extract(const vpImage<vpRGBa> &I_color, std::vector<cv::KeyPoint> &keyPoints, cv::Mat &descriptors,
+               std::vector<cv::Point3f> *trainPoints = NULL);
   void extract(const cv::Mat &matImg, std::vector<cv::KeyPoint> &keyPoints, cv::Mat &descriptors,
                std::vector<cv::Point3f> *trainPoints = NULL);
   void extract(const vpImage<unsigned char> &I, std::vector<cv::KeyPoint> &keyPoints, cv::Mat &descriptors,
                double &elapsedTime, std::vector<cv::Point3f> *trainPoints = NULL);
+  void extract(const vpImage<vpRGBa> &I_color, std::vector<cv::KeyPoint> &keyPoints, cv::Mat &descriptors,
+               double &elapsedTime, std::vector<cv::Point3f> *trainPoints = NULL);
   void extract(const cv::Mat &matImg, std::vector<cv::KeyPoint> &keyPoints, cv::Mat &descriptors, double &elapsedTime,
                std::vector<cv::Point3f> *trainPoints = NULL);
 
@@ -603,7 +634,9 @@ public:
 
     \return The number of train images.
   */
-  inline unsigned int getNbImages() const { return static_cast<unsigned int>(m_mapOfImages.size()); }
+  inline unsigned int getNbImages() const {
+    return static_cast<unsigned int>(m_mapOfImages.size());
+  }
 
   void getObjectPoints(std::vector<cv::Point3f> &objectPoints) const;
   void getObjectPoints(std::vector<vpPoint> &objectPoints) const;
@@ -660,24 +693,28 @@ public:
                            vpImage<unsigned char> &IMatching);
   void insertImageMatching(const vpImage<unsigned char> &ICurrent, vpImage<unsigned char> &IMatching);
 
-#ifdef VISP_HAVE_XML2
+  void insertImageMatching(const vpImage<vpRGBa> &IRef, const vpImage<vpRGBa> &ICurrent,
+                           vpImage<vpRGBa> &IMatching);
+  void insertImageMatching(const vpImage<vpRGBa> &ICurrent, vpImage<vpRGBa> &IMatching);
+
+#ifdef VISP_HAVE_PUGIXML
   void loadConfigFile(const std::string &configFile);
 #endif
 
-  void loadLearningData(const std::string &filename, const bool binaryMode = false, const bool append = false);
+  void loadLearningData(const std::string &filename, bool binaryMode = false, bool append = false);
 
   void match(const cv::Mat &trainDescriptors, const cv::Mat &queryDescriptors, std::vector<cv::DMatch> &matches,
              double &elapsedTime);
 
   unsigned int matchPoint(const vpImage<unsigned char> &I);
-  unsigned int matchPoint(const vpImage<unsigned char> &I, const vpImagePoint &iP, const unsigned int height,
-                          const unsigned int width);
+  unsigned int matchPoint(const vpImage<unsigned char> &I, const vpImagePoint &iP, unsigned int height,
+                          unsigned int width);
   unsigned int matchPoint(const vpImage<unsigned char> &I, const vpRect &rectangle);
 
   bool matchPoint(const vpImage<unsigned char> &I, const vpCameraParameters &cam, vpHomogeneousMatrix &cMo,
-                  bool (*func)(vpHomogeneousMatrix *) = NULL, const vpRect &rectangle = vpRect());
+                  bool (*func)(const vpHomogeneousMatrix &) = NULL, const vpRect &rectangle = vpRect());
   bool matchPoint(const vpImage<unsigned char> &I, const vpCameraParameters &cam, vpHomogeneousMatrix &cMo,
-                  double &error, double &elapsedTime, bool (*func)(vpHomogeneousMatrix *) = NULL,
+                  double &error, double &elapsedTime, bool (*func)(const vpHomogeneousMatrix &) = NULL,
                   const vpRect &rectangle = vpRect());
 
   bool matchPointAndDetect(const vpImage<unsigned char> &I, vpRect &boundingBox, vpImagePoint &centerOfGravity,
@@ -687,12 +724,23 @@ public:
 
   bool matchPointAndDetect(const vpImage<unsigned char> &I, const vpCameraParameters &cam, vpHomogeneousMatrix &cMo,
                            double &error, double &elapsedTime, vpRect &boundingBox, vpImagePoint &centerOfGravity,
-                           bool (*func)(vpHomogeneousMatrix *) = NULL, const vpRect &rectangle = vpRect());
+                           bool (*func)(const vpHomogeneousMatrix &) = NULL, const vpRect &rectangle = vpRect());
+
+  unsigned int matchPoint(const vpImage<vpRGBa> &I_color);
+  unsigned int matchPoint(const vpImage<vpRGBa> &I_color, const vpImagePoint &iP, unsigned int height,
+                          unsigned int width);
+  unsigned int matchPoint(const vpImage<vpRGBa> &I_color, const vpRect &rectangle);
+
+  bool matchPoint(const vpImage<vpRGBa> &I_color, const vpCameraParameters &cam, vpHomogeneousMatrix &cMo,
+                  bool (*func)(const vpHomogeneousMatrix &) = NULL, const vpRect &rectangle = vpRect());
+  bool matchPoint(const vpImage<vpRGBa> &I_color, const vpCameraParameters &cam, vpHomogeneousMatrix &cMo,
+                  double &error, double &elapsedTime, bool (*func)(const vpHomogeneousMatrix &) = NULL,
+                  const vpRect &rectangle = vpRect());
 
   void reset();
 
-  void saveLearningData(const std::string &filename, const bool binaryMode = false,
-                        const bool saveTrainingImages = true);
+  void saveLearningData(const std::string &filename, bool binaryMode = false,
+                        bool saveTrainingImages = true);
 
   /*!
     Set if the covariance matrix has to be computed in the Virtual Visual
@@ -930,7 +978,7 @@ public:
 
     \param ratio : Ratio value (]0 ; 1])
   */
-  inline void setMatchingRatioThreshold(const double ratio)
+  inline void setMatchingRatioThreshold(double ratio)
   {
     if (ratio > 0.0 && (ratio < 1.0 || std::fabs(ratio - 1.0) < std::numeric_limits<double>::epsilon())) {
       m_matchingRatioThreshold = ratio;
@@ -945,7 +993,7 @@ public:
 
     \param percentage : Percentage value (]0 ; 100])
   */
-  inline void setRansacConsensusPercentage(const double percentage)
+  inline void setRansacConsensusPercentage(double percentage)
   {
     if (percentage > 0.0 &&
         (percentage < 100.0 || std::fabs(percentage - 100.0) < std::numeric_limits<double>::epsilon())) {
@@ -955,13 +1003,21 @@ public:
     }
   }
 
+  /*!
+    Set filter flag for RANSAC pose estimation.
+  */
+  inline void setRansacFilterFlag(const vpPose::RANSAC_FILTER_FLAGS &flag)
+  {
+    m_ransacFilterFlag = flag;
+  }
+
   /*!
     Set the maximum number of iterations for the Ransac pose estimation
     method.
 
     \param nbIter : Maximum number of iterations for the Ransac
   */
-  inline void setRansacIteration(const int nbIter)
+  inline void setRansacIteration(int nbIter)
   {
     if (nbIter > 0) {
       m_nbRansacIterations = nbIter;
@@ -970,6 +1026,27 @@ public:
     }
   }
 
+  /*!
+    Use or not the multithreaded version.
+
+    \note Needs C++11 or higher.
+  */
+  inline void setRansacParallel(bool parallel)
+  {
+    m_ransacParallel = parallel;
+  }
+
+  /*!
+    Set the number of threads to use if multithreaded RANSAC pose.
+
+    \param nthreads : Number of threads, if 0 the number of CPU threads will be determined
+    \sa setRansacParallel
+  */
+  inline void setRansacParallelNbThreads(unsigned int nthreads)
+  {
+    m_ransacParallelNbThreads = nthreads;
+  }
+
   /*!
     Set the maximum reprojection error (in pixel) to determine if a point is
     an inlier or not.
@@ -977,7 +1054,7 @@ public:
     \param reprojectionError : Maximum reprojection error in pixel (used by
     OpenCV function)
   */
-  inline void setRansacReprojectionError(const double reprojectionError)
+  inline void setRansacReprojectionError(double reprojectionError)
   {
     if (reprojectionError > 0.0) {
       m_ransacReprojectionError = reprojectionError;
@@ -993,7 +1070,7 @@ public:
 
     \param minCount : Minimum number of inlier for the consensus
   */
-  inline void setRansacMinInlierCount(const int minCount)
+  inline void setRansacMinInlierCount(int minCount)
   {
     if (minCount > 0) {
       m_nbRansacMinInlierCount = minCount;
@@ -1008,7 +1085,7 @@ public:
 
     \param threshold : Maximum error in meter for ViSP function
   */
-  inline void setRansacThreshold(const double threshold)
+  inline void setRansacThreshold(double threshold)
   {
     if (threshold > 0.0) {
       m_ransacThreshold = threshold;
@@ -1024,7 +1101,7 @@ public:
     \param useAffine : True to use multiple affine transformations, false
     otherwise
   */
-  inline void setUseAffineDetection(const bool useAffine) { m_useAffineDetection = useAffine; }
+  inline void setUseAffineDetection(bool useAffine) { m_useAffineDetection = useAffine; }
 
 #if (VISP_HAVE_OPENCV_VERSION >= 0x020400 && VISP_HAVE_OPENCV_VERSION < 0x030000)
   /*!
@@ -1033,7 +1110,7 @@ public:
 
     \param useCrossCheck : True to use cross check, false otherwise
   */
-  inline void setUseBruteForceCrossCheck(const bool useCrossCheck)
+  inline void setUseBruteForceCrossCheck(bool useCrossCheck)
   {
     // Only available with BruteForce and with k=1 (i.e not used with a
     // ratioDistanceThreshold method)
@@ -1053,7 +1130,7 @@ public:
     \param useMatchTrainToQuery : True to match the train keypoints to the
     query keypoints
    */
-  inline void setUseMatchTrainToQuery(const bool useMatchTrainToQuery)
+  inline void setUseMatchTrainToQuery(bool useMatchTrainToQuery)
   {
     m_useMatchTrainToQuery = useMatchTrainToQuery;
   }
@@ -1065,7 +1142,7 @@ public:
     \param usePercentage : True to a percentage ratio of inliers, otherwise
     use a specified number of inliers
   */
-  inline void setUseRansacConsensusPercentage(const bool usePercentage) { m_useConsensusPercentage = usePercentage; }
+  inline void setUseRansacConsensusPercentage(bool usePercentage) { m_useConsensusPercentage = usePercentage; }
 
   /*!
     Set the flag to choose between the OpenCV or ViSP Ransac pose estimation
@@ -1074,7 +1151,7 @@ public:
     \param ransacVVS : True to use ViSP function, otherwise use OpenCV
     function
   */
-  inline void setUseRansacVVS(const bool ransacVVS) { m_useRansacVVS = ransacVVS; }
+  inline void setUseRansacVVS(bool ransacVVS) { m_useRansacVVS = ransacVVS; }
 
   /*!
     Set the flag to filter matches where multiple query keypoints are matched
@@ -1082,7 +1159,7 @@ public:
 
     \param singleMatchFilter : True to use the single match filter.
    */
-  inline void setUseSingleMatchFilter(const bool singleMatchFilter) { m_useSingleMatchFilter = singleMatchFilter; }
+  inline void setUseSingleMatchFilter(bool singleMatchFilter) { m_useSingleMatchFilter = singleMatchFilter; }
 
 private:
   //! If true, compute covariance matrix if the user select the pose
@@ -1169,10 +1246,16 @@ private:
   //! Percentage value to determine the number of inliers for the Ransac
   //! method.
   double m_ransacConsensusPercentage;
+  //!Filtering flag for RANSAC and degenerate configuration check
+  vpPose::RANSAC_FILTER_FLAGS m_ransacFilterFlag;
   //! List of inliers.
   std::vector<vpImagePoint> m_ransacInliers;
   //! List of outliers.
   std::vector<vpImagePoint> m_ransacOutliers;
+  //! If true, use parallel RANSAC
+  bool m_ransacParallel;
+  //! Number of threads (if 0, try to determine the number of CPU threads)
+  unsigned int m_ransacParallelNbThreads;
   //! Maximum reprojection error (in pixel for the OpenCV method) to decide if
   //! a point is an inlier or not.
   double m_ransacReprojectionError;
@@ -1215,6 +1298,8 @@ private:
   //! If true, keep only pairs of keypoints where each train keypoint is
   //! matched to a single query keypoint
   bool m_useSingleMatchFilter;
+  //! Grayscale image buffer, used when passing color images
+  vpImage<unsigned char> m_I;
 
   void affineSkew(double tilt, double phi, cv::Mat &img, cv::Mat &mask, cv::Mat &Ai);
 
