@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * This file is part of the ViSP software.
- * Copyright (C) 2005 - 2017 by Inria. All rights reserved.
+ * ViSP, open source Visual Servoing Platform software.
+ * Copyright (C) 2005 - 2019 by Inria. All rights reserved.
  *
  * This software is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -222,7 +222,7 @@ unsigned int vpKeyPointSurf::buildReference(const vpImage<unsigned char> &I)
 
   cvExtractSURF(model, 0, &ref_keypoints, &ref_descriptors, storage, params);
 
-  const unsigned int nbPoints = (unsigned int)ref_keypoints->total;
+  unsigned int nbPoints = (unsigned int)ref_keypoints->total;
 
   referenceImagePointsList.resize(nbPoints);
 
@@ -262,7 +262,7 @@ unsigned int vpKeyPointSurf::buildReference(const vpImage<unsigned char> &I)
   \return the number of reference points.
 */
 unsigned int vpKeyPointSurf::buildReference(const vpImage<unsigned char> &I, const vpImagePoint &iP,
-                                            const unsigned int height, const unsigned int width)
+                                            unsigned int height, unsigned int width)
 {
   if ((iP.get_i() + height) >= I.getHeight() || (iP.get_j() + width) >= I.getWidth()) {
     vpTRACE("Bad size for the subimage");
@@ -413,7 +413,7 @@ unsigned int vpKeyPointSurf::matchPoint(const vpImage<unsigned char> &I)
   \return the number of point which have been matched.
 */
 unsigned int vpKeyPointSurf::matchPoint(const vpImage<unsigned char> &I, const vpImagePoint &iP,
-                                        const unsigned int height, const unsigned int width)
+                                        unsigned int height, unsigned int width)
 {
   if ((iP.get_i() + height) >= I.getHeight() || (iP.get_j() + width) >= I.getWidth()) {
     vpTRACE("Bad size for the subimage");
@@ -563,7 +563,7 @@ std::list<int *> *vpKeyPointSurf::matchPoint(std::list<float *> descriptorList,
  \param index : The index of the desired reference point. The index must be
  between 0 and the number of reference points - 1.
 */
-float *vpKeyPointSurf::getDescriptorReferencePoint(const int index)
+float *vpKeyPointSurf::getDescriptorReferencePoint(int index)
 {
   if (index >= static_cast<int>(referenceImagePointsList.size()) || index < 0) {
     vpTRACE("Index of the reference point out of range");
@@ -592,7 +592,7 @@ float *vpKeyPointSurf::getDescriptorReferencePoint(const int index)
  \param index : The index of the desired reference point. The index must be
  between 0 and the number of reference points - 1.
 */
-int vpKeyPointSurf::getLaplacianReferencePoint(const int index)
+int vpKeyPointSurf::getLaplacianReferencePoint(int index)
 {
   if (index >= static_cast<int>(referenceImagePointsList.size()) || index < 0) {
     vpTRACE("Index of the reference point out of range");
@@ -624,7 +624,7 @@ int vpKeyPointSurf::getLaplacianReferencePoint(const int index)
  the point used to compute the descriptor. \param dir : The orientation of the
  descriptor (in degree).
 */
-void vpKeyPointSurf::getDescriptorParamReferencePoint(const int index, int &size, float &dir)
+void vpKeyPointSurf::getDescriptorParamReferencePoint(int index, int &size, float &dir)
 {
   if (index >= static_cast<int>(referenceImagePointsList.size()) || index < 0) {
     vpTRACE("Index of the reference point out of range");
