@@ -1,14 +1,16 @@
 //! \example tutorial-matching-keypoint-SIFT.cpp
 #include <visp3/gui/vpDisplayOpenCV.h>
-#include <visp3/io/vpVideoReader.h>
 #include <visp3/io/vpImageIo.h>
+#include <visp3/io/vpVideoReader.h>
 //! [Include]
 #include <visp3/vision/vpKeyPoint.h>
 //! [Include]
 
-int main() {
-  //! [Define]
-#if (VISP_HAVE_OPENCV_VERSION >= 0x020101) && (defined(VISP_HAVE_OPENCV_NONFREE) || defined(VISP_HAVE_OPENCV_XFEATURES2D))
+int main()
+{
+//! [Define]
+#if (VISP_HAVE_OPENCV_VERSION >= 0x020101) &&                                                                          \
+    (defined(VISP_HAVE_OPENCV_NONFREE) || defined(VISP_HAVE_OPENCV_XFEATURES2D))
   //! [Define]
   vpImage<unsigned char> I;
 
@@ -19,7 +21,8 @@ int main() {
   //! [Construction]
   const std::string detectorName = "SIFT";
   const std::string extractorName = "SIFT";
-  //Use L2 distance with a matching done using FLANN (Fast Library for Approximate Nearest Neighbors)
+  // Use L2 distance with a matching done using FLANN (Fast Library for
+  // Approximate Nearest Neighbors)
   const std::string matcherName = "FlannBased";
   vpKeyPoint::vpFilterMatchingType filterType = vpKeyPoint::ratioDistanceThreshold;
   vpKeyPoint keypoint(detectorName, extractorName, matcherName, filterType);
@@ -31,18 +34,17 @@ int main() {
 
   //! [Create image]
   vpImage<unsigned char> Idisp;
-  Idisp.resize(I.getHeight(), 2*I.getWidth());
+  Idisp.resize(I.getHeight(), 2 * I.getWidth());
   Idisp.insert(I, vpImagePoint(0, 0));
   Idisp.insert(I, vpImagePoint(0, I.getWidth()));
   //! [Create image]
   //! [Init display]
-  vpDisplayOpenCV d(Idisp, 0, 0, "Matching keypoints with SIFT keypoints") ;
+  vpDisplayOpenCV d(Idisp, 0, 0, "Matching keypoints with SIFT keypoints");
   vpDisplay::display(Idisp);
   vpDisplay::flush(Idisp);
   //! [Init display]
 
-  while ( ! reader.end() )
-  {
+  while (!reader.end()) {
     //! [Acquisition]
     reader.acquire(I);
     Idisp.insert(I, vpImagePoint(0, I.getWidth()));
@@ -50,7 +52,8 @@ int main() {
 
     //! [Display]
     vpDisplay::display(Idisp);
-    vpDisplay::displayLine(Idisp, vpImagePoint(0, I.getWidth()), vpImagePoint(I.getHeight(), I.getWidth()), vpColor::white, 2);
+    vpDisplay::displayLine(Idisp, vpImagePoint(0, I.getWidth()), vpImagePoint(I.getHeight(), I.getWidth()),
+                           vpColor::white, 2);
     //! [Display]
 
     //! [Matching]
@@ -61,8 +64,7 @@ int main() {
 
     //! [Get matches]
     vpImagePoint iPref, iPcur;
-    for (unsigned int i = 0; i < nbMatch; i++)
-    {
+    for (unsigned int i = 0; i < nbMatch; i++) {
       keypoint.getMatchedPoints(i, iPref, iPcur);
       //! [Get matches]
       //! [Display matches]
