@@ -5,7 +5,7 @@
 #include <visp3/gui/vpDisplayOpenCV.h>
 #include <visp3/gui/vpDisplayX.h>
 #include <visp3/io/vpVideoReader.h>
-#include <visp3/mbt/vpMbEdgeTracker.h>
+#include <visp3/mbt/vpMbGenericTracker.h>
 #include <visp3/vision/vpKeyPoint.h>
 
 int main(int argc, char **argv)
@@ -31,7 +31,7 @@ int main(int argc, char **argv)
 
     std::cout << "Video name: " << videoname << std::endl;
     std::cout << "Tracker requested config files: " << objectname << ".[init,"
-#ifdef VISP_HAVE_XML2
+#ifdef VISP_HAVE_PUGIXML
               << "xml,"
 #endif
               << "cao or wrl]" << std::endl;
@@ -58,9 +58,9 @@ int main(int argc, char **argv)
 
     display.init(I, 100, 100, "Model-based edge tracker");
 
-    vpMbEdgeTracker tracker;
+    vpMbGenericTracker tracker(vpMbGenericTracker::EDGE_TRACKER);
     bool usexml = false;
-#ifdef VISP_HAVE_XML2
+#ifdef VISP_HAVE_PUGIXML
     if (vpIoTools::checkFilename(objectname + ".xml")) {
       tracker.loadConfigFile(objectname + ".xml");
       tracker.getCameraParameters(cam);
@@ -116,7 +116,7 @@ int main(int argc, char **argv)
     //! [Keypoint declaration]
     if (usexml) {
 //! [Keypoint xml config]
-#ifdef VISP_HAVE_XML2
+#ifdef VISP_HAVE_PUGIXML
       keypoint_learning.loadConfigFile(configurationFile);
 #endif
       //! [Keypoint xml config]
@@ -168,7 +168,7 @@ int main(int argc, char **argv)
     //! [Init keypoint detection]
     vpKeyPoint keypoint_detection;
     if (usexml) {
-#ifdef VISP_HAVE_XML2
+#ifdef VISP_HAVE_PUGIXML
       keypoint_detection.loadConfigFile(configurationFile);
 #endif
     } else {
@@ -220,13 +220,10 @@ int main(int argc, char **argv)
     }
     if (!click_done)
       vpDisplay::getClick(I);
-#ifdef VISP_HAVE_XML2
-    vpXmlParser::cleanup();
-#endif
-#if defined(VISP_HAVE_COIN3D) && (COIN_MAJOR_VERSION == 3)
+#if defined(VISP_HAVE_COIN3D) && (COIN_MAJOR_VERSION >= 2)
     SoDB::finish();
 #endif
-  } catch (vpException &e) {
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e << std::endl;
   }
 #else
