@@ -7,7 +7,7 @@ int main()
   vpImage<unsigned char> I(2160, 3840, 128);
 
   try {
-    //! [vpDisplay scale manu]
+//! [vpDisplay scale manu]
 #if defined(VISP_HAVE_X11)
     vpDisplayX d(I, vpDisplay::SCALE_5);
 #elif defined(VISP_HAVE_GDI)
@@ -16,12 +16,11 @@ int main()
     //! [vpDisplay scale manu]
     vpDisplay::setTitle(I, "My image");
     vpDisplay::display(I);
-    vpDisplay::displayCircle(I, I.getHeight()/2, I.getWidth()/2, 200, vpColor::red, true);
+    vpDisplay::displayCircle(I, I.getHeight() / 2, I.getWidth() / 2, 200, vpColor::red, true);
     vpDisplay::flush(I);
     std::cout << "A click to quit..." << std::endl;
     vpDisplay::getClick(I);
-  }
-  catch(const vpException &e) {
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e.getMessage() << std::endl;
   }
 }
