@@ -13,7 +13,7 @@ int main()
 
     //! [Load camera parameters from xml]
     vpCameraParameters cam;
-#ifdef VISP_HAVE_XML2
+#ifdef VISP_HAVE_PUGIXML
     vpXmlParserCamera p;
     vpCameraParameters::vpCameraParametersProjType projModel;
     projModel = vpCameraParameters::perspectiveProjWithDistortion;
@@ -35,7 +35,7 @@ int main()
     vpImageTools::undistort(I, cam, Iud);
     vpImageIo::write(Iud, "chessboard-undistort.pgm");
     //! [Create image without distorsion]
-  } catch (vpException &e) {
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e << std::endl;
   }
 
