@@ -36,7 +36,7 @@ int main()
 
     robot.stopRunning();
     robot.waitForRunExit();
-  } catch (vpException &e) {
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e << std::endl;
   }
 #endif
