@@ -139,7 +139,7 @@ int main()
     task.print();
     task.kill();
 
-  } catch (vpException &e) {
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e << std::endl;
   }
 }
