@@ -147,7 +147,7 @@ int main(int argc, const char *argv[])
     //! [Release IplImage]
 
     return 0;
-  } catch (vpException &e) {
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e << std::endl;
   }
 #else
