@@ -93,7 +93,7 @@ int main()
 
     vpDisplay::getClick(plotter.I);
 #endif
-  } catch (vpException &e) {
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e << std::endl;
   }
 }
