@@ -47,7 +47,7 @@ int main()
     }
 
     task.kill();
-  } catch (vpException &e) {
+  } catch (const vpException &e) {
     std::cout << "Catch an exception: " << e << std::endl;
   }
 }
